#Read a configuration file.
#This file is mandatory and must be created by hand.
proc printMediaReadConfFile {} {
    global gvar deviceType

	if {![file exists ~/.romanicConfig]} {
	puts "Configuration file is missing."
	puts "Please create a file under you home directory named .romanicConfig"
	puts "and containing entries in the format:"
	puts "      <device address><TAB><tty><TAB><writer type>"
	puts "Eg."
	puts "0,0,0\t/dev/tt1p0\tCD"
	puts "0,1,0\t/dev/tt1p0\tDVD"
	puts "1,3,0\t/dev/tt0p0\tCD"
	puts "1,5,0\t/dev/tt0p0\tDVD"
	exit 1
    }

    if {![file readable ~/.romanicConfig]} {
	puts "Configuration file ~/.romanicConfig exists but is not readable"
	exit 1
    }

    if {[catch {set conFile [open ~/.romanicConfig r]}]} { 
	puts "Cannot open ~/.romanicConfig"
	exit 1
    }

####################################################################
    
    gets $conFile line 
    if {[llength $line] != 4} {
	puts "Configuration file corrupted."
	puts "Exit."
	exit 1
    }
    
    set gvar(device)  [lindex $line 0]
    set gvar(tty)     [lindex $line 1]
    set gvar(baud)    [lindex $line 2]
    set gvar(type)    [string toupper [lindex $line 3]]
	
    if {[checkBaud $gvar(baud)] == "error"} {
	puts "baud rate of $gvar(baud) is invalid."
	puts "Check ~/.romanicConfig"
	puts "Exit."
	exit 1
    }
        
}

#Reset a robot connected to a specific port.
proc romanicResetRobot {} {
    global gvar

    set tty $gvar(tty)
    set cd  $gvar(device)
    set gvar(ttyTail) [file tail $tty]
    
    puts "Close printer - $tty"
    closePrinter
    
   	puts "Reset writer - $cd"
	#resetCD $cd
	puts "Load writer - $cd"
	loadCD $gvar(device)
    
    armOnTop
    if {[checkArm $tty] == 1} {
	puts "Get rid of old medium - $tty"
	putBad}
    puts "Check if there is a medium in the printer - $tty"
    armOnTop
    openPrinter
    getPrinter
    closePrinter
    if {[checkArm $tty] == 1} {putBad}
    armOnTop
    
    puts "Check if there are media in the writers - $tty"
    armOnTop
    ejectCD $gvar(device)
    getMedia
    loadCD $gvar(device)
    if {[checkArm $tty] == 1} {putBad}
    armOnTop
}

#Calculate the number of CDs in all magazines.
proc romanicCalculateNumCDsAll {} {
    global gvar 

    puts "Calculate number of media in the magazine."
    Log "Calculate number of CDs in the magazine."
    foreach tty $gvar(ttyList) {
	romanicCalculateNumCDs $tty
    }
}


proc romanicCalculateNumCDs {tty} {
    global gvar
    
    set gvar(numBlankCDs)   [check $tty blank]
    set gvar(numWrittenCDs) [check $tty written]
    set gvar(numBadCDs)     [check $tty bad]
    set ttyTail             [file tail $tty]
    
    Debug "Media in Blank magazine:   $gvar(numBlankCDs)" 1
    Debug "Media in Written magazine: $gvar(numWrittenCDs)" 1
    Debug "Media in Bad magazine:     $gvar(numBadCDs)" 1
    
    if {$gvar(numBlankCDs) > 60} {
	puts "The magazine n. 1 contains $gvar(numBlankCDs)"
	puts "Maximum allowed is 60 media"
	puts "In robot $ttyTail"
	exit 1
    }

    if {$gvar(numBlankCDs) == 0} {
	puts "Magazine number 1 is empty."
	puts "In robot $ttyTail"
	exit 1
    }
	
    if {[expr $gvar(numBlankCDs) + $gvar(numWrittenCDs) +$gvar(numBadCDs)]  > 60} {
	puts "There are too many media in the carousel."
	puts "In robot $ttyTail"
	exit 1
    }
	
    	if {$gvar(numWrittenCDs) > 60} {
	    puts "The magazine n. 2 contains $gvar(numWrittenCDs)"
	    puts "Maximum allowed is 60 media"
	    puts "In robot $ttyTail"
	    exit 1
	}
    
	if {[expr $gvar(numBlankCDs) + $gvar(numBadCDs)]  > 60} {
	    puts "There are too many media in the carousel."
	    puts "Please empty the magazine 3."
	    puts "In robot $ttyTail"
	    exit 1
	}	
    
    if {[expr $gvar(numBlankCDs) + $gvar(numWrittenCDs)]  > 60} {
	    puts "There are too many media in the carousel."
	    puts "Please empty the magazine 2."
	    puts "In robot $ttyTail"
	    exit 1
	}
}

#Return the number of CDs in a particular magazine in
#of a particular robot.
proc check {tty magazine} {

    pickUpCD $tty $magazine
    dropCD   $tty $magazine
    set returnValue [readElement    $tty $magazine]
    set value       [string range   $returnValue 6 10]
    set numCD       [calculateValue $value]
    
    return $numCD    
}

#Check if there is a CD in the arm.
proc checkArm {tty} {
    
    set returnValue [readElement $tty arm]
    set value       [string range $returnValue 5 5]
    return $value
    

    #Only for debug
    switch $value {
	"0" {puts "The arm is empty"}
	"1" {puts "The arm is busy" }
	"2" {puts "No arm"}
	"3" {puts "Arm status undefined"}
    }

}

#Count the CDs of all the magazines.
proc checkAll {tty} {

    foreach magazine {blank written bad} {
	check $tty $magazine	
    }
}

#Pick up a CD from a specified magazine and don't complain
#if there isn't any. It's an operation used to count the CDs.
proc pickUpCD {tty magazine} {
    global gvar
    
    Log "Pick up CD"
    switch $magazine {
	"blank"    {positionMedium $gvar(blankAddress)    $gvar(takeMedia)}
	"written"  {positionMedium  $gvar(writtenAddress) $gvar(takeMedia)}
	"bad"      {positionMedium $gvar(badAddress)      $gvar(takeMedia)}
    }
}

proc dropCD {tty magazine} {
    global gvar 

    Log "Drop CD"
    switch $magazine {
	"blank"    {positionMedium $gvar(blankAddress)   $gvar(putMedia)}
	"written"  {positionMedium $gvar(writtenAddress) $gvar(putMedia)}
	"bad"      {positionMedium $gvar(badAddress)     $gvar(putMedia)}
    }
}

#Read the encoder steps of the arm. Used to count the CDs.
proc readElement {tty magazine} {
    global gvar
    
    switch $magazine {
	"blank"    {set address "\x32\x30\x30"}
	"written"  {set address "\x32\x30\x31"}
	"bad"      {set address "\x32\x30\x32"}
	"arm"      {set address "\x32\x32\x30"}
    }

    sendMessage "\x1B\x21\x66\x30$address\x43"
    set returnValue [readPort 12]
    return $returnValue
}

#Return the number of CDs starting from and encoder value.
proc calculateValue {value} {

    set numDisc [expr {(36150 - $value) / 120}]
    if {$numDisc < 0 } { set numDisc 0 }
    return $numDisc
}

#Check if there are request files. If there are already 2 requests
#queue, return: each robot has 2 writing devices only.
#I know, I can print while the writers are busy, but let's not
#complicate to much the machinery...
proc globLoop {} {
    global gvar 
    
    #puts "I'm process [pid] I enter in glob loop"

    set tty [file tail $gvar(ttyList)]
    set requestsPending [glob -nocomplain $gvar(reqDir)/inProcess-*-$tty.romanic]
    if {[llength $requestsPending] >= 2} {
	#puts "I'm process [pid] there are already more than 2 requests: $requestsPending"
	#puts "Already 2 requests pending"
	return
    }
    
    #set requests [lrange [glob -nocomplain $gvar(reqDir)/request-*.romanic] 0 1]
    set requests [glob -nocomplain $gvar(reqDir)/request-*.romanic]
    if {$requests == ""} { 
	#puts "I'm process [pid], I don't see requests under $gvar(reqDir)/"
	#No request files. Goodbye.
	return 
    } 
    #puts "I'm process [pid], I see these requests: $requests"

    #Check if a request is already in process. If not, queue it.
    foreach request $requests {
	set id [string range [file tail [file root $request]] 8 end]
	if {[file exists $gvar(reqDir)/inProcess-$id-$tty.romanic]} { 
	    Debug "id: $id is already in queue" 3
	    continue
	} else {
	    set file [open $request r]
	    gets $file requestLine
	    close $file
	    if {[checkDevice $requestLine]} { 
		#puts "I'm process [pid] and request $request is not my business"
		#puts "Because I ([pid])  take care of $tty"
		continue
	    } else {
		#puts "I'm process [pid] and request $request IS my business"
		lappend gvar(requests) "-id $id $requestLine"
	    }
	}
    }
    return 
}

#Serve all the requests in the queue.
proc serveRequestsLoop {} {
    global gvar

    if {$gvar(requests) == ""} { 
	#No requests pending. Goodbye.
	return 
    }
    
    set tty [file tail $gvar(ttyList)]
    #Process the request if not doing it already.
    foreach request $gvar(requests) {
	set id [setId $request]
	if {[file exists $gvar(reqDir)/inProcess-$id-$tty.romanic]} { 
	    Debug "Already processing request $id" 3
	    continue 
	} else {
	    write_file $gvar(reqDir)/inProcess-$id-$tty.romanic ""
	    set index [lsearch  $gvar(requests) "*-id $id*"]
	    set gvar(requests) [lreplace $gvar(requests) $index $index]
	    switch -- $gvar(robotMode) {
		"3-magazine" {forkProcess "threeMagazineMode \"$request\""}
		"efficient"  {forkProcess "efficientMode \"$request\""}
		"mixed"      {forkProcess "mixedMode \"$request\""}
		default { return } 
	    } 	
	}
    }
}


#Extract the id from the request.
proc setId {request} {

    set idIndex [lsearch -exact $request "-id"]
    if {$idIndex == -1} { 
	return "error"
    } else {
	set id [lindex $request [incr idIndex]]
	return $id
    }
}

#Check if this request belongs to the current tty-process.
proc checkDevice {requestLine} {
    global gvar

    if {[lindex $requestLine 0] == "printOnly"} { return 1 }
    
    set device [lindex $requestLine [expr [lsearch -exact $requestLine "-writerAddress"] +1]]
    #puts "I'm [pid] and device for request: $requestLine is: $device"

    #This request is not for this process...
    if {![info exists gvar($device)]} { 
	 #puts "I'm [pid] and gvar($device) doesn't exists"
	return 1 }

    if {[lindex $gvar($device) 0] == $gvar(ttyList)} { 
	return 0
    } else {
	#puts "I'm [pid] and gvar($device) contains $gvar($device) which is not equal to $gvar(ttyList)"
	return 1
    }
}


#Check if the request is valid.
proc checkRequest {request} {
    global gvar

    set id [setId $request]
    
    if {$id == "error"} { 
	writeError "Error: id $id  not valid"
	return 1
    } else {
	set gvar(id) $id
    }


    #Parse all the fields of the request:
    #-action= action to perfom. It can be: 
    #         writePrint: write a cd and print it.
    #         writeOnly:  just write a CD.
    #         printOnly:  just print a CD.
    #-volumeFile= path and name of the volume file to write.
    #-volumeName= name of the volume to use to write a CD.
    #-writeAddress= address of the writer in cdrecord format (Eg. 0,1,0)
    #-writerSpeed= speed of the writer
    #-dummymode= don't really write a CD.
    #-image= path and name of the image to print.

    for { set i 0} {$i <[llength $request]} {incr i} {
     
        switch -- [lindex $request $i] {
	    -id {incr i}
	    -action {
		incr i
		set gvar($id-action) [lindex $request $i]
	    }
	    -volumeFile {
		incr i
		set gvar($id-volumeFile) [lindex $request $i]
	    } 
	    -volumeName {
		incr i
		set gvar($id-volumeName) [lindex $request $i]
	    } 
	    -copies {
		incr i
		set gvar($id-copies) [lindex $request $i]
	    } 
	    -writerAddress {
		incr i
		set gvar($id-writerAddress) [lindex $request $i]
	    } 
	    -writerSpeed {
		incr i
		set gvar($id-writerSpeed) [lindex $request $i]
	    } 
	    -dummyMode {
		 set gvar($id-dummyMode) 1
	    }
	    -image {
		incr i
		set gvar($id-imageToPrint) [lindex $request $i]
	    }
	    default {
		Debug "Unknown option: [lindex $request $i]" 1
		return 1
	    }
	}
    }
    
    switch $gvar($id-action) {
	"writePrint" {set gvar($id-print) 1 ; set gvar($id-write) 1}
	"writeOnly"  {set gvar($id-print) 0 ; set gvar($id-write) 1}
	"printOnly"  {set gvar($id-print) 1 ; set gvar($id-write) 0}
	default      {Debug "Unknown option $gvar($id-action)" 1 ; return 1 }
    }
    
    if {$gvar($id-action) == "printOnly"} {
	if {![info exists gvar($id-volumeFile)] || $gvar($id-volumeFile) == ""} {
	    writeError "volume file not specified."
	    #puts "Volume file from request: $gvar($id-volumeFile)"
	    return 1
	}
	
	#if {![file exists $gvar($id-volumeFile)]} {
	#    writeError "volume file: $gvar($id-volumeFile) doesn't exist."
	#    return 1
	#}
	
	if {![info exists gvar($id-writerAddress)] || $gvar($id-writerAddress) == ""} {
	    writeError "writer address not specified."
	    return 1
	}
	
	if {![info exists gvar($id-writerSpeed)] || $gvar($id-writerSpeed) == ""} {
	    set gvar($id-writerSpeed) 2
	}
	
	if {![info exists gvar($id-volumeName)] || $gvar($id-volumeName) == ""} {
	    set gvar($id-volumeName) [file root [file tail $gvar($id-volumeFile)]]
	}
    }

    if {$gvar($id-action) == "printOnly"} {
	if {![info exists gvar($id-imageToPrint)] || $gvar($id-imageToPrint) == ""} {
	    writeError "Image file not specified"
	    return 1
	}
	if {![file exists $gvar($id-imageToPrint)]} {
	    writeError "image file: $gvar($id-imageToPrint) doesn't exist."
	    return 1
	}	
    }
    
    if {![info exists gvar($id-copies)] || $gvar($id-copies) == ""} {
	set gvar($id-copies) 1
    }
    
    if {![info exists gvar($id-dummyMode)]} {
	set gvar($id-dummyMode) 0
    }

    return 0
}

#Remove an entry from the queue list.
proc cleanUpQueue {id} {
    global gvar 

    Log "Cleanup queues"
    set tty [file tail $gvar(ttyList)]

    set index [lsearch $gvar(requests) "*-id $id*"]
    if {$index != -1} {
	set gvar(requests) [lreplace $gvar(requests) $index $index]
    }
    file delete $gvar(reqDir)/request-$id.romanic
    file delete $gvar(reqDir)/inProcess-$id-$tty.romanic
    file delete $gvar(reqDir)/count-$id.romanic
    unsetId $id
}

#Remove temporary files.
proc cleanUpAll {} {
    global gvar

    set inProcessFiles [glob -nocomplain $gvar(reqDir)/inProcess-*]
    set lockFiles      [glob -nocomplain $gvar(reqDir)/*.lock]
    set errorFiles     [glob -nocomplain $gvar(reqDir)/error-*.romanic]
    set fixFiles       [glob -nocomplain $gvar(reqDir)/fixed-*.romanic]
    set countFiles     [glob -nocomplain $gvar(reqDir)/count-*.romanic]
    set fatalErrors    [glob -nocomplain $gvar(reqDir)/fatalError-*.romanic]
    set fanFiles       [glob -nocomplain $gvar(reqDir)/fan-*.romanic]
    set currentMag     [glob -nocomplain $gvar(reqDir)/currentMagazine-*.romanic]
    set cdLabels       [glob -nocomplain $gvar(reqDir)/cdLabel-*]
    foreach list "$inProcessFiles $lockFiles $errorFiles $fixFiles $countFiles $fatalErrors $fanFiles $currentMag cdLabels" {
	foreach file $list {
	    Log "Delete file: $file"
	    file delete $file
	}
    }
}

#Unset some variables.
proc unsetId {id} {
    global gvar

    unset gvar(id)
    unset gvar($id-action)
    catch {unset gvar(tty)}
    catch {unset gvar(ttyTail)}
    catch {unset gvar($id-volumeFile)}
    catch {unset gvar($id-volumName) }
    catch {unset gvar($id-copies) }
    catch {unset gvar($id-writerAddress)}
    catch {unset gvar($id-writerSpeed) }
    catch {unset gvar($id-dummyMode) }
    catch {unset gvar($id-imageToPrint)}
    catch {unset gvar($id-write) }
    catch {unset gvar($id-print) }
}

proc checkRomanic {} {
   
    set romanicProcesses ""
    catch {set romanicProcesses [exec ps -fed | grep tclsh | grep romanic]}
    if {$romanicProcesses != ""} {
        puts "Romanic is running !"
        puts "You cannot use printMedia when Romanic is running"
        puts "Stop romanic first and then try again with printMedia"
        exit 1
    }
    return 0
}




##########################################


