#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# COPYRIGHT (c) 1996 European Southern Observatory
# LICENSE
#
# PROJECT:   VLT Data Flow System
# AUTHOR:    Carlos Guirao - ESO/DMD/SEG
# CI NUMBER: 
#
# $Revision: 1.1 $
#------------------------------------------------------------------------------

# setup DATAFLOW_DIR & SNIFF+
if  [ -z "$DATAFLOW_DIR" ];  then
  export DATAFLOW_DIR=$HOME
fi 

export DFS_EXTERNAL=external-3_6

# setup SNIFF_DIR to /opt/dpgsoft/sniff in development
# machines, otherwise to ${DATAFLOW_DIR}/${DFS_EXTERNAL}/sniff
if [ -z "$SNIFF_DIR" ];  then
  if [ -d /opt/dpgsoft/sniff ];  then
	export SNIFF_DIR=/opt/dpgsoft/sniff
  else
	export SNIFF_DIR=${DATAFLOW_DIR}/${DFS_EXTERNAL}/sniff
  fi
fi

PATH=${SNIFF_DIR}/bin:${PATH}
export PLATFORM=`sniff_arch`

if [ -z "$VLTROOT" ]; then
  export DFS_VLTROOT=/vlt/APR2004/NOCCS
else
  export DFS_VLTROOT=$VLTROOT
fi

if [ -z "$VLTSW_ROOT" ]; then
  export VLTSW_ROOT=`dirname $DFS_VLTROOT`
fi

# MIDAS
export DFS_MIDAS_PATH=${DATAFLOW_DIR}/${DFS_EXTERNAL}/midas/03SEP
export DFS_MIDAS_INCL=${DFS_MIDAS_PATH}/incl
export DFS_MIDAS_LIB=${DFS_MIDAS_PATH}/lib/${PLATFORM}

# VLT OSLX 
if [ -z "$OSLX" ]; then
  export DFS_OSLX=${DFS_VLTROOT}
  export DFS_OSLX_INCL=${DFS_OSLX}/include
  export DFS_OSLX_LIB=${DFS_OSLX}/lib
else
  export DFS_OSLX_INCL=${OSLX}/include
  export DFS_OSLX_LIB=${OSLX}/lib/${PLATFORM}
fi

# MSQL2
if [ -z "$DFS_MSQL" ]; then
  export DFS_MSQL=/opt/msql2
fi
export DFS_MSQL_INCL=${DFS_MSQL}/include
export DFS_MSQL_LIB=${DFS_MSQL}/lib

# Real Time Display
export DFS_RTD=${DFS_VLTROOT}
export DFS_RTD_INCL=${DFS_RTD}/include/cat
export DFS_RTD_LIB=${DFS_RTD}/lib

# RogueWave
export DFS_RWTOOL=${DATAFLOW_DIR}/${DFS_EXTERNAL}/rogueWave
export DFS_RWTOOL_INCL=${DFS_RWTOOL}/include
export DFS_RWTOOL_LIB=${DFS_RWTOOL}/lib/${PLATFORM}

# ZLIB package
export DFS_ZLIB=${DATAFLOW_DIR}/${DFS_EXTERNAL}/zlib
export DFS_ZLIB_INCL=${DFS_ZLIB}/include
export DFS_ZLIB_LIB=${DFS_ZLIB}/lib/${PLATFORM}

# SYBASE definitions 
if [ -z "$SYBASE" ]; then
  export DFS_SYBASE=/opt/sybase
else
  export DFS_SYBASE=${SYBASE}
fi
export DFS_SYBASE_INCL=${DFS_SYBASE}/include
export DFS_SYBASE_LIB=${DFS_SYBASE}/lib
#
# On Linux RH9.0 there is no valid static libsybdb.a, only
# the shared on works. For other systems use the static one.
#
if [ "$PLATFORM" = "i586-linux-glibc" ]; then
  export DFS_SYBASE_LIB_VCSOLAC="-L${DFS_SYBASE_LIB} -lsybdb"
else
  export DFS_SYBASE_LIB_VCSOLAC="${DFS_SYBASE_LIB}/libsybdb.a"
fi

# CFITSIO definitions
export DFS_CFITSIO=${DATAFLOW_DIR}/${DFS_EXTERNAL}/cfitsio
export DFS_CFITSIO_INCL=${DFS_CFITSIO}/include
export DFS_CFITSIO_LIB=${DFS_CFITSIO}/lib/${PLATFORM}

# QFITS definitions
export DFS_QFITS=${DATAFLOW_DIR}/${DFS_EXTERNAL}/qfits
export DFS_QFITS_INCL=${DFS_QFITS}/include
export DFS_QFITS_LIB=${DFS_QFITS}/lib/${PLATFORM}

#
# Next definitions are meant only for FV (Fits Verify package)
#
export DFS_CFITSIO_NEW=${VLTROOT}
export DFS_CFITSIO_NEW_INCL=${DFS_CFITSIO_NEW}/include
export DFS_CFITSIO_NEW_LIB=${DFS_CFITSIO_NEW}/lib

# SOCKET library needed on Solaris
if [ "$PLATFORM" = "sparc-sun-solaris2.3" ]; then
   export DFS_SOCKET_LIBS="-lnsl -lsocket"
   export DFS_DL_LIB="-ldl"
fi

# NSL (XDR modules) library needed on HP-UX 11.0
if   [ "$PLATFORM" = "pa_risc-hp-hpux10" ] &&   [ `uname -r` = "B.11.00" ];  then
   export DFS_SOCKET_LIBS="-lnsl"
fi

# VLT documentation script
export DFS_VLT_DOC=${DFS_VLTROOT}/bin/docDoManPages

# X11 include and library needed on Solaris
if [ "$PLATFORM" = "sparc-sun-solaris2.3" ]; then
   export DFS_X11_INCL="-I/usr/openwin/include"
   export DFS_X11_LIBS="-lX11"
fi  

case "$PLATFORM" in
  "sparc-sun-solaris2.3")
     export DFS_ASTO_TARGET="ultra_sol_opt"
     export DFS_ASTO_INSTALL_DIR="solaris"
     ;;
  "pa_risc-hp-hpux10")
     export DFS_ASTO_TARGET="hppa_hpux_opt"
     export DFS_ASTO_INSTALL_DIR="hpux"
     ;;
  "i586-linux-glibc")
     export DFS_ASTO_TARGET="x86_linux"
     export DFS_ASTO_INSTALL_DIR="linux-debug"
     ;;
esac     
