/**
 * @author Michele Zamparelli
 * @version 1.0
 * A JTree widget for displaying a ResultsTreeModel object.
 * 
 * TODO: DFSTreeTable, ResultsTreeTable and DBTreeTable all have 
 * a CustomListener - these should be extracted out to a common type.
 * 
 * TODO: refactoring: remove dependecy on GasganoDisplay.
 * 
 */

package org.eso.gasgano;

import java.awt.Component;
import java.awt.Container;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.MouseEvent;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.JFrame;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JTree;
import javax.swing.SwingUtilities;
import javax.swing.event.MouseInputAdapter;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

import org.eso.gasgano.datamodel.gui.FitsFileOBComponent;
import org.eso.gasgano.datamodel.gui.ResultsComponent;
import org.eso.gasgano.gui.DFSFileTreeCellRenderer;
import org.eso.gasgano.gui.JTreeTable;
import org.eso.gasgano.gui.ResultsTreeTableModel;
import org.eso.gasgano.properties.PropertyDB;
import org.eso.gasgano.tools.Exec;

public class ResultsTreeTable extends JTreeTable implements ActionListener {

    private Vector selectionVector = new Vector();
    protected String selectedDir = null;
    private JPopupMenu filePopUpMenu = null;
    private JPopupMenu containerPopUpMenu = null;
    private Object popUpDBComponent = null;
    private GasganoDisplay mainDisplay = null;
    // some labels
    private String retrieveLabel = "Retrieve file(s)";

    public ResultsTreeTable(
        ResultsTreeTableModel model,
        GasganoDisplay mainDisplay) {
        super(model);
        getTree().getSelectionModel().setSelectionMode(
            TreeSelectionModel.DISCONTIGUOUS_TREE_SELECTION);
        getTree().setCellRenderer(new DFSFileTreeCellRenderer());
        model.setTree(getTree());
        setIntercellSpacing(new Dimension(5, 1));
        setColumnWidths();
        setupPopUpMenu();
        this.mainDisplay = mainDisplay;
    }

    public void setColumnWidths() {
        TableColumnModel tcm = getColumnModel();
        TableColumn tCol = tcm.getColumn(0);
        tCol.setMinWidth(370);
    }

    /**
     * @return Return a Vector of String of paths names of the 
     * selected files - or an empty Vector is nothing is selected.
     */
    public Vector getSelectedFiles() {
        Vector rc = selectionVector;
        rc.removeAllElements();
        TreePath paths[] = getTree().getSelectionPaths();

        if (paths != null) {
            for (int i = 0; i < paths.length; i++) {
                Object obj = paths[i].getLastPathComponent();
                if (obj instanceof ResultsComponent) {
                    ResultsComponent dbobc = (ResultsComponent) obj;
                    rc.addElement(dbobc.getId());
                }
            }
        }
        return rc;
    }

    private void setupPopUpMenu() {
        filePopUpMenu = new JPopupMenu();
        containerPopUpMenu = new JPopupMenu();

        JMenuItem item = null;
        filePopUpMenu.add(
            item =
                new JMenuItem(
                    PropertyDB.getInstance().getProperty("MENU_CUSTOM1_LBL")));
        item.addActionListener(this);
        filePopUpMenu.add(
            item =
                new JMenuItem(
                    PropertyDB.getInstance().getProperty("MENU_CUSTOM2_LBL")));
        item.addActionListener(this);
        filePopUpMenu.addSeparator();
        filePopUpMenu.add(item = new JMenuItem(retrieveLabel));
        item.addActionListener(this);

        containerPopUpMenu.add(
            item =
                new JMenuItem(
                    PropertyDB.getInstance().getProperty("MENU_CUSTOM3_LBL")));
        item.addActionListener(this);
        containerPopUpMenu.add(
            item =
                new JMenuItem(
                    PropertyDB.getInstance().getProperty("MENU_CUSTOM4_LBL")));
        item.addActionListener(this);
    }

    public Vector getSelectedOBComponents() {
        Vector rc = selectionVector;
        rc.removeAllElements();
        TreePath paths[] = getTree().getSelectionPaths();

        if (paths != null) {
            for (int i = 0; i < paths.length; i++) {
                Object obj = paths[i].getLastPathComponent();
                if (obj instanceof ResultsComponent) {
                    rc.addElement(obj);
                }
            }
        }
        return rc;
    }

    public String getSelectedInstrument() {
        Vector selVect = getSelectedOBComponents();
        String rc = null;
        int c = 0;
        for (int i = 0; i < selVect.size(); i++) {
            if (selVect.elementAt(i) instanceof FitsFileOBComponent) {
                FitsFileOBComponent ffoc =
                    (FitsFileOBComponent) selVect.elementAt(i);
                if (c == 0) {
                    rc = ffoc.getInstrument();
                    c++;
                } else {
                    c++;
                    if (ffoc.getInstrument() != null
                        && !rc.equals(ffoc.getInstrument())) {
                        rc = null;
                        return rc;
                    }
                }
            }
        }
        return rc;
    }
    
    // actionListener interface for the PopUp Menu

    public void actionPerformed(ActionEvent e) {
        StringBuffer commandLong = null;
        // System.out.println( e.getActionCommand() + " " + popUpDBComponent );
        if (e.getActionCommand().toString().equals(retrieveLabel)) {
            mainDisplay.retrieveFilesFromDb(getSelectedFiles().elements());
            return;
        }
        for (int i = 0; i < 4; i++) {
            String tmpString = "MENU_CUSTOM" + String.valueOf(i + 1) + "_LBL";
            if (e
                .getActionCommand()
                .toString()
                .equals(PropertyDB.getInstance().getProperty(tmpString))) {
                String command =
                    PropertyDB.getInstance().getProperty(
                        "MENU_CUSTOM" + String.valueOf(i + 1));
                if (i > 1) {
                    System.out.println(command + " " + popUpDBComponent);
                    Exec.runCommand(command + " " + popUpDBComponent);
                } else {
                    commandLong = new StringBuffer(command + " ");
                    for (Enumeration ee = getSelectedFiles().elements();
                        ee.hasMoreElements();
                        ) {
                        commandLong.append(ee.nextElement().toString() + " ");
                    }
                    System.out.println(commandLong);
                    Exec.runCommand(commandLong.toString());
                }
                return;
            }
        }
    }
}
// end of file ResultsTreeTable.java
