/**
 * @author M. Zamparelli
 * @version 1.0
 * A class for defining a translation table
 * between the table data_products and a standard
 * ESO keyword set
 */

package org.eso.gasgano.datamodel.database;

import java.util.Vector;

import org.eso.gasgano.keyword.Keyword;
import org.eso.gasgano.tools.SortedHashtable;

public class DBTranslation extends SortedHashtable {
    private String blank = new String("BLANK");
    private Vector ordered = new Vector(28);
    private static DBTranslation theTranslation = null;

    public DBTranslation() {
        super();
        setDBTranslation();

        // this should implement a 
        // HEADER-Keyword  to Column association
        put(Keyword.ARCFILE, "dp_id");
        put(Keyword.OBS_ID, "ob_id");
        put(Keyword.TELESCOP, "tel_id");
        put(Keyword.INSTRUMENT, "ins_id");
        put("DPR.CATG", "dp_cat");
        put(Keyword.PROG_ID, "prog_id");
        put(Keyword.ORIGFILE, "origfile");

        put("EXPTIME", "exptime");
        put("SEQNO", "tpl_seqno");
        put("RA", "ra");
        put("DEC", "dec");

        /*
        put(blank,"dp_class");
        put(blank,"light_path");
        put(blank,"exp_start");
        put(blank,"tpl_expno");
        put(blank,"det_expid");
        put(blank,"qual_flag");
        */
        put("DPR.TYPE", "dp_type");
        put("DPR.TECH", "dp_tech");
        put("MJD-OBS", "mjd_obs");
        put("DET.ID", "det_id");
        put("DET.NDIT", "det_ndit");
        put("DET.EXP.ID", "det_expid");
        put("OBJECT", "object");
        put("INS.MODE", "ins_mode");
        put("OBS.NAME", "obs_name");
        put("NAXIS1", "naxis1");
        put("NAXIS2", "naxis2");
        put("TEL.AIRM.START", "tel_airm_start");
        put("DET.CHIP1.ID", "det_chip1id");
    }

    public Object put(Object key, Object value) {
        ordered.addElement(key);
        return super.put(key, value);
    }

    public String getTranslation(String key) {
        return (String) get(key);
    }

    public static DBTranslation getSelf() {
        return theTranslation;
    }

    private void setDBTranslation() {
        theTranslation = this;
    }

    public String getKeyAt(int z) {
        return (String) ordered.elementAt(z);
    }

    public int size() {
        return ordered.size();
    }
}
