package org.eso.gasgano.gui;

import java.awt.BorderLayout;

import javax.swing.JFileChooser;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.eso.dfs.gui.SwingFileChooser;
import org.eso.gasgano.properties.GasProp;


/**
 * This base class must be implemented by all property editor panels in the
 * GasPropEditor. It is a means of streamlining the design of the various
 * tab panes used in the GasPropEditor panel.
 */
public abstract class PropertyEditorSubPanel extends JPanel {
    
    private boolean debug;
    
    /**
     * No action is required when the properties are saved.
     */
    public static final int NOACTION_ON_SAVE = 0;
    
    /**
     * Data reload is required when the properties are saved.
     */
    public static final int RELOAD_ON_SAVE = 1;
    
    /**
     * Application Restart is required when the properties are saved.
     */
    public static final int RESTART_ON_SAVE = 2;
    
    /**
     * Load the fields in the panel from the given GasProp object.
     * 
     */
    public abstract void loadProperties(GasProp gp);
    
    /**
     * Save the GUI setting to the GasProp provided in
     * loadProperties().
     * @return null if all ok, else on error (invalid config) 
     * return an error string.
     */
    public abstract String saveProperties();
    
    /**
     * return true if a restart of the application
     * is required due to the change in properties.
     * @return NOACTION_ON_SAVE: no action required
     *         RELOAD_ON_SAVE  : data reload required
     *         RESTART_ON_SAVE : restart required.
     */
    public abstract int restartRequired();
    
    /**
     * turn on/off debug mode (for output to stdout)
     *
     */
    public void setDebug(boolean b) {
        debug = b;
    }
    
    public boolean isDebug() {
        return debug;
    }
    
    /**
     * Utility method to take a JTextField and place it in a panel with a FileChooserJButton
     * and return the resulting panel.
     */
    public static JPanel setupFileChooserPanel(
        JTextField fld,
        int fileSelectionMode) {
        JPanel rc;
        FileChooserJButton fcButton;
        JFileChooser fc;

        fc = new SwingFileChooser();
        fc.setFileSelectionMode(fileSelectionMode);
        fcButton = new FileChooserJButton(fld, fc);
        rc = new JPanel();
        rc.setLayout(new BorderLayout());
        rc.add(fld, BorderLayout.CENTER);
        rc.add(fcButton, BorderLayout.EAST);
        return rc;
    }
}