/**
 * @author M. Zamparelli
 * @version 1.0
 * This Model mapps a DataModel into an interface usable by JTree.
 * The root of the tree is a String, with Program children.
 * String->Program->Observation->OBComponent.
 * The class of a node is used often to determine the level in the tree.
 */

package org.eso.gasgano.gui;

import java.awt.Cursor;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.JTree;
import javax.swing.tree.TreePath;

import org.eso.gasgano.datamodel.DataModelEvent;
import org.eso.gasgano.datamodel.ResultsModel;
import org.eso.gasgano.datamodel.gui.ResultsComponent;
import org.eso.gasgano.datamodel.gui.ResultsContainer;
import org.eso.gasgano.properties.GasProp;

public class ResultsTreeTableModel extends AbstractTreeTableModel {

    private Vector treeListeners = null;
    private ResultsModel dm = null;
    private RootNode root = new RootNode();
    private Vector displayCols = null;
    private int groupDataByLevel = 0;
    protected Vector allTheLeaves = new Vector();

    private JTree theTree = null;

    public ResultsTreeTableModel(ResultsModel d) {
        init(d);
    }

    private void init(ResultsModel d) {
        dm = d;
        treeListeners = new Vector();
        setRootLabel(null);
        displayCols = GasProp.getProperties().fsDisplayColumns;
    }

    // I so much would like not to need this..
    public void setTree(JTree tree) {
        theTree = tree;
    }

    public int getGroupDataByLevel() {
        return groupDataByLevel;
    }

    public void setGroupDataByLevel(int gbLevel) {
        if (groupDataByLevel != gbLevel) {
            groupDataByLevel = gbLevel;
            setRootLabel(null);
            updateTree();
        }
    }

    public void setRootLabel(String name) {
        name =
            new String("Displaying " + dm.getReferenceCount() + " references ");
        root.setLabel(name);
    }

    public String getRootLabel() {
        String rc = null;
        if (root != null) {
            root.getLabel();
        }
        return rc;
    }

    public Object getChild(Object parent, int index) {
        Object rc = null;
        if (parent == root) {
            if (groupDataByLevel == GasProp.TREE_DEFAULT) {
                rc = dm.getReferenceAt(index);
            }
        } else if (parent instanceof ResultsContainer) {
            ResultsContainer tmpContainer = (ResultsContainer) parent;
            int fc = tmpContainer.getFramesCount();
            if (index >= fc) {
                rc = tmpContainer.getLabelAt(index - fc);
            } else {
                rc = tmpContainer.getFrameAt(index);
            }
        } else {
            rc = null;
        }
        return rc;
    }

    public int getIndexOfChild(Object parent, Object child) {
        int rc = -1;
        if (parent == root) {
            if (groupDataByLevel == GasProp.TREE_DEFAULT) {
                rc = dm.getIndexOfReference((ResultsContainer) child);
            }
        } else if (parent instanceof ResultsContainer) {
            ResultsContainer cont = (ResultsContainer) parent;

            if (child instanceof ResultsComponent) {
                rc = cont.getIndexOfFrame((ResultsComponent) child);
            }
            if (child instanceof ResultsContainer) {
                rc =
                    cont.getIndexOfLabel((ResultsContainer) child)
                        + cont.getLabelsCount();
            }
        } else {
            System.out.println("Child " + child + " was not located");
            rc = -1;
        }
        return rc;
    }

    public int getChildCount(Object parent) {
        int rc = 0;
        if (parent == root) {
            if (groupDataByLevel == GasProp.TREE_DEFAULT) {
                rc = dm.getReferenceCount();
            }
        } else if (parent instanceof ResultsContainer) {
            rc =
                ((ResultsContainer) parent).getLabelsCount()
                    + ((ResultsContainer) parent).getFramesCount();
        } else {
            rc = 0;
        }
        return rc;
    }

    public int getColumnCount() {
        // the file entry plus the entries in DISPLAY_COLUMNS.
        return 1+displayCols.size();
    }

    public String getColumnName(int column) {
        String rc = null;
        if (column == 0) {
            rc = "File";
        } else {
            rc = (String) displayCols.elementAt(column - 1);
        }
        return rc;
    }

    public Object getValueAt(Object node, int column) {
        Object rc = null;
        if (column > 0) {
            if (node instanceof ResultsComponent) {
                String colData[] = null;
                colData = ((ResultsComponent) node).getDisplayData();
                if (colData != null && colData.length >= column) {
                    rc = colData[column - 1];
                }
            }
        } else {
            if (node instanceof ResultsContainer) {

            }
            rc = node;
        }
        return rc;
    }

    // presumably not needed
    public void toggleColumn() {
    }

    public Class getColumnClass(int column) {
        Class rc = null;
        if (column == 0) {
            rc = TreeTableModel.class;
        } else {
            rc = ResultsComponent.class;
        }
        return rc;
    }

    public Object getRoot() {
        if (root == null) {
            setRootLabel(null);
        }
        return root;
    }

    public boolean isLeaf(Object node) {
        boolean rc = false;
        if (node instanceof ResultsComponent) {
            rc = true;
        }
        return rc;
    }

    public void valueForPathChanged(TreePath path, Object newValue) {
        // tree is not editable...ignore!
    }

    public void dataModelChanged(DataModelEvent e) {

    }

    /**
     * Simple way to update the entire tree.
     */
    public void updateTree() {
        setRootLabel(null);
        Object path[] = new Object[1];
        path[0] = root;
        fireTreeStructureChanged(this, path, null, null);
        theTree.expandPath(new TreePath(root));
    }

    protected Enumeration getChildren(Object parent) {
        Object rc = null;
        if (parent == root) {
            if (groupDataByLevel == 0) {
                rc = dm.getReferenceSet();
            }
        } else if (parent instanceof ResultsContainer) {
            rc = ((ResultsContainer) parent).getDescendants().elements();
            if (rc == null) {
                System.err.println("No children");
            }
        } else if (parent instanceof ResultsComponent) {
            rc = null;
        } else {
            rc = null;
        }
        return (Enumeration) rc;
    }

    public Object[] getPathToRoot(Object child) {
        Object[] path = null;
        int size = 0;
        if (child instanceof ResultsComponent) {
            Vector tmpVect = new Vector();
            ResultsComponent child2 = (ResultsComponent) child;
            tmpVect.addElement(child);

            ResultsContainer father = (ResultsContainer) child2.getFather();
            do {
                tmpVect.addElement(father);
                father = father.getFather();
            } while (father != null);

            tmpVect.addElement(getRoot());
            size = tmpVect.size();
            path = new Object[size];
            for (int i = 0; i < size; i++) {
                path[size - i - 1] = tmpVect.elementAt(i);
            }
        }

        if (child instanceof ResultsContainer) {
            ResultsContainer child2 = (ResultsContainer) child;
            Vector tmpVect = new Vector();
            tmpVect.addElement(child);
            ResultsContainer father = (ResultsContainer) (child2.getFather());

            do {
                tmpVect.addElement(father);
                father = father.getFather();
            } while (father != null);

            tmpVect.addElement(getRoot());
            size = tmpVect.size();
            path = new Object[size];
            for (int i = 0; i < size; i++) {
                path[size - i - 1] = tmpVect.elementAt(i);
            }
        }

        if (child instanceof RootNode) {
            path = new Object[1];
            path[0] = child;
        }
        return path;
    }

    public Vector getAllTheLeaves(Object father) {
        allTheLeaves.removeAllElements();
        rec_traverse(father);
        return allTheLeaves;
    }

    public void rec_traverse(Object father) {
        if (isLeaf(father)) {
            allTheLeaves.addElement(father);
            return;
        }

        for (Enumeration e = getChildren(father); e.hasMoreElements();) {
            rec_traverse(e.nextElement());
        }
    }

    /**
     * inner class RootNode
     * pretty much the same as for the DFSFileTreeTableModel
     *
     */

    class RootNode {
        private String label = null;

        public String toString() {
            return label;
        }

        void setLabel(String s) {
            label = s;
        }

        String getLabel() {
            return label;
        }
    }

    ////// RECURSIVE PROCEDURE ////////////////////

    public void expandAllTheNodes(int row) {
        final TreePath tp = theTree.getPathForRow(row);
        final Runnable recExpander = new Runnable() {
            public void run() {
                Cursor tmpCursor = theTree.getCursor();
                recExpand(tp);
                theTree.setCursor(tmpCursor);
            }
        };

        new Thread(recExpander).start();
        //recExpander.run();
    }

    private void recExpand(TreePath tp) {
        if (tp == null) {
            return;
        }
        Object parent = tp.getLastPathComponent();
        theTree.expandPath(tp);
        if (isLeaf(parent)) {
            return;
        }
        for (Enumeration e = getChildren(parent); e.hasMoreElements();) {
            Object tmpObj = e.nextElement();
            recExpand(tp.pathByAddingChild(tmpObj));
        }
    }

    ///////////////////////////////////////////////////////

    public boolean isPopupSuitable(Object container) {
        if (!(container instanceof ResultsContainer)) {
            return true;
        }
        return false;
    }

    /**
     * this is just a convenience class to
     * execute the fireTreeStructureChanged method
     * in a separate Thread
     *
     */

    class Reloader implements Runnable {
        Object[] path = null;
        public Reloader(Object[] path) {
            this.path = path;
        }
        public void run() {
            fireTreeStructureChanged(
                ResultsTreeTableModel.this,
                path,
                null,
                null);
        }
    }

}

// end of file ResultsTreeModel.java
