/**
 * @author N. Kornweibel
 * @version 1.0
 * The DefaultBoolKeywordExpr is an abstract class implementing
 * the BoolKeywordExpr interface but allowing the
 * expression to be represented in a Tree data structure.
 * 
 * TODO: it is unfortunate that this class requires swing simply because
 * it is a tree node. Should use a tree node interface that is not GUI
 * related.
 * 
 */

package org.eso.gasgano.keyword;

import java.io.Serializable;

import javax.swing.tree.DefaultMutableTreeNode;

public abstract class DefaultBoolKeywordExpr 
    extends DefaultMutableTreeNode implements BoolKeywordExpr, Serializable {

    static final long serialVersionUID = 8637099090928103415L;
    /**
     * Evaluate (using the KeywordSet) the expression, including the
     * evaluation of child nodes as appropriate.
     * @see org.eso.gasgano.keyword.BoolKeywordExpr#evaluate(KeywordSet)
     */
    public abstract boolean evaluate(KeywordSet s)
        throws ExpressionEvaluationException;

    public String toString() {
        return getOpString();
    }
    
    /**
     * Get the string representing the operator or expression in the node.
     * @return String
     */
    public abstract String getOpString();
    
    /**
     * Create and return a (deep) copy of this node, including
     * a copy of all its children, if any.
     * @return DefaultBoolKeywordExpr
     */
    public abstract DefaultBoolKeywordExpr copy();
}
// end of file DefaultBoolKeywordExpr.java.
