package org.eso.gasgano.properties;

import jal.String.Sorting;

import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.Vector;

import org.eso.gasgano.keyword.Keyword;
import org.eso.gasgano.tools.DirUtils;

/**
 * TODO: 
 * 1. refactor: this class is too monolithic - it needs MORE refactoring
 * to reduce its complexity and scope in the application.
 * 2. Remove the direct access to the various Vector/Map public members - these
 * should be encapsulated in the class and accessible through methods.
 * 
 * GasProp, revised edition Gasgano' Properties
 * this class contains a heap of static strings, to be used
 * as default for properties,  in case they are not provided
 * by some preferences file
 *
 * the static variables are not meant to be accessed directly but
 * rather through the more tedious 
 * getProperty("OLAS_DAYS") approach, which will always return a String
 * 
 * Functionality provided is saving to and loading from a file.
 * Preferred constructor is without file, since a loadProperties 
 * method can be called later, but the latter returns a boolean
 * indicating success or failure
 *
 * When no file is specificed, some, but not all values
 * are set from a static collection.
 * 
 * TODO: Refactoring note: GasProp follows a singleton pattern. 
 * The getInstance()/setyInstance() calls are named getProperties()
 * and setProperties(), respec. This should be changed and the singleton
 * pattern more strictly enforced.
 * 
 * TODO: All public members need to be hidden behind setters/getters.
 * 
 * TODO: All property names need to be defined as static strings somewhere.
 *
 * @author Michele Zamparelli
 */

public class GasProp extends Properties implements PropertySource {

    /**
     * This member is being used to replace some of the property/data aspects
     * of the Application class. The model needs further refactoring.
     */
    private static GasProp properties;
    
    /**
     * Int for recording if Gasgano is running in a "safe" environment - i.e.
     * where DB access is permitted. -1 == unknown, 0 == false and 1 == true.
     */
    private static int safeEnv = -1;

    /**
     * Period max and min
     */
    public static int periodMin = 63;
    public static int periodMax = 80;

    /** 
     * unknown soldiers
     */
    public static final String UNKNOWN_PROGRAM_ID = "Unknown Program";
    public static final String UNKNOWN_OBSERVATION_ID = "Unknown Observation";
    public static final String UNKNOWN_TELESCOPE_ID = "Unknown Telescope";

    /** 
     * Possible tree visualization modes - 
     * TODO: Refactor:
     * Used extensively in DFSFileTreeTableModel
     * maybe appropriate to move these settings there....
     */
    public static final int TREE_DEFAULT = 0;
    public static final int TREE_DIRECTORY = 1;
    public static final int TREE_TELESCOPE = 2;

    /** 
     * instance variables
     * TODO: this needs to be done for all keys instead of
     * using string literals in the code!
     */
    public static final String SHOW_EXTENSIONS_KEY = "SHOW_EXTENSIONS";
    public static final String AUTO_DISPLAY_FILE = "AUTO_DISPLAY";

    /**
     * here comes the endless list of static DEFAULT  values
     */
    public static final String KEYWORDLIST_DIR = "gasgano/keywords";
    public static final String SCRIPTS_DIR = "gasgano/scripts";
    public static final String FILTER_FILE = "gasgano/config/filter.rul";
    public static final String CLASSRULE_FILE = "gasgano/config/VLT.rul";
    public static final String TFITS_TO_VIEWER = "";
    public static final String FITS_TO_VIEWER = "";
    public static final String CUBE_TO_VIEWER = "";
    public static final String TEXTFILE_EXTENSIONS = "ascii;paf;longlog;log";
    public static final String FRAME_BOUNDS = "8,100,920,650";
    public static String VERBOSE_LEVEL = "0";
    public static String PRINT_FONT = "Monospaced:16";
    public static String PRINT_ORIENTATION = "P";

    public static String COMPONENT_FONT = "Dialog:14";
    public static String DATA_FILES = "";
    public static String UPDATE_TIMER = "0";

    public static String DISPLAY_COLUMNS =
        "CLASSIFICATION:TPL.ID:ORIGFILE:TPL.EXPNO:TPL.NEXP";
    public static String DISPLAY_COLUMNS_SIZE = "0:0:0:0:0";

    /**
     * Preferences for OB Status information
     */
    public static String OB_STATUS_FROM_DB_REQUIRED = "false";
    public static String OBSTATUS_MAPPING =
        "P:Partially defined D:Defined Q:queued for scheduling I:Initiated S:Started p:Paused during execution A:Aborted X:Executed R:Reduced C:Completed L:released K:Cancelled ";

    public static String DB_SERVER_URL = "";
    public static String DB_USER_NAME = "qc";
    public static final String RADEC_CONVERSION = "true";
    public static final String SHORT_FILENAME = "true";

    public static final String DBOBS_REQUIRED = "false";
    public static final String DBOBS_SERVER_URL = "";
    public static final String DBOBS_SERVER_ID = "";
    public static final String DBOBS_USERID = "";
    public static final String DBOBS_PERIOD = "66";

    public static final String MENU_CUSTOM1 = "";
    public static final String MENU_CUSTOM2 = "";
    public static final String MENU_CUSTOM3 = "";
    public static final String MENU_CUSTOM4 = "";

    public static final String MENU_CUSTOM1_LBL = "Custom  Menu1";
    public static final String MENU_CUSTOM2_LBL = "Custom  Menu2";
    public static final String MENU_CUSTOM3_LBL = "Custom  Menu3";
    public static final String MENU_CUSTOM4_LBL = "Custom  Menu4";
    public static final String MENU_RETRIEVE =
        "dpclient  -renameschema ARCFILE -outpath ";
    public static final String DBOBS_INSTRUMENT = "All";

    public static final String INSTRUMENT_GROUPING =
        "ISAAC=;FORS1=;FORS2=;UVES=;VIMOS=;NAOS+CONICA=;EFOSC=;CES=;SUSI=;SOFI=;EMMI=;WFI=;VINCI=;MIDI=";

    public static String DBOBS_DISPLAY_COLUMNS =
        "CLASSIFICATION:RA:DEC:TPL.EXPNO:TPL.NEXP";
    public static String DBOBS_DISPLAY_COLUMNS_SIZE = "0:0:0:0:0";
    public static String SHORTEN_FILES_PATH = "true";
    public static String MJD_SORTING = "false";
    public static String SHOW_EXTENSIONS = "true";
    public static String PRINTER_NAME = "lp";
    public static String DBOBS_INSTRUMENTS =
        "ISAAC;FORS1;FORS2;UVES;VIMOS;CONICA;EFOSC;CES;SUSI;SOFI;EMMI;WFI;VINCI;MIDI";
        
    // The start of String constants of property names.
    public static final String RECIPE_SET = "RECIPE_SET";

    /**
     * set of listeners interested in receiving notification
     * of changes to the properties instance.
     */
    private static Vector changeListeners = new Vector();
    // end of static part

    /**
     * Important: when adding members, be sure to update the copyFrom()
     * method to include those member.
     */
    /**
     * Source file where properties were originally loaded.
     */
    private String sourceFile = null;

    /**
     * Is this class in debug mode or not.
     */
    private boolean debug = false;

    /**
     * Array type properties (built from String properties).
     */
    public Vector obsStatus = new Vector();
    public Vector obsStatusMeaning = new Vector();
    public Vector fsDisplayColumns;
    public Vector fsDisplayColumnsSize;
    public Vector dbDisplayColumns;
    public Vector dbDisplayColumnsSize;
    public Vector dataFiles;
    public Vector textFileExtensions;

    /**
     * Maps library names to recipe list names (comma seperated list of recipes),
     * both key and values are String objects.
     */
    private Map recipeSet;

    /**
     * instrument specific intermediate containers
     * A Vector of String names of instruments.
     */
    public Vector instruments;

    /**
     * A Vector of String (comma seperated keywords).
     */
    public Vector instrumentGrouping;

    /**
     * 
     */
    public Vector dbInstruments;

    /**
     * constructor - build a default property set.
     */
    public GasProp() {
        // here we should first set the minimal properties 
        // which the code expects to find
        put("KEYWORDLIST_DIR", KEYWORDLIST_DIR);
        put("SCRIPTS_DIR", SCRIPTS_DIR);
        put("FILTER_FILE", FILTER_FILE);
        put("CLASSRULE_FILE", CLASSRULE_FILE);
        put("TFITS_TO_VIEWER", TFITS_TO_VIEWER);
        put("FITS_TO_VIEWER", FITS_TO_VIEWER);
        put("CUBE_TO_VIEWER", CUBE_TO_VIEWER);
        put("TEXTFILE_EXTENSIONS", TEXTFILE_EXTENSIONS);
        put("VERBOSE_LEVEL", VERBOSE_LEVEL);
        put("DISPLAY_COLUMNS", DISPLAY_COLUMNS);
        put("DISPLAY_COLUMNS_SIZE", DISPLAY_COLUMNS_SIZE);
        put("OB_STATUS_FROM_DB_REQUIRED", OB_STATUS_FROM_DB_REQUIRED);
        put("DB_SERVER_URL", DB_SERVER_URL);
        put("DB_USER_NAME", DB_USER_NAME);
        put("PRINT_FONT", PRINT_FONT);
        put("PRINT_ORIENTATION", PRINT_ORIENTATION);

        put("COMPONENT_FONT", COMPONENT_FONT);
        put("FRAME_BOUNDS", FRAME_BOUNDS);
        put("DATA_FILES", DATA_FILES);
        put("RADEC_CONVERSION", RADEC_CONVERSION);
        put("SHORT_FILENAME", SHORT_FILENAME);
        put("UPDATE_TIMER", UPDATE_TIMER);
        put("DBOBS_REQUIRED", DBOBS_REQUIRED);
        put("DBOBS_SERVER_URL", DBOBS_SERVER_URL);
        put("DBOBS_SERVER_ID", DBOBS_SERVER_ID);
        put("DBOBS_USERID", DBOBS_USERID);
        put("DBOBS_PERIOD", DBOBS_PERIOD);

        put("MENU_CUSTOM1", MENU_CUSTOM1);
        put("MENU_CUSTOM2", MENU_CUSTOM2);
        put("MENU_CUSTOM3", MENU_CUSTOM3);
        put("MENU_CUSTOM4", MENU_CUSTOM4);

        put("MENU_CUSTOM1_LBL", MENU_CUSTOM1_LBL);
        put("MENU_CUSTOM2_LBL", MENU_CUSTOM2_LBL);
        put("MENU_CUSTOM3_LBL", MENU_CUSTOM3_LBL);
        put("MENU_CUSTOM4_LBL", MENU_CUSTOM4_LBL);

        put("MENU_RETRIEVE", MENU_RETRIEVE);
        put("INSTRUMENT_GROUPING", INSTRUMENT_GROUPING);
        put("DBOBS_INSTRUMENT", DBOBS_INSTRUMENT);
        put("DBOBS_DISPLAY_COLUMNS", DBOBS_DISPLAY_COLUMNS);
        put("DBOBS_DISPLAY_COLUMNS_SIZE", DBOBS_DISPLAY_COLUMNS_SIZE);
        put("SHORTEN_FILES_PATH", SHORTEN_FILES_PATH);
        put("MJD_SORTING", MJD_SORTING);
        put(GasProp.SHOW_EXTENSIONS_KEY, SHOW_EXTENSIONS);
        put("PRINTER_NAME", PRINTER_NAME);
        put("OBSTATUS_MAPPING", OBSTATUS_MAPPING);
        put("DBOBS_INSTRUMENTS", DBOBS_INSTRUMENTS);
        put(GasProp.AUTO_DISPLAY_FILE, "false");
        //setupInstruments();
        buildVectors();

    }

    // constructor with filename 
    public GasProp(String filePath) {
        // here we should first set the minimal properties 
        // which the code expects to find
        this();
        //setupInstruments();
        loadProperties(filePath);
    }

    // constructor with other properties
    public GasProp(GasProp gp) {
        // I do not like this solution, but it will work for the moment
        copyFrom(gp);
    }

    /**
     * the order in which this method adds the instruments
     * to the Vector is critical, since the user preferences
     * store the keywords corresponding to each instrument
     * according to this list. This is a bit weak...
     */

    /*
    private void setupInstruments() { 
      hcInstruments.addElement("ISAAC") ; // ("TPL.START",null));
      hcInstruments.addElement("FORS1" );
      hcInstruments.addElement("FORS2");
      hcInstruments.addElement("UVES" ) ;
      hcInstruments.addElement("VIMOS"); //  new Keyword("OCS.CON.QUAD",null));
      hcInstruments.addElement("CONICA");
      hcInstruments.addElement("EFOSC");
      hcInstruments.addElement("CES");
      hcInstruments.addElement("SUSI");
      hcInstruments.addElement("SOFI");
      hcInstruments.addElement("EMMI");
      hcInstruments.addElement("WFI");
      hcInstruments.addElement("VINCI");
      hcInstruments.addElement("MIDI");
    }
    */

    /**
     * Copy method. Should be replaced with clone() to be sure all members
     * are copied.
     * Vector data is built from the string properties, not from the
     * source vector data as it is not know if the source has built
     * the vector data or not.
     */
    public void copyFrom(GasProp source) {
        sourceFile = source.sourceFile;
        debug = source.debug;

        clear();
        // notice that we do not know if the source has dumped its vectors
        // so we should rebuild the vectors from the string properties instead
        // of simply copying the vector contents from the source.
        for (Enumeration e = source.propertyNames(); e.hasMoreElements();) {
            String p = new String((String) (e.nextElement()));
            put(p, new String(source.getProperty(p)));
        }
        buildVectors();
        fsDisplayColumnsSize = (Vector) source.fsDisplayColumnsSize.clone();
        dbDisplayColumnsSize = (Vector) source.dbDisplayColumnsSize.clone();
    }

    /**
     * A tokenizer-like function - break a String into tokens and return
     * them as a Vector of Strings.
     * TODO: replace with String.split().
     */
    public static Vector stringScanner(String property, char sep) {
        if (property == null) {
            return null;
        }
        Vector v = new Vector();
        for (int i = 0; i < property.length();) {

            int colonpos = property.indexOf(sep, i);
            if (colonpos == -1) {
                colonpos = property.length();
            }
            v.addElement(property.substring(i, colonpos).trim());
            i = colonpos + 1;
        }
        return v;
    }

    /**
     * Detokenizer type function: take a vector of string tokens
     * and build a single string from it.
     */
    public static String stringBuilder(Vector property, char sep) {
        StringBuffer sb = new StringBuffer("");
        if (property == null) {
            return sb.toString();
        };
        for (int i = 0; i < property.size(); i++) {
            sb.append(property.elementAt(i).toString());
            if (i != property.size() - 1) {
                sb.append(String.valueOf(sep));
            }
        }
        return sb.toString();
    }

    /**
     * Load the preferences from a file
     * @param filePath
     * @return boolean
     */
    public boolean loadProperties(String filePath) {
        boolean debug = false;
        Enumeration e;
        FileInputStream fileStream = null;
        sourceFile = filePath;
        try {
            fileStream = new FileInputStream(filePath);
            // Note on Properties.load():
            // There is no exception thrown if the file is an invalid format
            // (such as a binary file) - simply no properties will be read in.
            load(fileStream);
        } catch (IOException ioe) {
            return false;
        }
        // check over the frame bounds
        if (getProperty("FRAME_BOUNDS") == null) {
            put("FRAME_BOUNDS", FRAME_BOUNDS);
        }
        // in fact now some checks shoule be made
        for (e = propertyNames(); e.hasMoreElements();) {
            String p = new String((String) (e.nextElement()));
            // System.out.println(p +" = "+getProperty(p)  );  
        }
        // here we try to check whether the Period is consistent
        String period = (String) getProperty("DBOBS_PERIOD");
        try {
            if (!period.equals("All")
                && !(periodMin < Integer.parseInt(period)
                    && Integer.parseInt(period) < periodMax)) {
                put("DBOBS_PERIOD", String.valueOf(periodMin));
            }
        } catch (NumberFormatException nfe) {
            put("DBOBS_PERIOD", String.valueOf(periodMin));
        }
        buildVectors();
        return true;
    }

    /**
     * Construct all vector information from string property values.
     * Called when new properties are set/read-in or a new instance is assigned.
     */
    private void buildVectors() {
        // first make sure that all vectors point to nothing
        instrumentGrouping = null;
        obsStatus = new Vector();
        obsStatusMeaning = new Vector();
        fsDisplayColumns = null;
        fsDisplayColumnsSize = null;
        dbDisplayColumns = null;
        dataFiles = null;
        textFileExtensions = null;
        recipeSet = new HashMap();
        String prop = null;
        // here we try to put the obs status variables into an array of strings
        if (getProperty("OBSTATUS_MAPPING") != null
            && !getProperty("OBSTATUS_MAPPING").equals("")) {
            prop = new String(getProperty("OBSTATUS_MAPPING"));
            char curr;
            curr = prop.charAt(0);
            for (int i = 0; i < prop.length(); i++) {
                int colonpos = prop.indexOf(":", i);
                if (colonpos == 0) {
                    break;
                };
                int colonpos2 = prop.indexOf(":", colonpos + 1);
                if (colonpos2 == -1) {
                    colonpos2 = prop.length() + 1;
                }
                obsStatus.addElement(new Character(prop.charAt(colonpos - 1)));
                obsStatusMeaning.addElement(
                    (prop.substring(colonpos + 1, colonpos2 - 2)));
                i = colonpos2 - 1;
            }
            if (debug) {
                for (int i = 0; i < obsStatus.size(); i++) {
                    System.out.println(
                        obsStatus.elementAt(i).toString()
                            + "+"
                            + obsStatusMeaning.elementAt(i).toString());
                }
            }
        }

        if (getProperty("DATA_FILES") != null) {
            prop = getProperty("DATA_FILES");
            dataFiles = stringScanner(prop, ';');
        }

        prop = getProperty("DISPLAY_COLUMNS");
        fsDisplayColumns = stringScanner(prop, ':');

        prop = getProperty("DISPLAY_COLUMNS_SIZE");
        fsDisplayColumnsSize = stringScanner(prop, ':');

        if (fsDisplayColumnsSize.size() != fsDisplayColumns.size()) {

            if (debug) {
                System.out.println(
                    "Extending the Size vector to " + fsDisplayColumns.size());
            }
            fsDisplayColumnsSize = new Vector();
            for (int i = 0; i < fsDisplayColumns.size(); i++) {
                fsDisplayColumnsSize.addElement(new Integer(0));
            }
        }

        prop = getProperty("DBOBS_DISPLAY_COLUMNS");
        dbDisplayColumns = stringScanner(prop, ':');

        prop = getProperty("DBOBS_DISPLAY_COLUMNS_SIZE");
        dbDisplayColumnsSize = stringScanner(prop, ':');
        if (dbDisplayColumnsSize.size() != dbDisplayColumns.size()) {
            System.out.println(
                "Extending the Size vector to " + dbDisplayColumns.size());
            dbDisplayColumnsSize = new Vector(dbDisplayColumns.size());
            for (int i = 0; i < dbDisplayColumns.size(); i++) {
                dbDisplayColumnsSize.setElementAt(new Integer(0), i);
            }
        }

        prop = getProperty("TEXTFILE_EXTENSIONS");
        textFileExtensions = stringScanner(prop, ';');

        prop = getProperty(RECIPE_SET);

        // break RECIPE_SET into a seperated strings, each of format:
        // recipe1,recipe2,etc=LIBRARY.  
        // full format is: rec1,rec2=LIB1;rec3=LIB2;
        if (prop != null) {
            List librarySet = stringScanner(prop, ';');
            for (int i = 0; i < librarySet.size(); i++) {
                String tmpString = (String) librarySet.get(i);
                int equalIndex = tmpString.indexOf('=');
                if (equalIndex == -1) {
                    // invalid/junk data
                    continue;
                } else {
                    
                    String libName = tmpString.substring(equalIndex + 1,tmpString.length());
                    recipeSet.put(
                        DirUtils.convertAbsoluteDir(libName),
                        tmpString.substring(0, equalIndex));
                }
            }
        }

        prop = getProperty("INSTRUMENT_GROUPING");

        // at this point the instrumentGrouping vector contains
        // an array of Strings of format INST=KEYWORD1,KEYWORD2,etc
        instrumentGrouping = stringScanner(prop, ';');
        instruments = new Vector();
        for (int i = 0; i < instrumentGrouping.size(); i++) {
            String tmpString = (String) instrumentGrouping.elementAt(i);
            int equalIndex = tmpString.indexOf('=');
            if (equalIndex == -1) {
                instrumentGrouping.removeElementAt(i);
            } else {
                String inst = tmpString.substring(0, equalIndex);
                if (instruments.contains(inst)) {
                    instrumentGrouping.removeElementAt(i);
                } else {
                    // add the instrument to the instrumens vector.
                    instruments.addElement(inst);
                    // add the comma seperated list of keywords to
                    // replace the existing entry (INST=KEYWORD1, etc).
                    instrumentGrouping.setElementAt(
                        tmpString.substring(equalIndex + 1, tmpString.length()),
                        i);
                }
            }
        }
        prop = getProperty("DBOBS_INSTRUMENTS");
        dbInstruments = stringScanner(prop, ';');
    }

    /**
     * Dump the vector type properties.
     */
    public void dumpVectors() {
        // here we save all the properties
        //  notice that since an editor might have been active we
        // can only restore the strings from those vectors
        // which are touched by the editor
        boolean debug = false;

        // but the obsStatusMappings back into a property string
        StringBuffer t = new StringBuffer("");
        for (int i = 0; i < obsStatus.size(); i++) {
            t.append(
                obsStatus.elementAt(i).toString()
                    + ":"
                    + obsStatusMeaning.elementAt(i).toString()
                    + " ");
        }

        if (debug) {
            System.out.println(t);
        }
        put("OBSTATUS_MAPPING", t.toString());
        put("DISPLAY_COLUMNS", stringBuilder(fsDisplayColumns, ':'));
        put("DBOBS_DISPLAY_COLUMNS", stringBuilder(dbDisplayColumns, ':'));
        // column sizes
        put("DISPLAY_COLUMNS_SIZE", stringBuilder(fsDisplayColumnsSize, ':'));
        put(
            "DBOBS_DISPLAY_COLUMNS_SIZE",
            stringBuilder(dbDisplayColumnsSize, ':'));
        t = new StringBuffer("");
        for (int i = 0; i < instruments.size(); i++) {
            t.append(
                instruments.elementAt(i)
                    + "="
                    + instrumentGrouping.elementAt(i)
                    + ";");
        }
        put("INSTRUMENT_GROUPING", t.toString());
        put("DBOBS_INSTRUMENTS", stringBuilder(dbInstruments, ';'));
    }

    /**
     * Save the properties to the sourceFile.
     * @return boolean
     */
    public boolean saveProperties() {
        return saveProperties(sourceFile);
    }

    /**
     * Get the source file.
     * @return String the source file path 
     */
    public String getSourceFile() {
        return sourceFile;
    }

    /**
     * SaveAs type functionality: save properties to the specified file.
     * @param filePath
     * @return boolean
     */
    public boolean saveProperties(String filePath) {
        Vector tmpVect = new Vector();
        String[] tmpArray = null;
        FileWriter fileOutputStream = null;

        if (filePath == null) {
            return false;
        };

        dumpVectors();

        for (Enumeration e = propertyNames(); e.hasMoreElements();) {
            tmpVect.addElement(e.nextElement());
        }
        tmpArray = new String[tmpVect.size()];
        tmpVect.copyInto(tmpArray);
        Sorting.sort(tmpArray, 0, tmpArray.length);

        try {
            fileOutputStream = new FileWriter(filePath);
            String header = "# Gasgano Application Properties \n# Date: "+(new Date())+"\n";
            fileOutputStream.write(header, 0, header.length());
            for (int i = 0; i < tmpArray.length; i++) {
                String value = escape((String) get(tmpArray[i]));
                String line = tmpArray[i] + "=" + value + "\n";
                fileOutputStream.write(line, 0, line.length());
                //System.out.println(line);
            }
            fileOutputStream.close();
            return true;
        } catch (IOException ioe) {
            return false;
        }
    }

    /**
     * Add a listener to the set of listeners.
     * @param l
     */
    public static void addChangeListener(GasPropListener l) {
        changeListeners.add(l);
    }

    /**
     * Remove a listener from the set of listeners.
     * @param l
     */
    public static void removeChangeListener(GasPropListener l) {
        changeListeners.remove(l);
    }

    /**
     * Notifiy all listeners that a new instance has been assigned.
     */
    private static void notifyListeners(boolean dataModelChange) {
        for (int i = 0; i < changeListeners.size(); i++) {
            ((GasPropListener) changeListeners.elementAt(i)).newProperties(
                dataModelChange);
        }
    }

    /**
     * GasProp follows a singleton pattern - this call sets the singleton
     * object.
     * @param prop
     */
    public static void setProperties(GasProp prop) {
        setProperties(prop, false);
    }

    /**
     * GasProp follows a singleton pattern - this call sets the singleton
     * object.
     * @param prop
     * @param dataModelChange - if true, indicates that the new
     * instance has properties different to the current instance that
     * are related to the data model. This info is useful as it may
     * require rebuilding the data model or restarting the application.
     */
    public static void setProperties(GasProp prop, boolean dataModelChange) {
        properties = prop;
        if (dataModelChange) {
            // to be sure, rebuild the vectors.
            properties.buildVectors();
            // set this instance to the PropertyDB property source.
        }
        PropertyDB.getInstance().setPropertySource(properties);
        // notify all listeners that there is a new property instance.
        notifyListeners(dataModelChange);
    }

    /**
     * GasProp follows a singleton pattern - this call gets the singleton
     * object.
     * @param prop
     */
    public static GasProp getProperties() {
        return properties;
    }

    /**
     * Force the setting of the environment in to the unsafe 
     * i.e. untrusted mode. This needs to be done before the
     * first call the isEnvironmentSafe().
     */
    public static void setEnvironmentUnsafe() {
        if (safeEnv == -1) {
            // only set it if the member has not yet been set.
            safeEnv = 0;
        }
    }

    /**
     * simply assess whether some classes are available or not
     * If they aren't, it means we are running in a non secure environment
     * The classes which are inspected are <tt>OBStatusDictionary</tt> and
     * <tt>DBDataModel</tt>
     * The safe mode can be forced off using setEnvironmentUnsafe().
     */
    public static boolean isEnvironmentSafe() {
        boolean rc = safeEnv == 0 ? false : true;
        if (safeEnv == -1) {
            safeEnv = 1;
            try {
                Class t =
                    Class.forName(
                        "org.eso.gasgano.datamodel.OBStatusDictionary");
            } catch (Exception cnfe) {
                safeEnv = 0;
                getProperties().put("OB_STATUS_FROM_DB_REQUIRED", "false");
            }

            //ClassNotFoundException
            try {
                Class t =
                    Class.forName(
                        "org.eso.gasgano.datamodel.database.DBDataModel");
            } catch (Exception cnfe) {
                safeEnv = 0;
                getProperties().put("DBOBS_REQUIRED", "false");
            }
            rc = safeEnv == 0 ? false : true;
            if (rc) {
                System.out.println("Gasgano running in 'safe' environment.");
            }
        }
        return rc;
    }

    /**
     * @params: code is 0 for filesystem, 1 for database
     * 
     * sets the new size for a specific column in the vector 
     * of the preferences
     */
    public static void columnSizeChanged(
        int code,
        String columnName,
        int newSize) {
        Vector displayCols, columnSize = null;
        int pos = 0;
        if (code == 0) {
            displayCols = getProperties().fsDisplayColumns;
            columnSize = getProperties().fsDisplayColumnsSize;
        } else if (code == 1) {
            displayCols = getProperties().dbDisplayColumns;
            columnSize = getProperties().dbDisplayColumnsSize;
        } else {
            System.out.println(
                "gasgano internal error in Application::columnSizeChanged");
            return;
        }
        pos = displayCols.indexOf(columnName);

        if (pos != -1) {
            Integer nSize = new Integer(newSize);
            columnSize.setElementAt(nSize, pos);
            //System.out.println("Set  size "+ columnSize.elementAt(pos) + " for column "+columnName+" in pos "+pos);
        } else {
            System.out.println(
                "gasgano internal error: "
                    + columnName
                    + " is not in the displayed keywords preferences");
        }
    }

    /**
     * Get an array of Keyword objects for the requested instrument.  
     * @param instrument
     * @return Keyword
     */
    public Keyword[] getInstrumentGrouping(String instrument) {
        Keyword[] rc = null;
        int instIdx = instruments.indexOf(instrument);
        if (instIdx != -1) {
            String tmpString = (String) instrumentGrouping.elementAt(instIdx);
            if (tmpString.length() > 0) {
                // split the tmpString into individual keywords.
                String[] kwords = tmpString.split(",");
                // trim strings and remove empty strings
                ArrayList kwVec = new ArrayList();
                for (int i = 0; i < kwords.length; i++) {
                    String kw = kwords[i].trim();
                    if (kw.length() > 0) {
                        kwVec.add(new Keyword(kw, "null"));
                    }
                }
                rc = new Keyword[kwVec.size()];
                kwVec.toArray(rc);
            }
        }
        return rc;
    }

    /**  
     * @param instrument
     * @return boolean if this instrument has keyword grouping information.
     */
    public boolean hasInstrumentGrouping(String instrument) {
        return instruments.indexOf(instrument) != -1;
    }

    /**
     * @return String, with the "\" character escaped.
     */
    private static String escape(String value) {
        StringBuffer escapedValue = new StringBuffer();
        if (value != null) {
            for (int i = 0; i < value.length(); i++) {
                if (value.charAt(i) == '\\') {
                    escapedValue.append('\\');
                }
                escapedValue.append(value.charAt(i));
            }
        }
        return escapedValue.toString();
    }

    /**
     * Get an iterator for the String names of the CPL library names configured.
     * @return Iterator
     */
    public Iterator getRecipeLibraryNames() {
        return recipeSet.keySet().iterator();
    }

    /**
     * Get all the recipes configured for the given library, or an empty list
     * if the library was not in the configuration or contained no recipes.
     * This query deals with the config data only and has nothing to do with
     * the libraries themselves, or loading the actual libraries.....
     * @param libName
     * @return List
     */
    public List getRecipesForLibrary(String libName) {
        List rc;
        String recipeList = (String) recipeSet.get(libName);
        if (recipeList != null) {
            rc = stringScanner(recipeList, ',');
        } else {
            rc = null;
        }
        return rc;
    }
}
