package org.eso.ohs.core.dbb.client;

import java.awt.Choice;
import java.awt.Component;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import org.eso.ohs.core.dbb.sql.DbbSqlChunk;

/**
 * This is an implementation of the DbbWidget abstract class,
 * defining a Choice widget.
 * 
 * @author Carlo Boarotto, ESO 19-Jul-1999
 */

//$Id: DbbChoice.java,v 1.1 2004/02/17 11:16:24 tcanavan Exp $

public class DbbChoice extends DbbWidget implements ItemListener {

    /** Override super class variable because choice is not a JComponent. */
    protected Choice widget_;
    
	private Object[] dbArray_; //added by Judith
	private static String CLEAR_SELECTION = DbbSelectionList.CLEAR_SELECTION;//added by Judith

    //----------------------------------------
    // Constructors
    //----------------------------------------

    /**
     * Create a DbbChoice object.
     * 
     * @param labeltext	Label text to be displayed.
     * @param chunk	DbbSqlChunk object
     *  
     */
    public DbbChoice( String labeltext, DbbSqlChunk chunk ) {

	super( labeltext, chunk );

	setDefault( "" );

	widget_ = new Choice();
	label_.setLabelFor(widget_);
	widget_.addItemListener(this);
	addItem( "" );
    }

    //----------------------------------------
    // Public Methods
    //----------------------------------------

    /** Add a numeric item to the list.
     * @param item	Integer item.
     */
    public void addItem(int item) {
	widget_.addItem(String.valueOf(item));
    }

    /** Add a string item to the list.
     * @param item	String item.
     */
    public void addItem(String item) {
	widget_.addItem(item);
    }

    /** Add a numeric item list.
     * @param item_list	Integer item list.
     */
    public void addItemList(int[] item_list) {
	for(int i = 0; i < item_list.length; i++) {
	    widget_.addItem(String.valueOf(item_list[i]));
	}
    }

    /** Add a string item list.
     * @param item_list	String item list.
     */
    public void addItemList(String[] item_list) {
	for(int i = 0; i < item_list.length; i++) {
	    widget_.addItem(item_list[i]);
	}
    }

    /** Get the displayed string item.
     * @return Currently displayed item.
     */
    public String get() {
	return widget_.getSelectedItem();
    }

    /** Set the displayed string item.
     * @param Integer value to be displayed.
     */
    public void set( String value ) {
	widget_.select( String.valueOf( value ));
    }

    /** 
     * It is necessary to redefine it in order to access the
     * overridden attribute widget_.
     *
     * @return The widget
     * @see #widget_ 
     */
    public Component getWidget() { return widget_; }

    /**
     * Method to check if the widget contents have been cleared.
     *
     * @return A boolean value: <code>true</code> if it is empty,
     *         <code>false</code> otherwise.  
     */
    public boolean isEmpty() {
	String sel = get();
	return( sel.equals( "" ) );
    }

    //----------------------------------------
    // Event methods
    //----------------------------------------

    /**
     * Listener interface implementation.
     * @param ev	 Event triggering the action.
     */
    public void itemStateChanged(ItemEvent ev) {
	fireDbbWidgetAction();
    }
}

