package org.eso.ohs.core.dbb.client;

import java.awt.Dimension;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.StringTokenizer;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JList;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.ListModel;
import javax.swing.ListSelectionModel;
import javax.swing.event.EventListenerList;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.apache.log4j.Logger;
import org.eso.ohs.core.dbb.sql.DbbSqlChunk;
import org.eso.ohs.core.utilities.TextUtils;

/**
 * This is an implementation of the DbbWidget abstract class, defining a
 * SelectionList widget. The selection list allows the user to select more than
 * one item using the standard mouse action for multiple selection lists:
 * CTRL-Button1 to select separated items and Shift-Button1 to select a set of
 * contiguous items.
 * 
 * @author Carlo Boarotto, ESO 19-Jul-1999 creator
 * @version $Revision: 1.15 $
 * 
 * $Id: DbbSelectionList.java,v 1.15 2005/06/24 16:22:04 gdonino Exp $
 *  
 */

public class DbbSelectionList extends DbbWidget implements
        ListSelectionListener {

    private static final String rcsid = "$Id: DbbSelectionList.java,v 1.15 2005/06/24 16:22:04 gdonino Exp $";

    private static Logger stdlog_ = Logger.getLogger(DbbSelectionList.class);

    protected static String CLEAR_SELECTION = "[clear]";

    public static String EMPTY_SELECTION = "";

    private static int DEF_BUTTON_WIDTH = 145;

    private static int DEF_BUTTON_HIGH = 30;

    /** Internal JLIST to be popped up within a JPopupMenu. */
    protected JList jlist_ = null;

    /** Internal JPopupMenu to pop up the item list */
    protected JPopupMenu jlistButton_ = null;

    /** Database values for the items in the list */
    private Object[] dataObjects_ = null;

    /** Display values for the items in the list */
    private Object[] displayObjects_ = null;

    /**
     * Internal boolean flag to force the conversion between database values and
     * display values.
     */
    private boolean conversionFlag_ = true;

    /**
     * Boolean flag to remind if the list has been popped up or down.
     */
    private boolean popupFlag_ = false;

    /** Size of JList */
    private static final int LIST_SIZE = 10;

    //----------------------------------------
    // Constructors
    //----------------------------------------

    /**
     * Create a DbbSelectionList object with a label, associated chunk and
     * displaying values to be mapped to a specific list of data values.
     * 
     * @param labeltext
     *            Label text to be displayed.
     * @param chunk
     *            DbbSqlChunk object
     * @param listData
     *            List of data values
     * @param listDisplayData
     *            List of displaying values
     */
    public DbbSelectionList(String labeltext, DbbSqlChunk chunk,
            Object[] listData, Object[] listDisplayData) {
        super(labeltext, chunk);
        initializeDbbSelectionList(labeltext, chunk, listData, listDisplayData);
    }

    /**
     * Initialize a DbbSelectionList object with a label, associated chunk and
     * displaying values to be mapped to a specific list of data values.
     * 
     * @param labeltext
     *            Label text to be displayed.
     * @param chunk
     *            DbbSqlChunk object
     * @param listData
     *            List of data values
     * @param listDisplayData
     *            List of displaying values
     */
    private void initializeDbbSelectionList(String labeltext,
            DbbSqlChunk chunk, Object[] listData, Object[] listDisplayData) {
        default_ = "          ";

        widget_ = new JButton(default_);
        applyWidgetDimension(widget_, DEF_BUTTON_WIDTH, DEF_BUTTON_HIGH);
        //   widget_.setMaximumSize(new Dimension(145, 30));
        jlist_ = new JList();
        jlist_.setPrototypeCellValue("12345");
        jlistButton_ = new JPopupMenu();


        ((JButton) widget_).addMouseListener(new MouseListener() {

            public void mouseClicked(MouseEvent arg0) {

                if (jlistButton_.isVisible()) {
                    jlistButton_.setVisible(false);
                } else {
                    jlistButton_.show(widget_, 0, widget_.getSize().height);
                    jlistButton_.setVisible(true);

                }

            }

            public void mousePressed(MouseEvent arg0) {

            }

            public void mouseReleased(MouseEvent arg0) {

            }

            public void mouseEntered(MouseEvent arg0) {

            }

            public void mouseExited(MouseEvent arg0) {
            }

        });

        JScrollPane jlist_scroll = new JScrollPane(jlist_);
        jlistButton_.add(jlist_scroll);

        label_.setLabelFor(widget_);

        jlist_.addListSelectionListener(this);
        jlist_.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
        int numRow = setListDisplayData(listDisplayData);
        setListData(listData);
    }

    /**
     * void
     * 
     * @param widget
     * @param button_high
     * @param button_width
     */
    private void applyWidgetDimension(JComponent widget, int button_width,
            int button_high) {
        Dimension bDim = new Dimension(button_width, button_high);
        widget.setPreferredSize(bDim);
        widget.setMinimumSize(bDim);
    }

    public void setPreferredCharWidth(int char_number, boolean expand) {
        if (widget_ != null) {
            int width = getCharWidth(widget_, char_number);
            int height = (widget_.getPreferredSize()).height;

            try {
                if (expand) {
                    widget_.setPreferredSize(new Dimension(width, height));
                    widget_.setMinimumSize(new Dimension(width, height));
                } else {
                    widget_.setMaximumSize(new Dimension(width, height));
                    widget_.setMinimumSize(new Dimension(width, height));
                    widget_.setSize(width, height);
                }
            } catch (Exception e) {
                widget_.setSize(width, height);
            }

        }

    }

    public JList getList() {
        return jlist_;
    }

    /**
     * Create a DbbSelectionList object with a label, associated chunk and
     * displaying values to be mapped to a specific list of data values.
     * 
     * @param labeltext
     *            Label text to be displayed.
     * @param chunk
     *            DbbSqlChunk object
     * @param conn
     *            Connection object
     * @param sqlValues
     *            SQL Statement getting of data values
     * @param sqlDisplay
     *            SQL Statement getting of displaying values
     */
    public DbbSelectionList(String labeltext, DbbSqlChunk chunk,
            Connection con, String sqlValues, String sqlDisplay) {
        super(labeltext, chunk);
        Object[] listData = getDataFromSql(con, sqlValues);
        Object[] listDisplay = getDataFromSql(con, sqlDisplay);

        initializeDbbSelectionList(labeltext, chunk, listData, listDisplay);
    }

    /**
     * Create a DbbSelectionList object with a label, associated chunk and
     * displaying values to be mapped to a specific list of data values.
     * 
     * @param labeltext
     *            Label text to be displayed.
     * @param chunk
     *            DbbSqlChunk object
     * @param conn
     *            Connection object
     * @param sqlValues
     *            SQL Statement getting of data values. If there is more then
     *            one column the first column will be extracted.
     */
    public DbbSelectionList(String labeltext, DbbSqlChunk chunk,
            Connection con, String sqlValues) {
        super(labeltext, chunk);

        Object[] listData = getDataFromSql(con, sqlValues);
        initializeDbbSelectionList(labeltext, chunk, listData, listData);
    }

    private Object[] getDataFromSql(Connection con, String sqlValues) {
        Vector result = new Vector();
        ResultSet rs = null;
        Object[] listData;
        Object fieldCol;
        Statement stmt = null;
        //      logCateg.info(sqlValues);
        try {
            stmt = con.createStatement();
            rs = stmt.executeQuery(sqlValues);
            while (rs.next()) {
                fieldCol = rs.getObject(1);
                //          logCateg.debug(fieldCol.toString);
                result.addElement(fieldCol);
            }
            listData = new Object[result.size()];
            result.copyInto(listData);
        } catch (SQLException e) {
            // logCateg.error(e.getMessage());
            listData = new String[]
            { "" };
        }
        return listData;
    }

    /**
     * Create a DbbSelectionList object with a label, associated chunk and
     * displaying values (the same as data values).
     * 
     * @param labeltext
     *            Label text to be displayed.
     * @param chunk
     *            DbbSqlChunk object
     * @param listData
     *            List of data values
     */
    public DbbSelectionList(String labeltext, DbbSqlChunk chunk,
            Object[] listData) {

        this(labeltext, chunk, listData, listData);
        conversionFlag_ = false;
    }

    //----------------------------------------
    // Private methods
    //----------------------------------------

    /**
     * Set the text of the button according to the list selection. In case of
     * multiple selections, the widget displays only the first selected item
     * followed by dots.
     */
    private void setButtonText() {

        StringBuffer ret_str = new StringBuffer();
        Object[] objects = jlist_.getSelectedValues();
        int len = objects.length;

        if (len > 0) {
            ret_str.append(objects[0].toString());
            for (int i = 1; i < len; i++) {
                ret_str.append(", ");
                ret_str.append(objects[i].toString());
            }
        } else {
            ret_str.append(default_);
        }

        ((JButton) widget_).setText(ret_str.toString());
    }

    //----------------------------------------
    // Protected methods
    //----------------------------------------

    /**
     * Resize the popup list width according to longest word to be displayed
     * within the item list.
     */
    protected void setMaxListWidth() {

        String max_str = "12345";
        int max_len = 0;
        if (displayObjects_ != null) {
            for (int i = 0; i < displayObjects_.length; i++) {
                String tmp_str = displayObjects_[i].toString();
                int len = tmp_str.length();
                if (len > max_len) {
                    max_len = len;
                    max_str = tmp_str;
                }
            }
        }
        //www used for increasing the string size to be displayed
        max_str = max_str + "wwww";
        jlist_.setPrototypeCellValue(max_str);
    }

    /**
     * Get the data values corresponding to the selected items.
     * 
     * @return an object array defining the selected data.
     */
    public Object[] getSelectedDataObjects() {
        Object[] objects = new Object[0];
        if (dataObjects_ == null || !conversionFlag_) {
            objects = jlist_.getSelectedValues();
            for (int i = 0; i < objects.length; i++) {
                if (objects[i].equals(CLEAR_SELECTION)) {
                    jlist_.clearSelection();
                    objects = new Object[0];
                    break;
                }
            }

        } else {
            int[] indices = jlist_.getSelectedIndices();
            int len = indices.length;

            objects = new Object[len];
            
            for (int i = 0; i < len; i++) {
                int idx = indices[i];
				//questa e' la condizione da cambiare.
				//dovrei dire che se l'indice e' 0 allora e' il clear altrimenti se l'indice vale i allora prendo l'elemento i-1
               /* if (idx < dataObjects_.length) {
                    objects[i] = dataObjects_[idx];*/
				 if (idx > 0 ) {
	                    objects[i] = dataObjects_[idx-1];
                } else {
                    // selected the clear. everithing is cleared
                    
                    objects = new Object[0];
                    break;
                }
            }
        }
        return objects;
    }

    /**
     * Convert a display value into the corresponding data value.
     * 
     * @param display_obj
     *            Display value to be converted.
     * @return a data object corresponding to the given display value.
     */
    protected Object displayObjectToDataObject(Object display_obj) {

        if (dataObjects_ == null || !conversionFlag_) {         
                return display_obj;
        }
          

        String value = display_obj.toString();
        ListModel model = jlist_.getModel();
        int len = model.getSize();
        int idx = -1;

        for (int i = 0; i < len; i++) {
            Object item = model.getElementAt(i);
            if (value.equals(item.toString())) {
                idx = i;
                break;
            }
        }

        if (idx != -1 && idx < dataObjects_.length)
            return dataObjects_[idx];

        return null;
    }

    /**
     * Convert a data value into the corresponding display value.
     * 
     * @param data_obj
     *            Data value to be converted.
     * @return a data object corresponding to the given data value.
     */
    protected Object DataObjectToDisplayObject(Object data_obj) {

        if (dataObjects_ == null || !conversionFlag_) {
            return data_obj;
        }
        String value = data_obj.toString();
        int len = dataObjects_.length;
        int idx = -1;

        for (int i = 0; i < len; i++) {
            Object item = dataObjects_[i];

            if (value.equals(item.toString())) {
                idx = i;
                break;
            }
        }

        ListModel model = jlist_.getModel();
        if (idx != -1 && idx < model.getSize())
            return model.getElementAt(idx);

        return null;
    }

    /**
     * Convert a data value into the corresponding display value.
     * 
     * @param data_obj
     *            Data value to be converted.
     * @return a data object corresponding to the given data value.
     */
    protected Integer[] ObjectListToDisplayObject(String valueList) {

        Vector idList = new Vector();
        Integer[] data_int_array = null;
        if (dataObjects_ != null) {
            String[] valueArray = convertStringSetToList(valueList);
            for (int j = 0; j < valueArray.length; j++) {
                for (int i = 0; i < dataObjects_.length; i++) {
                    Object itemDisplay = dataObjects_[i];
                    if (valueArray[j].equals(itemDisplay.toString())) {
                        idList.addElement(new Integer(i));
                    }
                }
            }
        }
        data_int_array = new Integer[idList.size()];
        idList.copyInto(data_int_array);
        return data_int_array;
    }

    public static String[] convertStringSetToList(String valueList) {
        String valueListNoParenthesis = "";
        if (valueList.length() > 2) {
            valueListNoParenthesis = valueList.substring(
                    1, valueList.length() - 1);
        }
        StringTokenizer st = new StringTokenizer(valueListNoParenthesis, ",");
        Vector valueVector = new Vector();
        while (st.hasMoreTokens()) {
            String rowValue = st.nextToken();
            StringTokenizer subst = new StringTokenizer(rowValue, "'");
            String value = subst.nextToken();
            value = value.trim();
            valueVector.addElement(value);
        }
        String[] valueArray = (String[]) valueVector
                .toArray(new String[valueVector.size()]);

        return valueArray;
    }

    //----------------------------------------
    // Public methods
    //----------------------------------------

    /**
     * Set the internal list of display values.
     * 
     * @param listDisplayData
     *            Array of items to be displayed
     * @return the number of data that are displayed
     */
    public int setListDisplayData(Object[] listDisplayData) {

		//Fixing bugzilla item 4312. The old code is still here
		
		 Object[] listDisplayDataClear = new Object[0];
	        if (listDisplayData != null) {
	           listDisplayDataClear =listDisplayData;
	            listDisplayDataClear = new Object[listDisplayData.length + 1];
				/*for (int i = 0; i < listDisplayData.length; i++) {
	                listDisplayDataClear[i] = listDisplayData[i];
	            }
	            listDisplayDataClear[listDisplayData.length] = CLEAR_SELECTION;*/
				listDisplayDataClear[0] = CLEAR_SELECTION;
	            for (int i = 0; i < listDisplayData.length; i++) {
	                listDisplayDataClear[i+1] = listDisplayData[i];
	            }
	        } 
	        jlist_.setListData(listDisplayDataClear);
	        displayObjects_ = listDisplayDataClear;

	        setMaxListWidth();
	        setButtonText();

	        int numRows = listDisplayDataClear.length;
	        numRows = (numRows < LIST_SIZE) ? numRows : LIST_SIZE;
	        jlist_.setVisibleRowCount(numRows);

	        return listDisplayDataClear.length;
    }

    /**
     * Set the internal list of data values.
     * 
     * @param listDisplayData
     *            Array of items to be associated to the display values.
     */
    public void setListData(Object[] listData) {
        dataObjects_ = listData;
        if (!conversionFlag_) {
            setListDisplayData(listData);
        }
    }

    /**
     * Get the displayed string item.
     * 
     * @return Currently displayed items as a sql "in" string. <BR>
     *         Ex: ('x1', 'x2', 'x3').
     *  
     */
    public String get() {
        //A M Chavan, 01-Feb-2000: rewritten to fix a bug [no SPR]
        String ret = "";
        Object[] objects = getSelectedDataObjects();
        int len = objects.length;
        Class c = sqlChunk_.getDataType();

        if (len > 0) {
            for (int i = 0; i < len; i++) {
                String value = TextUtils.quoteSqlString(objects[i].toString());
                if (i > 0) {
                    ret += ", ";
                }
                if (c == String.class) {
                    ret += "\'" + value + "\'";
                } else {
                    ret += value;
                }
            }
            if (CLEAR_SELECTION.equals(ret)) {
                ret = "";
            }
            ret = "(" + ret + ")";
        } else {
            ret = "()";
        }
        return ret;
    }

    /**
     * Set the displayed string item.
     * 
     * @param value
     *            value to be displayed.
     */
    public void set(String value) {

        //stdlog_.debug ("the selection value \""+value+"\"");
        Integer[] idArray = ObjectListToDisplayObject(value);

        ListModel model = jlist_.getModel();
        int len = model.getSize();
        int[] idIntArray = new int[idArray.length];
        for (int i = 0; i < idArray.length; i++) {
            int id = idArray[i].intValue();

            if (id < len) {
				// id + 1 because as [CLEAR] selection is in position 0, all the 
				// following data are shifted by one.
                idIntArray[i] = id + 1;
            } else {
                idIntArray[i] = 0;
            }
        }
        jlist_.setSelectedIndices(idIntArray);

        setButtonText();
    }

    /**
     * Method to check if the widget contents have been cleared.
     * 
     * @return A boolean value: <code>true</code> if it is empty,
     *         <code>false</code> otherwise.
     */
    public boolean isEmpty() {
        Object[] objects = jlist_.getSelectedValues();
        int len = objects.length;
        return (len == 0);
    }

    /**
     * Reset the widget to its default state.
     */
    public void reset() {
        jlist_.clearSelection();
        setButtonText();
    }

    //----------------------------------------
    // Event methods
    //----------------------------------------

    /**
     * Listener interface implementation.
     * 
     * @param ev
     *            Event triggering the action.
     */
    public void valueChanged(ListSelectionEvent e) {

        // DbbWidgetEvent has to be released
        fireDbbWidgetAction();

        Object[] objects = jlist_.getSelectedValues();
        for (int i = 0; i < objects.length; i++) {
            if (objects[i].equals(CLEAR_SELECTION)) {
                jlist_.clearSelection();
                break;
            }
        }
        setButtonText();
    }

    /**
     * @param telList
     * @param telList2
     *            void
     */
    public void setListDataNoListeners(String[] objList) {
        EventListenerList tmpListenerList = listenerList_;
        listenerList_ = new EventListenerList();
        setListData(objList);
        listenerList_ = tmpListenerList;

    }
}