package org.eso.ohs.core.dbb.client;

// $Id: DbbTable.java,v 1.29 2005/11/30 14:13:41 gdonino Exp $

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Rectangle;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.lang.reflect.Field;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Vector;

import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.event.TableModelEvent;
import javax.swing.plaf.basic.BasicTableUI;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableColumnModel;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;
import javax.swing.table.TableModel;

import org.apache.log4j.Logger;
import org.eso.ohs.core.dbb.gui.SortButtonRenderer;
import org.eso.ohs.core.dbb.sql.DbbSqlQueryResult;
import org.eso.ohs.core.gui.widgets.OhsTableSorter;
import org.eso.ohs.core.gui.widgets.TableSorter;
import org.eso.ohs.core.utilities.GTreeNode;
import org.eso.ohs.core.utilities.MsgManager;
import org.eso.ohs.core.utilities.OHSConfig;
import org.eso.ohs.core.utilities.OHSGlobalVariables;

/**
 * Grid to display the query results. It contains a JTable associated
 * with a table model and a column model. It uses a table sorter class
 * in order to sort the data in memory.
 *  @see      <a href=" http: //www. eso. org/~uss/extJavadoc/jdk1.4/docs/api/javax/swing/JTable. html" >javax. swing. table.JTable.</a>
 * @see OhsTableSorter
 *
 * @author Carlo Boarotto, ESO 19-Jul-1999
 * @author cmavros 22MAY02 [1650/OT] Review DBB column width calculation
 * @author cmavros 22MAY02 [1723/OT] When a dbbtable is displayed
 *    select the first item in the list if the row count is greater than 0.
 *
 **/

public class DbbTable {
	private int defaultFontSize_ = OHSGlobalVariables.DEF_FONT_SIZE;
    private static final String rcsid =
		"$Id: DbbTable.java,v 1.29 2005/11/30 14:13:41 gdonino Exp $";
	private static Logger stdlog_ = Logger.getLogger(DbbTable.class);

	/** Scroll panel to allow horizontal and vertical scrolling. */
	protected JScrollPane scrollPanel_;

	/** JTable widget. */
	protected JTable jtable_ = null;

	protected Hashtable columnBgMap_ = null;

	protected Hashtable columnFgMap_ = null;
	/** Number of displayed rows. */
	protected int rowCount_ = 0;

	/** Number of displayed columns. */
	protected int colCount_ = 0;

	/** Table padding: number of extra pixels to pad a cell */
	private final static int PADDING_ = 6;

	/* no longer in use, since getLongValues is deprecated
	 * Store the column long values into this attribute. 
	 private String[] longValues_ = null;     
	 */
    
  //  private SortButtonRenderer sortButtonRenderer;

	/**
	 * JTable's TableModel implementation.
	 * @see DbbTableModel
	 */
	protected DbbTableModel model_ = null;

	/**
	 * Table sorter implementation.
	 * @see OhsTableSorter
	 */
	protected OhsTableSorter sorter = null;

	/**
	 * Link to the internal tabable column model. 
	 * @see TableColumnModel
	 */
	protected TableColumnModel column_model = null;

	/** Flag to force tha table repainting (and the column creation). */
	private boolean repaintTable_ = true;

	/**
	 * Hashtable containing the columns that shall not be displayed
	 * on the grid 
	 */
	private Hashtable hideCols = new Hashtable();
	private SortButtonRenderer headerRenderer_;
	private HashMap defaultColWidth_ = null;

	//----------------------------------------
	// Constructors
	//----------------------------------------

	/** Create a DbbTable object. */
	public DbbTable() {
		columnBgMap_ = new Hashtable();
		columnFgMap_ = new Hashtable();
		model_ = new DbbTableModel();
		sorter = new OhsTableSorter(model_);
		jtable_ = new JTable(sorter);
		scrollPanel_ = new JScrollPane(jtable_);
		
		/* Sort the column just clicking on its header */
		//sorter.addMouseListenerToHeaderInTable(jtable_);
		//adding listener to manage the buttons
		JTableHeader header = jtable_.getTableHeader();
		setHeaderRenderer(new SortButtonRenderer());
  //      sortButtonRenderer = new SortButtonRenderer();
		//header.addMouseListener(new HeaderListener (header, getHeaderRenderer()));
		header.addMouseListener(new HeaderListener (header, getHeaderRenderer(), this));

		/* Without shutting off autoResizeMode, the table won't scroll
		 * correctly (horizontally, anyway).
		 */
		jtable_.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);

		setUI();
		setDefaultRenderers();
	}
	/* (non-Javadoc)
	 * @see org.eso.ohs.core.dbb.client.DbbTableInterface#saveSortCriteriaToModel()
	 */
	public void saveSortCriteriaToModel() {
		int[] indexes = sorter.getIndexes();
		Vector newVec = new Vector();
		Vector dataVector = model_.getDataVector();
		for (int i = 0; i < indexes.length; i++) {
			newVec.addElement(dataVector.elementAt(indexes[i]));
		}
		model_.setDataVector(newVec);
	}

	/* (non-Javadoc)
	 * @see org.eso.ohs.core.dbb.client.DbbTableInterface#removeSortListener()
	 */
	public void removeSortListener() {
		sorter.removeMouseListenerToHeaderInTable(jtable_);
	}

	//----------------------------------------
	// Protected methods
	//----------------------------------------

	protected void setUI() {
		jtable_.setUI(new DbbTableUI());
	}

	protected void setDefaultRenderers() {
		/* Set default table cell renderers */
		jtable_.setDefaultRenderer(Integer.class, new DbbIntegerRenderer());
		jtable_.setDefaultRenderer(Float.class, new DbbFloatRenderer());
		jtable_.setDefaultRenderer(Boolean.class, new DbbBooleanRenderer());

	}

	/**
	 * Create columns setting their preferred size according to the
	 * maximum width of the data to be displayed.
	 */
	protected void setColumnSizes() {

		int width = 0;
		int columnIndex = 0;
		String column_name = null;
		TableColumn column = null;
		//re_initialise the default width hashmap as we are going to create a new JTable.
		defaultColWidth_ = new HashMap();
		/* Get the table font metrics to evaluate the cell max len */
		FontMetrics fmTable = jtable_.getFontMetrics(jtable_.getFont());
		/* Get the tableHeader font metrics to evaluate the cell max len */
//		FontMetrics fmHeader = jtable_.getTableHeader().getFontMetrics(jtable_.getTableHeader().getFont());

	//	Component[] co = jtable_.getTableHeader().getComponents();
		
		/* Set the column width (max value between the header label and
		 * the column value) 
		 */
		for (int i = 0; i < colCount_; i++) {

			column_name = model_.getColumnName(i);
			if (hideCols.containsKey(column_name))
				continue;

			/* Set the specific column renderer for DbbDataType columns */
			Class column_class = model_.getColumnClass(i);
			DbbDataTypeRenderer cell_renderer = null;
			int column_len = 0;

			/* Set the column width according to the specific
			     * DbbDataType (if properly defined) or to the column size
			     * retrieved from the model. Set the size to the column
			     * name if it is longer then the calculated width. 
			 */
			int realDataSize = 0;
//			int realLabelSize = column_name.length() * fmHeader.stringWidth("0") + ICONPADDING_;
			if (DbbDataType.class.isAssignableFrom(column_class)) {
				cell_renderer = new DbbDataTypeRenderer(column_class);
				int column_class_len = 0;
				try {
					DbbDataType new_inst =
						(DbbDataType) column_class.newInstance();
					column_class_len = new_inst.getDataTypeLength();
					realDataSize = column_class_len * fmTable.stringWidth("B");
				} catch (Exception e) {
					stdlog_.warn("", e);
				}
				
			} else {
				realDataSize = model_.getColumnSize(i) * fmTable.stringWidth("0");
			}
			//column_len = Math.max(realDataSize, realLabelSize );
			width =  column_len + PADDING_* fmTable.stringWidth("B") ;

			//compare column_name size and the calculated width, and then take the bigger as new column_width
			/*FontMetrics fmHeader = jtable_.getTableHeader().getFontMetrics(jtable_.getTableHeader().getFont());
			
			if (column_name.length() > width)
				width = column_name.length();
			*/
			/* Create the column (with a specific width and header value) and
			 * add it to the table column model 
			 */
			column = new TableColumn(i, width);
			// Uncomment that to set minWidth for DbbTable Columns
			//column.setMinWidth(width);
			column.setHeaderValue(column_name);
			column_model.addColumn(column);
			defaultColWidth_.put(column_name, new Integer(width));
			if (cell_renderer != null) 
				column.setCellRenderer(cell_renderer);
		}
	}
	
	/**
	 * Create columns setting their preferred size according to the
	 * maximum width of the data to be displayed.
	 */
	/**
	 * @TODO 
	 * 1) non devo prendere column_len =
						Math.max(column_class_len, column_name.length());
					me piuttosto il massimo del width caratteri*fontmetrics
	   2) Credo che il padding sia aggiunto dopo l'icona, cmq devo mettere un padding che consideri anche l'icona
	 */
	protected void setColumnSizes2() {

		int width = 0;
		int columnIndex = 0;
		String column_name = null;
		TableColumn column = null;
		//re_initialise the default width hashmap as we are going to create a new JTable.
		defaultColWidth_ = new HashMap();
		/* Get the table font metrics to evaluate the cell max len */
		FontMetrics fmTable = jtable_.getFontMetrics(jtable_.getFont());
		/* Get the tableHeader font metrics to evaluate the cell max len */
		FontMetrics fmHeader = jtable_.getTableHeader().getFontMetrics(jtable_.getTableHeader().getFont());

		/* Set the column width (max value between the header label and
		 * the column value) 
		 */
		for (int i = 0; i < colCount_; i++) {

			column_name = model_.getColumnName(i);
			if (hideCols.containsKey(column_name))
				continue;

			/* Set the specific column renderer for DbbDataType columns */
			Class column_class = model_.getColumnClass(i);
			DbbDataTypeRenderer cell_renderer = null;
			int column_len = 0;

			/* Set the column width according to the specific
			     * DbbDataType (if properly defined) or to the column size
			     * retrieved from the model. Set the size to the column
			     * name if it is longer then the calculated width. 
			 */
			SortButtonRenderer headerButtonRenderer = getHeaderRenderer();
			if (DbbDataType.class.isAssignableFrom(column_class)) {
				cell_renderer = new DbbDataTypeRenderer(column_class);
				int column_class_len = 0;
				try {
                    
                    headerButtonRenderer.setText(column_name);
                    Dimension d = headerButtonRenderer.getPreferredSize();
					DbbDataType new_inst =
						(DbbDataType) column_class.newInstance();
					column_class_len = new_inst.getDataTypeLength();
					column_len = Math.max(column_class_len, column_name.length());
                    int char_len = fmTable.stringWidth("0");
                    width = char_len * column_len + PADDING_;
                    width=Math.max(width,d.width + PADDING_);
				} catch (Exception e) {
					stdlog_.warn("", e);
				}
			
			} else {
                headerButtonRenderer.setText(column_name);
                column_len = model_.getColumnSize(i);
                int char_len = fmTable.stringWidth("0");
                width = char_len * column_len + PADDING_ ;                
                Dimension d = headerButtonRenderer.getPreferredSize();
				width = Math.max(width,d.width + PADDING_);
			}

			//compare column_name size and the calculated width, and then take the bigger as new column_width
			/*FontMetrics fmHeader = jtable_.getTableHeader().getFontMetrics(jtable_.getTableHeader().getFont());
			
			if (column_name.length() > width)
				width = column_name.length();
			*/
			/* Create the column (with a specific width and header value) and
			 * add it to the table column model 
			 */
			column = new TableColumn(i, width);
			// Uncomment that to set minWidth for DbbTable Columns
			//column.setMinWidth(width);
			column.setHeaderValue(column_name);
			column_model.addColumn(column);
			defaultColWidth_.put(column_name, new Integer(width));
			if (cell_renderer != null) 
				column.setCellRenderer(cell_renderer);
		}
	}

	/**
	 * Set the column width according to the max size of the values to
	 * be displayed in the column itself.  
	 */
	protected void buildTableColumns() {
		/* Create my own table column model, don't use the table default */
		column_model = new DefaultTableColumnModel();

		/* Create the table columns and set their size. */
		setColumnSizes2();

		/* Add the newly created table column model to the table (override
		 * the default) 
		 */
		jtable_.setColumnModel(column_model);

		applyColRendererConf();
		
		/* Create a new table header and link it to the table */
		JTableHeader table_header = new JTableHeader(column_model);
		table_header.setTable(jtable_);
	//	jtable_.setTableHeader(table_header);
		

		int rows = jtable_.getRowCount();
		if (rows > 0) {
			// the selection during query caused wrong event firing
			//      jtable_.setRowSelectionInterval(0, 0);
			if (jtable_.getColumnCount() > 0) {
				jtable_.setColumnSelectionInterval(0, 0);
			}
		} else
			// Java1.4 port jtable_.setRowSelectionInterval(-1,-1);
			jtable_.clearSelection();
		
		TableColumnModel model = jtable_.getColumnModel();
		int colCount = model.getColumnCount();
		for (int i = 0; i < colCount; i++){
			TableColumn col = model.getColumn(i);
			col.setHeaderRenderer(getHeaderRenderer());
		}
	}

	//----------------------------------------
	// Public methods
	//----------------------------------------

	/* (non-Javadoc)
	 * @see org.eso.ohs.core.dbb.client.DbbTableInterface#getRows()
	 */
	public int getRows() {
		return rowCount_;
	}

	/* (non-Javadoc)
	 * @see org.eso.ohs.core.dbb.client.DbbTableInterface#getColumns()
	 */
	public int getColumns() {
		return colCount_;
	}
    
    public int getColIndex(String colName) {
        return sorter.getColIndex(colName);
    }

	/* (non-Javadoc)
	 * @see org.eso.ohs.core.dbb.client.DbbTableInterface#getDistinctCellCountAtColumn(int)
	 */
	public int getDistinctCellCountAtColumn(int column) {

		Vector v = new Vector();

		for (int i = 0; i < getRows(); i++) {
			Object o = getValueAt(i, column);
			if (!v.contains(o))
				v.addElement(o);
		}

		return v.size();
	}

	/* (non-Javadoc)
	 * @see org.eso.ohs.core.dbb.client.DbbTableInterface#Rows()
	 */
	public int[] getSelectedRows() {
		return jtable_.getSelectedRows();
	}

	/* (non-Javadoc)
	 * @see org.eso.ohs.core.dbb.client.DbbTableInterface#getSelectedRow()
	 */
	public int getSelectedRow() {
	//	return jtable_.getSelectedRow();
		return jtable_.getSelectedRow();
	}

	/* (non-Javadoc)
	 * @see org.eso.ohs.core.dbb.client.DbbTableInterface#getRowCount()
	 */
	public int getRowCount() {
		return jtable_.getRowCount();
	}

	/* (non-Javadoc)
	 * @see org.eso.ohs.core.dbb.client.DbbTableInterface#getTable()
	 */
	public JTable getTable() {
		return jtable_;
	}

	/* (non-Javadoc)
	 * @see org.eso.ohs.core.dbb.client.DbbTableInterface#getModel()
	 */
	public TableModel getModel() {
		return jtable_.getModel();
	}

	/* (non-Javadoc)
	 * @see org.eso.ohs.core.dbb.client.DbbTableInterface#getTableModel()
	 */
	public TableModel getTableModel() {
		return model_;
	}

	/* (non-Javadoc)
	 * @see org.eso.ohs.core.dbb.client.DbbTableInterface#setSorter(org.eso.ohs.core.gui.widgets.OhsTableSorter)
	 */
	public void setSorter(OhsTableSorter new_sorter) {
		sorter = new_sorter;
		sorter.setModel(model_);
		jtable_.setModel(sorter);
	}

	/* (non-Javadoc)
	 * @see org.eso.ohs.core.dbb.client.DbbTableInterface#enableHeaderSorting(boolean)
	 * @deprecated Use the class DbbTableHeaderListener.
	 * Example:
	 * 
	 */
	public void enableHeaderSorting(boolean b) {
		if (b)
			sorter.addMouseListenerToHeaderInTable(jtable_);
		else
			sorter.removeMouseListenerToHeaderInTable(jtable_);
	}

	/* (non-Javadoc)
	 * @see org.eso.ohs.core.dbb.client.DbbTableInterface#hideColumn(java.lang.String)
	 */
	public void hideColumn(String columnName) {
		hideCols.put(columnName, columnName);
	}

	/* (non-Javadoc)
	 * @see org.eso.ohs.core.dbb.client.DbbTableInterface#setSelectedRow(int)
	 */
	public void setSelectedRow(int row) {
		if (row < 0) {
			jtable_.clearSelection();
		} else {
			jtable_.setRowSelectionInterval(row, row);
		}
	}

	/* (non-Javadoc)
	 * @see org.eso.ohs.core.dbb.client.DbbTableInterface#deselectRow(int)
	 */
	public void deselectRow(int row) {
		jtable_.removeRowSelectionInterval(row, row);
	}

	/* (non-Javadoc)
	 * @see org.eso.ohs.core.dbb.client.DbbTableInterface#selectAllRows()
	 */
	public void selectAllRows() {
		jtable_.selectAll();
	}

	/* (non-Javadoc)
	 * @see org.eso.ohs.core.dbb.client.DbbTableInterface#clearSelection()
	 */
	public void clearSelection() {
		jtable_.clearSelection();
	}

	/* (non-Javadoc)
	 * @see org.eso.ohs.core.dbb.client.DbbTableInterface#setFont(java.awt.Font)
	 */
	public void setFont(Font newFont) {
		jtable_.setFont(newFont);
	}

	/* (non-Javadoc)
	 * @see org.eso.ohs.core.dbb.client.DbbTableInterface#setShowGrid(boolean)
	 */
	public void setShowGrid(boolean grid) {
		jtable_.setShowGrid(grid);
	}

	/* (non-Javadoc)
	 * @see org.eso.ohs.core.dbb.client.DbbTableInterface#setShowVerticalLines(boolean)
	 */
	public void setShowVerticalLines(boolean grid) {
		jtable_.setShowVerticalLines(grid);
	}
	/* (non-Javadoc)
	 * @see org.eso.ohs.core.dbb.client.DbbTableInterface#setShowHorizontalLines(boolean)
	 */
	public void setShowHorizontalLines(boolean grid) {
		jtable_.setShowHorizontalLines(grid);
	}

	/* (non-Javadoc)
	 * @see org.eso.ohs.core.dbb.client.DbbTableInterface#setGridColor(java.lang.String)
	 */
	public void setGridColor(String colorString) {
		Color color = getColor(colorString);
		jtable_.setGridColor(color);
	}

	/* (non-Javadoc)
	 * @see org.eso.ohs.core.dbb.client.DbbTableInterface#setBackground(java.lang.Class, java.lang.String)
	 */
	public void setBackground(Class renderClass, String colorString) {
		Color color = getColor(colorString);
		JComponent render =
			(JComponent) jtable_.getDefaultRenderer(renderClass);
		/* stdlog_.debug("set background " ); */
		if (render != null) {
			render.setBackground(color);
		}
		columnBgMap_.put(renderClass, color);
	}

	/* (non-Javadoc)
	 * @see org.eso.ohs.core.dbb.client.DbbTableInterface#setSelectionBackground(java.lang.String)
	 */
	public void setSelectionBackground(String colorString) {
		Color bgColor = getColor(colorString);
		jtable_.setSelectionBackground(bgColor);

	}

	/* (non-Javadoc)
	 * @see org.eso.ohs.core.dbb.client.DbbTableInterface#setBackground(java.lang.String)
	 */
	public void setBackground(String colorString) {
		Color bgColor = getColor(colorString);
		jtable_.setBackground(bgColor);
		((JComponent) jtable_.getDefaultRenderer(Boolean.class)).setBackground(
			bgColor);
		((JComponent) jtable_.getDefaultRenderer(Integer.class)).setBackground(
			bgColor);
		((JComponent) jtable_.getDefaultRenderer(Float.class)).setBackground(
			bgColor);

	}

	/* (non-Javadoc)
	 * @see org.eso.ohs.core.dbb.client.DbbTableInterface#setForeground(java.lang.String)
	 */
	public void setForeground(String colorString) {
		Color bgColor = getColor(colorString);
		jtable_.setForeground(bgColor);
		((JComponent) jtable_.getDefaultRenderer(Boolean.class)).setForeground(
			bgColor);
		((JComponent) jtable_.getDefaultRenderer(Integer.class)).setForeground(
			bgColor);
		((JComponent) jtable_.getDefaultRenderer(Float.class)).setForeground(
			bgColor);
	}

	/* (non-Javadoc)
	 * @see org.eso.ohs.core.dbb.client.DbbTableInterface#setSelectionForeground(java.lang.String)
	 */
	public void setSelectionForeground(String colorString) {
		Color bgColor = getColor(colorString);
		jtable_.setSelectionForeground(bgColor);
	}

	/**
	* applyes color to column renderer classes
	* requested in setBackground(Class, String)
	* or setForeground(Class,String)
	*/
	protected void applyColRendererConf() {
		if (columnFgMap_.size() > 0 || columnBgMap_.size() > 0) {
			TableColumnModel col_model = jtable_.getColumnModel();
			int colCount = col_model.getColumnCount();

			// foreground setting
			String column_name = null;

			for (int i = 0; i < colCount; i++) {
				column_name = jtable_.getColumnName(i);
				/* stdlog_.debug(" apply render to col: "+column_name); */
				
				if (hideCols.containsKey(column_name))
					continue;

				TableColumn col = col_model.getColumn(i);
				
				JComponent customRender = (JComponent) col.getCellRenderer();
				if (customRender != null) {
					Class rendererClass = null;
					// the class is rendered by DbbDataTypeRenderer
					if (DbbDataTypeRenderer
						.class
						.isAssignableFrom(customRender.getClass())) {
						rendererClass =
							((DbbDataTypeRenderer) customRender)
								.getDbbDataTypeRenderer();
					} else {
						rendererClass = customRender.getClass();
					}
					/* stdlog_.debug(" found renderer "+customRender.getClass()); */
					Color color = null;
					if ((color = (Color) columnFgMap_.get(rendererClass))
						!= null) {
						customRender.setForeground(color);
					}
					if ((color = (Color) columnBgMap_.get(rendererClass))
						!= null) {
						customRender.setBackground(color);
					}
				}
			}
		}
	}

	/* (non-Javadoc)
	 * @see org.eso.ohs.core.dbb.client.DbbTableInterface#setForeground(java.lang.Class, java.lang.String)
	 */
	public void setForeground(Class renderClass, String colorString) {
		Color color = getColor(colorString);
		JComponent render =
			(JComponent) jtable_.getDefaultRenderer(renderClass);
		/* stdlog_.debug("set foreground " ); */
		if (render != null) {
			render.setForeground(color);
		}
		columnFgMap_.put(renderClass, color);
	}
	/**
	 * Converts string into Color class
	 * @param colorString Possible colors: from Color class
	 * static variable. 
	 * Es: "white" "lightGray" "gray" 
	 * "darkGray" "black" "red" "pink"
	 * "orange"  "yellow"  "green" 
	 * "magenta" "cyan" "blue" 
	 */
	protected Color getColor(String colorString) {
		Color retColor = Color.white;
		try {
			Class colorClass = Class.forName("java.awt.Color");
			Field[] colors = colorClass.getFields();
			for (int i = 0; i < colors.length; i++) {
				if (colors[i].getType().getName().equals("java.awt.Color")) {
					if (colors[i].getName().equals(colorString)) {
						retColor = (Color) colors[i].get(null);
					}
				}
			}
		} catch (ClassNotFoundException e) {
			stdlog_.error("Class Color not found", e);
		} catch (IllegalAccessException e) {
			stdlog_.error("field \"" + colorString + "\" not accessible", e);
		}
		return retColor;
	}

	/* (non-Javadoc)
	 * @see org.eso.ohs.core.dbb.client.DbbTableInterface#getValueAt(int, int)
	 */
	public Object getValueAt(int row, int column) {
		return sorter.getValueAt(row, column);
	}

    /* (non-Javadoc)
     * @see org.eso.ohs.core.dbb.client.DbbTableInterface#getValueAt(int, java.lang.String)
     */
    public Object getValueAt(int row, String column_name) {
        return sorter.getValueAt(row, column_name);
    }

	/* (non-Javadoc)
	 * @see org.eso.ohs.core.dbb.client.DbbTableInterface#getValueFromJtableAt(int, int)
	 */
	public Object getValueFromJtableAt(int row, int column) {
		return jtable_.getValueAt(row, column);
	}

	/* (non-Javadoc)
	 * @see org.eso.ohs.core.dbb.client.DbbTableInterface#getColIdFromName(java.lang.String)
	 */
	public int getColIdFromName(String aColumn) {
		return sorter.getColIdFromName(aColumn);
	}

	/* (non-Javadoc)
	 * @see org.eso.ohs.core.dbb.client.DbbTableInterface#getScrollPanel()
	 */
	public JScrollPane getScrollPanel() {
		return scrollPanel_;
	}

	/* (non-Javadoc)
	 * @see org.eso.ohs.core.dbb.client.DbbTableInterface#setReorderingAllowed(boolean)
	 */
	public void setReorderingAllowed(boolean b) {
		JTableHeader table_header = jtable_.getTableHeader();
		table_header.setReorderingAllowed(b);
	}

	/* (non-Javadoc)
	 * @see org.eso.ohs.core.dbb.client.DbbTableInterface#addRows(org.eso.ohs.core.dbb.sql.DbbSqlQueryResult)
	 */
	public void addRows(DbbSqlQueryResult result) {

		/* getLongValues() is deprecated and does not return anything
		 * different from null anyway
		 longValues_ = result.getLongValues();
		    */

		/* Update the model with the new data */
		model_.addRows(result);
		rowCount_ = model_.getRowCount();
		colCount_ = model_.getColumnCount();

		/* Build the table columns only at the first iteration*/
		if (repaintTable_) {
			repaintTable_ = false;
			buildTableColumns();
		}

		sorter.tableChanged(new TableModelEvent(model_));
	}

	/* (non-Javadoc)
	 * @see org.eso.ohs.core.dbb.client.DbbTableInterface#addColumn(java.lang.String, java.lang.Object[], java.lang.Class)
	 */
	public void addColumn(String col_name, Object[] items, Class col_class) {
		/* Update the model with the new data */
		model_.addColumn(col_name, items, col_class);
		colCount_ = model_.getColumnCount();

		buildTableColumns();

		sorter.setModel(model_);
		sorter.tableChanged(new TableModelEvent(model_));
	}

	/* (non-Javadoc)
	 * @see org.eso.ohs.core.dbb.client.DbbTableInterface#addColumn(java.lang.String, java.lang.Object[], java.lang.Class, int)
	 */
	public void addColumn(
		String col_name,
		Object[] items,
		Class col_class,
		int pos) {

		/* Update the model with the new data */
		model_.addColumn(col_name, items, col_class, pos);
		colCount_ = model_.getColumnCount();

		buildTableColumns();

		sorter.setModel(model_);
		sorter.tableChanged(new TableModelEvent(model_));
	}

	/* (non-Javadoc)
	 * @see org.eso.ohs.core.dbb.client.DbbTableInterface#setColumn(java.lang.String, java.lang.Object[], java.lang.Class, int)
	 */
	public void setColumn(
		String col_name,
		Object[] items,
		Class col_class,
		int pos) {

		/* Update the model with the new data */
		model_.setColumn(col_name, items, col_class, pos);

		buildTableColumns();

		sorter.setModel(model_);
		sorter.tableChanged(new TableModelEvent(model_));
	}

	/* (non-Javadoc)
	 * @see org.eso.ohs.core.dbb.client.DbbTableInterface#insertRow(java.lang.Object[], int)
	 */
	public void insertRow(Object[] new_row, int pos) {
		/* Update the model with the new data */
		model_.insertRow(new_row, pos);
		rowCount_ = model_.getRowCount();

		sorter.setModel(model_);
		sorter.tableChanged(new TableModelEvent(model_));
	}

	/* (non-Javadoc)
	 * @see org.eso.ohs.core.dbb.client.DbbTableInterface#removeRows(int[])
	 */
	// Giorgio Passera 15-sept-2000
	public void removeRows(int[] rows) {

		

		/* Update the model with the new data */
		model_.removeRows(rows);
		rowCount_ = model_.getRowCount();
		colCount_ = model_.getColumnCount();

		stdlog_.debug("after rows " + rowCount_);
		//buildTableColumns();
	}

	/* (non-Javadoc)
	 * @see org.eso.ohs.core.dbb.client.DbbTableInterface#moveUpRows(int[], int)
	 */
	//Giorgio Passera 15-sep-2000
	public void moveUpRows(int[] rows, int insertPosition) {
		/* Update the model*/
		model_.moveUpRows(rows, insertPosition);
		if (rows[0] - 1 >= 0) {
			jtable_.setRowSelectionInterval(
				rows[0] - 1,
				rows[0] - 1 + rows.length - 1);
		}

		//buildTableColumns();
	}

	/* (non-Javadoc)
	 * @see org.eso.ohs.core.dbb.client.DbbTableInterface#moveUpRows(int[])
	 */
	public void moveUpRows(int[] rows) {
		/* Update the model*/
		model_.moveUpRows(rows);
		if (rows[0] - 1 >= 0) {
			jtable_.setRowSelectionInterval(
				rows[0] - 1,
				rows[0] - 1 + rows.length - 1);
		}

		//buildTableColumns();
	}

	/* (non-Javadoc)
	 * @see org.eso.ohs.core.dbb.client.DbbTableInterface#moveDownRows(int[], int)
	 */
	public void moveDownRows(int[] rows, int insertPosition) {
		model_.moveDownRows(rows, insertPosition);
	}

	/* (non-Javadoc)
	 * @see org.eso.ohs.core.dbb.client.DbbTableInterface#moveToTop(int[])
	 */
	public void moveToTop(int[] rows) {
		/* Update the model*/
		if (rows.length > 0) {
			model_.moveToTop(rows);
			jtable_.setRowSelectionInterval(0, 0);
		}
		//buildTableColumns();
	}

	/* (non-Javadoc)
	 * @see org.eso.ohs.core.dbb.client.DbbTableInterface#setGraphicProperties(org.eso.ohs.core.utilities.GTreeNode)
	 */
	public void setGraphicProperties(GTreeNode tableRootNode) {
		String valueString = null;
		boolean valueBool = false;
		String fontFace = null;
		int fontStyle = Font.PLAIN;
		int fontSize = defaultFontSize_;
		if (tableRootNode != null) {
			OHSConfig config = new OHSConfig(tableRootNode);

			stdlog_.debug(" Table custom properties:\n" + config.toString());
			valueBool = config.getBoolean("horizontal");
			stdlog_.debug("horizontal: " + valueBool);
			setShowHorizontalLines(valueBool);
			valueBool = config.getBoolean("vertical");
			stdlog_.debug("vertical: " + valueBool);
			setShowVerticalLines(valueBool);

			valueString = config.getValue("background");
			if (!valueString.equals("")) {
				stdlog_.debug("background: " + valueString);
				setBackground(valueString);
			}
			valueString = config.getValue("foreground");
			if (!valueString.equals("")) {
				stdlog_.debug("foreground: " + valueString);
				setForeground(valueString);
			}
			valueString = config.getValue("gridColor");
			if (!valueString.equals("")) {
				stdlog_.debug("gridColor: " + valueString);
				setGridColor(valueString);
			}
			valueString = config.getValue("activeForeground");
			if (!valueString.equals("")) {
				stdlog_.debug("activeForeground: " + valueString);
				setSelectionForeground(valueString);
			}
			valueString = config.getValue("activeBackground");
			if (!valueString.equals("")) {
				stdlog_.debug("activeBackground: " + valueString);
				setSelectionBackground(valueString);
			}
			fontFace = config.getValue("fontFace");
			if (!fontFace.equals("")) {
				stdlog_.debug("fontFace: " + fontFace);
				String fontStyleString = config.getValue("fontStyle");
				if (fontStyleString.equals("BOLD")) {
					fontStyle = Font.BOLD;
				} else if (fontStyleString.equals("ITALIC")) {
					fontStyle = Font.ITALIC;
				}
				stdlog_.debug("fontStyle: " + fontStyleString);
				fontSize = config.getInteger("fontSize", defaultFontSize_);
				stdlog_.debug("fontSize: " + fontSize);
				setFont(new Font(fontFace, fontStyle, fontSize));
			}
		}
	}

	/* (non-Javadoc)
	 * @see org.eso.ohs.core.dbb.client.DbbTableInterface#moveDownRows(int[])
	 */
	public void moveDownRows(int[] rows) {
		//Update the model
		model_.moveDownRows(rows);
		if (rows[rows.length - 1] < getRows() - 1) {
			int index1 = rows[rows.length - 1] + 2 - rows.length;
			int index2 = rows[rows.length - 1] + 1;
			jtable_.setRowSelectionInterval(index1, index2);
		}
	}

	/* (non-Javadoc)
	 * @see org.eso.ohs.core.dbb.client.DbbTableInterface#clear()
	 */
	public void clear() {
		//Java1.4 port
		jtable_.clearSelection();
		model_.clear();
		repaintTable_ = true;
	}

	/** Sort the data according to the values set in the sorting panel (it is not the columnHeader sorting!)
	 * @see org.eso.ohs.core.dbb.client.DbbTableInterface#sort(java.util.Vector, java.util.Vector)
	 */
	public void sort(Vector column_vector, Vector criteria_vector) {

		int column_len = column_vector.size();
		int criteria_len = criteria_vector.size();

		String column_name = null;
		String model_column_name = null;

		Boolean criterion = null;
		Boolean ascending_criterion = new Boolean(true);

		Vector col_indexes = new Vector();
		Vector boolean_values = new Vector();

		/* Check if the column to be sorted are contained in the table. */
		for (int i = 0; i < column_len; i++) {

			column_name = (String) column_vector.elementAt(i);

			if (i < criteria_len)
				criterion = (Boolean) criteria_vector.elementAt(i);
			else
				criterion = ascending_criterion;

			/* Take into account only the columns that match the columns in
			 * the model and discharge the others.
			 */
			for (int k = 0; k < colCount_; k++) {

				/* Get the column name from the model. */
				model_column_name = model_.getColumnName(k);

				/* stdlog_.debug("Comparing <" + column_name +
				   "> with <" + model_column_name + ">"); */

				/* If the column name matchs one of the table columns, then
				 * add the column and its sorting criterion. 
				 */
				if (column_name.equals(model_column_name)) {
					col_indexes.addElement(new Integer(k));
					boolean_values.addElement(criterion);
					break;
				}
			}
		}

		/* Start sorting. */
		sorter.sortByColumns(col_indexes, boolean_values);
	}

	/* (non-Javadoc)
	 * @see org.eso.ohs.core.dbb.client.DbbTableInterface#printTable()
	 */
	public String printTable() {

		long start_time = System.currentTimeMillis();

		String separator = " ";
		PrintFormat format[] = new PrintFormat[colCount_];
		String header_str = "";

		/* Get the table font metrics to evaluate the cell max len */
		FontMetrics fm = jtable_.getFontMetrics(jtable_.getFont());
		int char_len = fm.stringWidth("0");

		/* Get the column class and the column width */
		for (int i = 0; i < jtable_.getColumnCount(); i++) {
			/* Get the column class from the model */
			Class column_class = jtable_.getColumnClass(i);
			String column_name = jtable_.getColumnName(i);
			String class_name = column_class.getName();

			boolean left = true;

			if (class_name.equals((Integer.class).getName())
				|| class_name.equals((Long.class).getName())
				|| class_name.equals((Double.class).getName())
				|| class_name.equals((Float.class).getName())) {

				left = false;
			}

			/* Get the column width from the column itself */
			TableColumn col = column_model.getColumn(i);
			int col_width = col.getWidth();
		
			int numChars = ((col_width - PADDING_) / char_len);
			
			// + 1 needed to avoid "rounding" problem introducted by the operator /
			// + 1 needed to avoid some fonts metric problems /
			if (numChars > 0)
				numChars = numChars + 2;

			// 	    format[i] = new PrintfFormat(fmt_pattern + width + class_pattern);
			// 	    header_str = header_str + 
			// 		format[i].sprintf(column_name) + separator;

			format[i] = new PrintFormat(numChars, left);

			header_str =
				header_str
					+ new PrintFormat(numChars).sprintf(column_name)
					+ separator;
		}

		StringBuffer header_line = new StringBuffer();
		for (int i = 0; i < header_str.length(); i++) {
			header_line.append("=");
		}

		StringBuffer ret = new StringBuffer();
		ret.append(header_str).append("\n").append(header_line).append("\n");

		//itera su tutte le righe
		for (int i = 0; i < rowCount_; i++) {
			StringBuffer row_str = new StringBuffer();
			
			//itera su tutte le colonne
			for (int k = 0; k < jtable_.getColumnCount(); k++) {

				Object obj_value = jtable_.getValueAt(i, k);

				String v = "";

				if (obj_value != null) {
					/* Try to get the data converted by the renderer */
					try {
						TableCellRenderer r = jtable_.getCellRenderer(i, k);
						Component cell =
							r.getTableCellRendererComponent(
								jtable_,
								obj_value,
								true,
								true,
								i,
								k);
						v = ((JLabel) cell).getText();
					} catch (ClassCastException e) {
						v = jtable_.getValueAt(i, k).toString();
					}
				}
				try {
					row_str.append(format[k].sprintf(v)).append(separator);
				} catch (Exception e) {
					row_str.append(v).append(separator);
				}
			}

			ret.append(row_str).append("\n");
		}

		long end_time = System.currentTimeMillis();
		long diff_time = end_time - start_time;

		stdlog_.debug("Print execution time: " + diff_time + " ms");
		SimpleDateFormat date_format = new SimpleDateFormat("HH:mm:ss.SSS");
		stdlog_.debug(
			"Print execution time: "
				+ date_format.format(new Date(diff_time - 12600000)));
		return ret.toString();
	}
    /* (non-Javadoc)
	 * @see org.eso.ohs.core.dbb.client.DbbTableInterface#getDefaultFontSize()
	 */
    public  int getDefaultFontSize() {
        return defaultFontSize_;
    }
    /* (non-Javadoc)
	 * @see org.eso.ohs.core.dbb.client.DbbTableInterface#setDefaultFontSize(int)
	 */
    public void setDefaultFontSize(int defaultFontSize) {
        defaultFontSize_ = defaultFontSize;
    }
	public void setHeaderRenderer(SortButtonRenderer renderer) {
		this.headerRenderer_ = renderer;
	}
	public SortButtonRenderer getHeaderRenderer() {
		return headerRenderer_;
	}
	public int getDefaultColumnWidth(String columnName) {
		int result = ((Integer) defaultColWidth_.get(columnName)).intValue(); 
		return result;
	}
	public void setDefaultColumnWidth(String columnName, int width) {
		 defaultColWidth_.put(columnName, new Integer(width)); 	
	}
}

//----------------------------------------
// Renderers and Accessories
//----------------------------------------

/**
 * DbbDataType renderer: created in order to display objects
 * implementing the DbbDataType interface.  
 */
class DbbDataTypeRenderer extends DefaultTableCellRenderer {

	private static Logger stdlog_ = Logger.getLogger(DbbDataTypeRenderer.class);
	protected Class cls_;

	public DbbDataTypeRenderer(Class cls) {
		cls_ = cls;
	}

	public Class getDbbDataTypeRenderer() {
		return cls_;
	}

	public Component getTableCellRendererComponent(
		JTable table,
		Object value,
		boolean isSelected,
		boolean isFocused,
		int row,
		int column) {

		Component c =
			super.getTableCellRendererComponent(
				table,
				value,
				isSelected,
				isFocused,
				row,
				column);

		try {
			if (value != null) {
				DbbDataType obj = (DbbDataType) cls_.newInstance();
				String val = obj.fromDatabaseData(value);
				((JLabel) c).setText(val);
			}
		} catch (Exception e) {
			stdlog_.warn("", e);
		}

		return c;
	}
}

/**
 * Integer renderer: created in order to display integers not in a
 * "comma" format (11200 instead of 11,200).
 */
class DbbIntegerRenderer extends DefaultTableCellRenderer {

	public DbbIntegerRenderer() {
		super();
		setHorizontalAlignment(JLabel.RIGHT);
	}

	public Component getTableCellRendererComponent(
		JTable table,
		Object value,
		boolean isSelected,
		boolean isFocused,
		int row,
		int column) {

		Component c =
			super.getTableCellRendererComponent(
				table,
				value,
				isSelected,
				isFocused,
				row,
				column);

		if (value != null && value instanceof Integer) {
			((JLabel) c).setText(value.toString());
		}

		return c;
	}
}

/**
 * Integer renderer: created in order to display integers not in a
 * "comma" format (11200 instead of 11,200).
 */
class DbbFloatRenderer extends DefaultTableCellRenderer {

	public DbbFloatRenderer() {
		super();
		setHorizontalAlignment(JLabel.RIGHT);
	}

	public Component getTableCellRendererComponent(
		JTable table,
		Object value,
		boolean isSelected,
		boolean isFocused,
		int row,
		int column) {

		Component c =
			super.getTableCellRendererComponent(
				table,
				value,
				isSelected,
				isFocused,
				row,
				column);

		if (value != null && value instanceof Float) {
			((JLabel) c).setText(value.toString());
		}

		return c;
	}
}

/**
 * Boolean renderer: display booleans using a checkbox. Convert any
 * value (integer or string) to a boolean value before displaying it.  
 */
class DbbBooleanRenderer extends JCheckBox implements TableCellRenderer {

	Color fg_ = null;
	Color bg_ = null;
	public DbbBooleanRenderer() {
		super();
		setHorizontalAlignment(JLabel.CENTER);
	}

	public void setBackground(Color color) {
		super.setBackground(color);
		bg_ = color;
	}

	public void setForeground(Color color) {
		super.setForeground(color);
		fg_ = color;
	}

	/** Convert a given object to a Boolean before displaying it. */
	private boolean getValue(Object value) {

		boolean val = false;

		// Integer value: (0, -1) "false" or 1 "true"      
		if (value != null) {

			if (value instanceof Integer) {

				Integer i = (Integer) value;

				switch (i.intValue()) {
					case 0 :
					case -1 :
						val = false;
						break;
					case 1 :
						val = true;
						break;
				}
			}

			// String value: "false" or "true"      
			if (value instanceof String) {

				String s = (String) value;
				char ch = s.trim().charAt(0);

				switch (ch) {
					case 'f' :
						val = false;
						break;
					case 't' :
						val = true;
						break;
				}
			}
		}

		return val;
	}

	/** Implement the interface method. */
	public Component getTableCellRendererComponent(
		JTable table,
		Object value,
		boolean isSelected,
		boolean hasFocus,
		int row,
		int column) {
		if (bg_ == null) {
			bg_ = table.getBackground();
		}
		if (fg_ == null) {
			fg_ = table.getForeground();
		}
		if (isSelected) {
			setForeground(table.getSelectionForeground());
			super.setBackground(table.getSelectionBackground());
		} else {
			setForeground(fg_);
			setBackground(bg_);
		}

		setSelected(getValue(value));
		return this;
	}
}

/**
 * User Interface delegate, defined in order to handle the table
 * focus events.
 */
class DbbTableUI extends BasicTableUI {
	DbbTableUI() {
		super();
	}

	protected FocusListener createFocusListener() {
		return new DbbFocusHandler();
	}
	public class DbbFocusHandler extends FocusHandler {
		public void focusGained(FocusEvent e) {
		}
		public void focusLost(FocusEvent e) {
			super.focusLost(e);
		}
	}
}

/**
 * A very simple class to format strings.
 */
class PrintFormat {

	int width_ = 0;
	boolean left_ = true;

	// Static and final to speed up the formatting mechanism
	private static final String[] b =
		{
			"",
			" ",
			"  ",
			"   ",
			"    ",
			"     ",
			"      ",
			"       ",
			"        ",
			"         ",
			"          ",
			"           ",
			"            ",
			"             ",
			"              ",
			"               ",
			"                ",
			"                 ",
			"                  ",
			"                   ",
			"                    " };

	public PrintFormat(int width) {
		width_ = width;
	}
	
	public PrintFormat(int width, boolean align_left) {
		width_ = width;
		left_ = align_left;
	}

	public String sprintf(String s) {
		String ret = "";
		String blanks = "";

		if (s == null) s = "";
		int len = width_ - s.length();

		// bug fixed {=}  and performance benefit 
		if (len < 0){
		//If the string lenght is greater than the column width 
		// then truncate the string.
			if (width_ != 1)
				s = s.substring(0, width_);	
			else
				s = "";
			
		}else if (len >= b.length) {
	//			If the string lenght is lower than the column width then AND.
	// if we do not have have precalculated padding for it.
				StringBuffer blanksBuff = new StringBuffer();
				blanksBuff.append(b[b.length - 1]);
				for (int i = b.length - 1; i < len; i++)
					blanksBuff.append(" ");
				blanks = blanksBuff.toString();
			} else{ 
				// we have a precalculated padding, let's use it.
				blanks = b[len];
			}
		
		if (left_)
			ret = s + blanks;
		else
			ret = blanks + s;

		return ret;
	}
}
	//added to manage sorting icons over header columns
/**
 * This class provide a listener over a JTable Header.
 * Left-clicking a column Header sort the column (ascending/descending)
 * Clicking any other mouse button will set the column width to closed/defaultWidth (if it is defaultWidth then close else set to defaultWidth)
 *  
 * @author gdonino
 *
 */
 class HeaderListener extends MouseAdapter {
	    JTableHeader   header;
	    SortButtonRenderer renderer;
		DbbTable dbbTable;
		final int closedWidth = 15;
		//if columnJustResized is not null then set the defaultWidth of the column when the mouse button is released
		private TableColumn columnJustResized = null;
	  
	    HeaderListener(JTableHeader header,SortButtonRenderer renderer, DbbTable dbbTable) {
	      this.header   = header;
	      this.renderer = renderer;
		  this.dbbTable = dbbTable;
	    }
	    public void mousePressed(MouseEvent e){
	    	JTableHeader tableHeader = ((JTableHeader)e.getSource());
	    	columnJustResized = tableHeader.getResizingColumn();
	    }
	    
	    public void mouseClicked(MouseEvent e) {
	    	JTableHeader tableHeader = ((JTableHeader)e.getSource());
			JTable table = tableHeader.getTable();
/****DbC****/if (table == null) throw new IllegalArgumentException(MsgManager.errPrecondition("I cannot find a valid JTable. The attribute table is null."));
			
            TableColumnModel colModel = table.getColumnModel();			
/****DbC****/if (colModel == null) throw new IllegalArgumentException(MsgManager.errPrecondition("I cannot find a valid JTableModel. The attribute tableModel is null."));
			
            // The index of the column whose header was clicked
            int vColIndex = colModel.getColumnIndexAtX(e.getX());
            int mColIndex = table.convertColumnIndexToModel(vColIndex);
    
            // Return if not clicked on any column header
            if (vColIndex == -1) {
                return;
            }
    
            // Determine if mouse was clicked between column heads
            Rectangle headerRect = table.getTableHeader().getHeaderRect(vColIndex);
            if (vColIndex == 0) {
                headerRect.width -= 3;    // Hard-coded constant
            } else {
                headerRect.grow(-3, 0);   // Hard-coded constant
            }
            if (headerRect.contains(e.getX(), e.getY())) {
				int col = header.columnAtPoint(e.getPoint());
				int tableModelColIndex = header.getTable().convertColumnIndexToModel(col);
				if (e.getButton()==1){
//					It's a left-button click on the column: let's sort!
					
					String colName = header.getTable().getColumnName(col);
					renderer.setPressedColumn(colName, col);
					renderer.setSelectedColumn(colName, col);
					boolean isAscent;
					if (SortButtonRenderer.DOWN == renderer.getState(colName, col)) {
						isAscent = true;
					} else {
						isAscent = false;
					} 
					((TableSorter)header.getTable().getModel()).sortByColumn(tableModelColIndex, isAscent); 
				}else{
//					it's not a left-button click: then let's hide/show the column
					TableColumn currentColumn = colModel.getColumn(col);
					String columnName = header.getTable().getColumnName(col);
					int defaultWidth = dbbTable.getDefaultColumnWidth(columnName);
							
					if (currentColumn.getWidth() == defaultWidth){
						currentColumn.setPreferredWidth(closedWidth);
					}else{
						currentColumn.setPreferredWidth(defaultWidth);
					}
				}
            }
			/**
			 * Uncomment the else statement if you want to handle 
			 * the buttonclick between 2 columns 
			 **/
			/*else{
                //Mouse was clicked between column heads
                // vColIndex is the column head closest to the click
                // vLeftColIndex is the column head to the left of the click
                columnJustResized = tableHeader.getResizingColumn();
            }*/
			header.repaint();
			if (header.getTable().isEditing()) {
				header.getTable().getCellEditor().stopCellEditing();
	    } 
	   }
	  
	    public void mouseReleased(MouseEvent e) {
			//Did anybody resize a column?? if yes, then we have to change the defaulwidth of the column
			if (columnJustResized != null){ 
			    String columnName = (String) columnJustResized.getHeaderValue();
			    int width = columnJustResized.getWidth();
			    dbbTable.setDefaultColumnWidth(columnName, width);
			}
	    	
	    	
	    	int col = header.columnAtPoint(e.getPoint());  	
			renderer.setPressedColumn(header.getTable().getColumnName(col), -1);                // clear
		    header.repaint();
	    }
 	}
 

 
