package org.eso.ohs.core.dbb.client;

// $Id: DbbTextField.java,v 1.2 2005/06/10 09:42:27 gdonino Exp $

import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

import javax.swing.JTextField;

import org.eso.ohs.core.dbb.sql.DbbSqlChunk;

/**  
 * This is an implementation of the DbbWidget abstract class,
 * defining a text field widget.
 * 
 * <br>
 * <br>
 * @author Carlo Boarotto, ESO 20-Jul-1999
 * <br>ChangeLog: cmavros 08-Oct-2001 Add a default 
 *   column size to the DbbTextField.
 * <p>
 * 
 */

public class DbbTextField extends DbbWidget implements KeyListener {

    /** Deafult width for text widgets */
    public final static int COLUMNSIZE = 15;
	
	protected boolean isCaseSensitive = false;

    //----------------------------------------
    // Constructors
    //----------------------------------------

    /** Create a case insensitive DbbTextField object.
     * 
     * @param labeltext	Label text to be displayed.
     * @param chunk	DbbSqlChunk object
     * @param width	Width of the widget in characters
     */
    // amchavan, 13-Jun-2002: added
    public DbbTextField( String labeltext, DbbSqlChunk chunk, int width ) {

	super(labeltext, chunk);
	setDefault( "" );

	widget_ = new JTextField( width );
	label_.setLabelFor(widget_);
	((JTextField) widget_).addKeyListener(this);
    }

	/** Create a DbbTextField object.
     * 
     * @param labeltext	Label text to be displayed.
     * @param chunk	DbbSqlChunk object
     * @param width	Width of the widget in characters
     * @param isCaseSensitive a Boolean
     */
	 public DbbTextField( String labeltext, DbbSqlChunk chunk, int width, boolean isCaseSensitive ) {
		this(labeltext, chunk, width);
		setCaseSensitive(chunk, isCaseSensitive);
	  }
    /** Create a case insensitive DbbTextField object with a default width.
     * 
     * @param labeltext	Label text to be displayed.
     * @param chunk	DbbSqlChunk object
     * @see #COLUMNSIZE
     */
    public DbbTextField(String labeltext, DbbSqlChunk chunk) {

	this( labeltext, chunk, COLUMNSIZE );
    }
	 /** Create a DbbTextField object with a default width.
     * 
     * @param labeltext	Label text to be displayed.
     * @param chunk	DbbSqlChunk object
     * @param isCaseSensitive a Boolean
     * @see #COLUMNSIZE
     */
	public DbbTextField(String labeltext, DbbSqlChunk chunk, boolean isCaseSensitive) {
		 this( labeltext, chunk, COLUMNSIZE, isCaseSensitive );
	}




    //----------------------------------------
    // Public Methods
    //----------------------------------------

    /** 
     * Get the widget value.
     *
     * @return The currently displayed string value. 
     */
    public String get() {
		String result = ((JTextField) widget_).getText();
		if (!isCaseSensitive){
			result = result.toLowerCase();
		}	
		return result ;
    }

    /** 
     * Set the displayed string item.
     * @param Integer value to be displayed.
     */
    public void set(Object val) {
	((JTextField) widget_).setText(String.valueOf(val));
	fireDbbWidgetAction();
    }

    /**
     * Reset the widget to its default state
     */
    public void reset () {
	if(widget_.isEnabled())
	    set( "" );
	else 
	    set(((JTextField)widget_).getText());
    }

    /** 
     * Set the widget value parameters.
     * @param s String value to be displayed.
     */
    public void set(String s) {
	((JTextField) widget_).setText(s);
	fireDbbWidgetAction();
    }

    /**
     * Method to check if the widget contents have been cleared.
     *
     * @return A boolean value: <code>true</code> if it is not empty,
     *         <code>false</code> otherwise.  
     */
    public boolean isEmpty() {
	String sel = get();
	return (sel.equals( "" ));
    }

	protected void setCaseSensitive(DbbSqlChunk chunk, boolean isCaseSensitive){
		this.isCaseSensitive = isCaseSensitive;
		chunk.setCaseSensitive(isCaseSensitive);
	}

    //----------------------------------------
    // Event methods
    //----------------------------------------

    /** Listener interface implementation. Not used.*/
    public void keyPressed(KeyEvent ev) {
    }

    /** Listener interface implementation. Not used.*/
    public void keyTyped(KeyEvent ev) {
    }

    /**
     * Listener interface implementation.
     * @param ev	 Event triggering the action.
     */
    public void keyReleased(KeyEvent ev) {
	fireDbbWidgetAction();
    }
}

