package org.eso.ohs.core.dbb.server;

// $Id: DbbSqlEngine.java,v 1.1 2004/02/17 11:16:24 tcanavan Exp $

import java.net.MalformedURLException;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.sql.SQLException;

import org.eso.ohs.core.dbb.sql.DbbSqlChunk;
import org.eso.ohs.core.dbb.sql.DbbSqlQueryChunks;
import org.eso.ohs.core.dbb.sql.DbbSqlTable;

/**   
 * This is the main engine of a database browser: builds SQL
 * statements, performs queries (within threads), returns result data.
 *
 * @author Carlo Boarotto, ESO 19-Jul-1999
 * <br>ChangeLog: A M Chavan, ESO 26-Jul-1999
 *
 */
public interface DbbSqlEngine {


    /**
     * Create and start a thread to get the result set and to process it
     * into an Object array.  
     *
     * @param sql	Sql statement.
     * @exception SQLException JDBC exception occurs
     * @exception RemoteException RMI exception occurs
     * @exception MalformedURLException malformed url i.e. servername//port
     * @exception NotBoundException client could not bind to server
     */
    public DbbQuery startQuery(DbbSqlQueryChunks chunks)
	throws RemoteException, SQLException,
	MalformedURLException, NotBoundException;


    /**
     * Number of records in a result set   
     *
     * @param sql	Sql statement.
     * @exception SQLException JDBC exception occurs
     * @exception RemoteException RMI exception occurs
     * @exception MalformedURLException malformed url i.e. servername//port
     * @exception NotBoundException client could not bind to server
     */
    public int execSqlCount(DbbSqlQueryChunks chunks)
	throws SQLException, RemoteException,
	MalformedURLException, NotBoundException;


    /**
     * Add default colums to a particular chunk
     *
     * @param sql	Sql statement.
     * @exception SQLException JDBC exception occurs
     * @exception RemoteException RMI exception occurs
     * @exception MalformedURLException malformed url i.e. servername//port
     * @exception NotBoundException client could not bind to server
     */
    public void addDefaultColumnsChunk(DbbSqlChunk chunk)
	throws SQLException, RemoteException,
	MalformedURLException, NotBoundException;


    public DbbSqlChunk[] getDefaultColumnsChunk()
	throws SQLException, RemoteException,
	MalformedURLException, NotBoundException;

    /**
     * Add a preferential join condition to be considered when building
     * automatically the query.
     *
     * @param from_tbl	DbbSqlTable indicating the primary key table.
     * @param from_col	String indicating the primary key table column.
     * @param to_tbl	DbbSqlTable indicating the foreign key table.
     * @param to_col	String indicating the foreign key table column.
     *
     * @exception SQLException JDBC exception occurs
     * @exception RemoteException RMI exception occurs
     * @exception MalformedURLException malformed url i.e. servername//port
     * @exception NotBoundException client could not bind to server
     */
    public void addJoinCondition(DbbSqlTable from_tbl,
				 String from_col,
				 DbbSqlTable to_tbl,
				 String to_col)
	throws SQLException, RemoteException,
	MalformedURLException, NotBoundException;
	
    /**
     * Specify a left outer join condition to be considered when building
     * automatically the query.
     *
     * @param from_tbl  DbbSqlTable indicating the primary key table.
     * @param from_col  String indicating the primary key table column.
     * @param to_tbl    DbbSqlTable indicating the foreign key table.
     * @param to_col    String indicating the foreign key table column.
     *
     * @exception SQLException JDBC exception occurs
     * @exception RemoteException RMI exception occurs
     * @exception MalformedURLException malformed url i.e. servername//port
     * @exception NotBoundException client could not bind to server
     */
    public void addCompositeJoinCondition(DbbSqlTable from_tbl,
                 String from_col[],
                 DbbSqlTable to_tbl,
                 String to_col[], boolean leftOuterJoin)
    throws SQLException, RemoteException,
    MalformedURLException, NotBoundException;
}


