package org.eso.ohs.core.gui.models;

import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;

/**
 * A Document model to only permit the input of an epoch.
 * 
 * <p>
 * $Id: SeeingModel.java,v 1.1 2004/02/17 10:35:21 tcanavan Exp $
 * @author cmavros 29-10-01 Add insertString()
 */

public class SeeingModel extends InputMaskModel {

    private final double SEEING_MAX = 2;
    private final double SEEING_MIN = 0.2;

    /**
     * The regular expression describing legal buffer values.
     * Looking for values between .2 and 2.0
     */
    private final static String legalValuesPattern_ =
	"\\d{0,1}(\\.\\d{0,1})?";

    /**
     * Constructor for the model.
     */
    public SeeingModel() {
	super(legalValuesPattern_);
    }

    /**
     * Refuse input that is above or below the limits of the
     * textfield.
     */
    public void insertString(int offset, String str, AttributeSet a)
      throws BadLocationException {

        //stdlog_.debug("String: <"+str+">");
        super.insertString(offset, str, a);
        //stdlog_.debug("buffer: <"+buffer_+">");
        try {
            String buffer_val = buffer_.toString();
            //Do not allow a single . character
            if (buffer_val.equals("."))
                remove(offset, str.length());
            else{
                double f = Double.valueOf(buffer_val).doubleValue();
		if(f > SEEING_MAX) 
                    remove(offset, str.length());
            }
            if (buffer_val.length() > 2) {
                double f = Double.valueOf(buffer_val).doubleValue();
		if(f < SEEING_MIN) 
                    remove(offset, str.length());

            }

        } catch (NumberFormatException e) {
            // nothing, the string is not complete...
        }
    }

}

