package org.eso.ohs.core.gui.utilities;

import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Toolkit;

import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;

import org.apache.log4j.Logger;

/**
 * This class contains a series of helper utility methods for Swing.
 */

public class Utilities {

    /** Standard log channel */
    private static Logger stdlog_ = Logger.getLogger(Utilities.class);

    /*
         * set window dimensions to smaller than screen size
         *
         * @param frame     window to resize
         * @return new screen dimension object
         */
    public static Dimension setScreenSize(JFrame frame) {
        frame.pack();
        Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();

        Dimension window = frame.getPreferredSize();
        int maxWinHeight = (int) (screen.height * 0.9);
        int maxWinWidth = (int) (screen.width * 0.9);
        if (window.height > maxWinHeight) {
            window.height = maxWinHeight;
        }
        if (window.width > maxWinWidth) {
            window.width = maxWinWidth;
        }
        frame.setSize(window);
        return window;
    }

    /**
     * Displat a normal cursor.
     * @param frm The frame associated with the cursor
     *            This is used to set the glasspane to false so it can except
     *            events
     */

    public static void normalCursor(JFrame frm) {
        JComponent glassPane = (JComponent) frm.getGlassPane();
        glassPane.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
        glassPane.setVisible(false);
    }

    /**
     * Display the wait Cursor
     * @param frm The frame sets the cursor. This is used to set the glasspane
     *            so it can reject any events from the user
     */

    public static void waitCursor(JFrame frm) {
        JComponent glassPane = (JComponent) frm.getGlassPane();
        glassPane.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        glassPane.setVisible(true);
    }

    /**
     * Because Swing believes that there is only One platform that uses 
     * the Motif L&F then the following code has to be inserted for Linux and
     * HP-UX.
     */

    public static void setLookAndFeel() {
        try {
            String laf = UIManager.getSystemLookAndFeelClassName();

            if (System.getProperty("os.name").equals("HP-UX")) {
                laf = "com.sun.java.swing.plaf.motif.MotifLookAndFeel";
            }
            if (System.getProperty("os.name").equals("Linux")) {
                laf = "com.sun.java.swing.plaf.motif.MotifLookAndFeel";
            }

            UIManager.setLookAndFeel(laf);
        } catch (UnsupportedLookAndFeelException e) {
            stdlog_.debug("Exception: " + e);
        } catch (ClassNotFoundException e) {
            stdlog_.debug("Exception: " + e);
        } catch (InstantiationException e) {
            stdlog_.debug("Exception: " + e);
        } catch (IllegalAccessException e) {
            stdlog_.debug("Exception: " + e);
        }

    }



    public static boolean isHeadless() {
         
         // If we can get a valid Toolkit we are not headless
         //--------------------------------------------------
         try {
             Toolkit t = Toolkit.getDefaultToolkit();
             return false;
         }
         catch( Throwable e ) {
             // No Toolikit, we're decapitated
             //-------------------------------
             return true;
         }

    }
}