package org.eso.ohs.core.gui.widgets;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.border.BevelBorder;
import javax.swing.border.Border;

import org.apache.log4j.Logger;


/**
 * Excepts the user id and password and returns a flag to
 * the caller to determine what action has taken place.
 * <p>
 * $Id: SimpleInputDialog.java,v 1.4 2005/05/30 10:00:50 gdonino Exp $
 * <br>
 * @author Tim Canavan
 * <br>ChangeLog: Bob Kemp
 * <br>ChangeLog: cmavros 18-10-01 Add method for defining the dialog differently
 * from the default layout.
 */

public class SimpleInputDialog extends JDialog {

    /** Standard log channel */
    private static Logger stdlog_ =
        Logger.getLogger(SimpleInputDialog.class);

    /**
     * Field returnValue_
     */
    private int returnValue_ = JOptionPane.CANCEL_OPTION;

    /**
     * Field cancelButton_
     */
    protected JButton cancelButton_;

    /**
     * Field okButton_
     */
    protected JButton okButton_;

    /**
     * Field frame_
     */
    private Frame frame_;

    /**
     * Field initialFocusComponent_
     */
    private Component initialFocusComponent_;

    /** Panel layout parameters */
    public final static int NORTH = 0;
    /**
     * Field WEST
     */
    public final static int WEST = 1;
    /**
     * Field EAST
     */
    public final static int EAST = 2;
    /**
     * Field SOUTH
     */
    public final static int SOUTH = 3;
    /**
     * Field buttonPosition_
     */
    private int buttonPosition_ = SOUTH;
    /**
     * Field buttonRows_
     */
    private int buttonRows_ = 1;
    /**
     * Field buttonCols_
     */
    private int buttonCols_ = 0;
    /**
     * Field setBlank_
     */
    private boolean setBlank_ = true;

    /**
     * Constructor for SimpleInputDialog
     * @param frm Frame
     * @param title String
     * @param message String
     * @param labels String[]
     * @param comps Component[]
     */
    public SimpleInputDialog(
        Frame frm,
        String title,
        String message,
        String[] labels,
        final Component[] comps) {
        this(frm, title);

        initialise(message, labels, comps);
    }
	
	/**
     * Constructor for SimpleInputDialog
     * @param frm Frame
     * @param title String
     * @param message String
     * @param labels String[]
     * @param comps Component[]
     * @param jButtons JButton[] An array of custom JButtons
     */
    public SimpleInputDialog(
        Frame frm,
        String title,
        String message,
        String[] labels,
        final Component[] comps, final JButton[] jButtons) {
        this(frm, title);

        initialise(message, labels, comps, jButtons);
    }


	/**
     * Constructor for SimpleInputDialog
     * @param frm Frame
     * @param title String
     * @param message String
     * @param labels String[]
     * @param comps Component[]
     * @param cancel JButton external cancel button 
     * @param ok JButton external ok button
     */
    public SimpleInputDialog(
        Frame frm,
        String title,
        String message,
        String[] labels,
        final Component[] comps,
        JButton cancel,
        JButton ok) {
        this(frm, title);
        
        initialise(message, labels, comps, cancel, ok);
    }

	/**
	 * Constructor for SimpleInputDialog. This is used for informative purposes, only
	 * @param frm Frame
	 * @param title String
	 * @param message String
	 * @param labels String[]
	 * @param comps Component[]
	 * @param ok JButton 
	 */
	public SimpleInputDialog(
		Frame frm,
		String title,
		String message,
		String[] labels,
		final Component[] comps,
		JButton ok) {
		this(frm, title);
        
		initialise(message, labels, comps, ok);
	}

    /**
     * Constructor for SimpleInputDialog
     * @param frm Frame
     * @param title String
     * @param message String
     */
    public SimpleInputDialog(Frame frm, String title, String message) {

        this(frm, title);
        initialise(message, new String[] {
        }, new Component[] {
        });
    }

    /**
     * Basic Constructor.  When using this constructor,
     * it is the caller's responsibility to also call the initialise
     * method before making the window visible!
     * @param frm Frame
     * @param title String
     */
    protected SimpleInputDialog(Frame frm, String title) {

        super(frm, title, true);
        frame_ = frm;
    }

    /**
     * initialize method used by users that want to change 
     * the default dialog layout.
     * <br>ChangeLog: cmavros 18-10-01
     * @param message
     * @param labels
     * @param comps
     * @param bPosition - location of the buttonBox in the panel
     * @param rows - number of rows in panel
     * @param cols - number of cols in panel
     */
    protected void initialise(
        String message,
        String[] labels,
        final Component[] comps,
        int bPosition,
        int rows,
        int cols) {

        buttonPosition_ = bPosition;
        buttonRows_ = rows;
        buttonCols_ = cols;
        setBlank_ = false;
        initialise(message, labels, comps);

    }

    /**
     * Method initialise
     * @param message String
     * @param labels String[]
     * @param comps Component[]
     */
    protected void initialise(
            String message,
            String[] labels,
            final Component[] comps) {
            
        JLabel lab;
        JPanel panel = new JPanel();
        JPanel componentPanel = new JPanel();
        JPanel leftPanel = new JPanel();
        JPanel rightPanel = new JPanel();
        JTextArea messageArea = null;
        cancelButton_ = new JButton("Cancel");
        okButton_ = new JButton("OK");

        initialFocusComponent_ = (comps.length > 0) ? comps[0] : okButton_;

        /*
         * Commented out for Java 1.4 port - may need to be replaced with a 
         * FocusTraversalPolicy object.
         *
         * cancelButton_.setNextFocusableComponent(initialFocusComponent_);
         */

        panel.setLayout(new BorderLayout());
        leftPanel.setLayout(new GridLayout(0, 1));
        rightPanel.setLayout(new GridLayout(0, 1));
        componentPanel.setLayout(new GridLayout(buttonRows_, buttonCols_));

        if (message != null) {
            messageArea = new JTextArea(message) {
                /*
                 * Java 1.1.8 code
                public boolean isFocusTraversable() {
                    return false;
                }
                 */
                public boolean isFocusable() {
                    return false;
                }
                /*
                 * Java 1.1.8 code              
                public boolean isManagingFocus() {
                    return false;
                }
                 */
            };

            messageArea.setEditable(false);
            messageArea.setOpaque(false);

            messageArea.setFont(new JLabel().getFont());

            panel.add(messageArea, BorderLayout.NORTH);
        }

        panel.add(componentPanel, BorderLayout.CENTER);

        FocusLnr lnr = new FocusLnr();

        okButton_.addFocusListener(lnr);
        cancelButton_.addFocusListener(lnr);

        if (labels == null || labels.length != comps.length) {
            for (int j = 0; j < comps.length; j++) {
                rightPanel.add(comps[j]);
            }
        } else {
            componentPanel.add(leftPanel);
            for (int j = 0; j < comps.length; j++) {
                lab = new JLabel(labels[j]);
                lab.setHorizontalAlignment(JLabel.RIGHT);
                leftPanel.add(lab);
                rightPanel.add(comps[j]);
            }
        }

        componentPanel.add(rightPanel);

        // leftPanel.setBorder(
        // BorderFactory.createEmptyBorder(10, 10, 10, 10)
        // );

        // rightPanel.setBorder(
        // BorderFactory.createEmptyBorder(10, 10, 10, 10)
        // );

        panel.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));

        JPanel buttonBox = new JPanel(new GridLayout(buttonRows_, buttonCols_));
        //panel.add(buttonBox, BorderLayout.SOUTH);
        if (buttonPosition_ == NORTH)
            panel.add(buttonBox, BorderLayout.NORTH);
        else if (buttonPosition_ == WEST)
            panel.add(buttonBox, BorderLayout.WEST);
        else if (buttonPosition_ == EAST)
            panel.add(buttonBox, BorderLayout.EAST);
        else
            panel.add(buttonBox, BorderLayout.SOUTH);

        buttonBox.add(okButton_);
        if (setBlank_)
            buttonBox.add(new JLabel(" "));
        buttonBox.add(cancelButton_);

        ActionListener okClicker = new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                okButton_.doClick(150);
            }
        };

        for (int j = 0; j < comps.length; j++) {
            if (comps[j] instanceof JTextField) {
                ((JTextField) comps[j]).addActionListener(okClicker);
            }
        }

        okButton_.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                returnValue_ = JOptionPane.OK_OPTION;
                setVisible(false);
            }
        });
        cancelButton_.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                returnValue_ = JOptionPane.CANCEL_OPTION;
                setVisible(false);
            }
        });

        panel.registerKeyboardAction(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                okButton_.doClick(150);
            }
        },
            KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0),
            JComponent.WHEN_IN_FOCUSED_WINDOW);

        panel.registerKeyboardAction(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                cancelButton_.doClick(150);
            }
        },
            KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0),
            JComponent.WHEN_IN_FOCUSED_WINDOW);

        setContentPane(panel);
        pack();

        if (frame_ != null) {
            setLocationRelativeTo(frame_);
        } else {
            Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();

            setLocation(
                (screen.width - getSize().width) / 2,
                (screen.height - getSize().height) / 2);
        }

    }
	 /**
     * Method initialise
     * @param message String
     * @param labels String[]
     * @param comps Component[]
     */
    protected void initialise(
            String message,
            String[] labels,
            final Component[] comps, final JButton[] jButtons) {
            
        JLabel lab;
        JPanel panel = new JPanel();
        JPanel componentPanel = new JPanel();
        JPanel leftPanel = new JPanel();
        JPanel rightPanel = new JPanel();
        JTextArea messageArea = null;
        cancelButton_ = new JButton("Cancel");
        okButton_ = new JButton("OK");

        initialFocusComponent_ = (comps.length > 0) ? comps[0] : okButton_;

        /*
         * Commented out for Java 1.4 port - may need to be replaced with a 
         * FocusTraversalPolicy object.
         *
         * cancelButton_.setNextFocusableComponent(initialFocusComponent_);
         */

        panel.setLayout(new BorderLayout());
        leftPanel.setLayout(new GridLayout(0, 1));
        rightPanel.setLayout(new GridLayout(0, 1));
        componentPanel.setLayout(new GridLayout(buttonRows_, buttonCols_));

        if (message != null) {
            messageArea = new JTextArea(message) {
                /*
                 * Java 1.1.8 code
                public boolean isFocusTraversable() {
                    return false;
                }
                 */
                public boolean isFocusable() {
                    return false;
                }
                /*
                 * Java 1.1.8 code              
                public boolean isManagingFocus() {
                    return false;
                }
                 */
            };

            messageArea.setEditable(false);
            messageArea.setOpaque(false);

            messageArea.setFont(new JLabel().getFont());

            panel.add(messageArea, BorderLayout.NORTH);
        }

        panel.add(componentPanel, BorderLayout.CENTER);

        FocusLnr lnr = new FocusLnr();

        okButton_.addFocusListener(lnr);
        cancelButton_.addFocusListener(lnr);

        if (labels == null || labels.length != comps.length) {
            for (int j = 0; j < comps.length; j++) {
                rightPanel.add(comps[j]);
            }
        } else {
            componentPanel.add(leftPanel);
            for (int j = 0; j < comps.length; j++) {
                lab = new JLabel(labels[j]);
                lab.setHorizontalAlignment(JLabel.RIGHT);
                leftPanel.add(lab);
                rightPanel.add(comps[j]);
            }
        }

        componentPanel.add(rightPanel);

        // leftPanel.setBorder(
        // BorderFactory.createEmptyBorder(10, 10, 10, 10)
        // );

        // rightPanel.setBorder(
        // BorderFactory.createEmptyBorder(10, 10, 10, 10)
        // );

        panel.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));

        JPanel buttonBox = new JPanel(new GridLayout(buttonRows_, buttonCols_));
        //panel.add(buttonBox, BorderLayout.SOUTH);
        if (buttonPosition_ == NORTH)
            panel.add(buttonBox, BorderLayout.NORTH);
        else if (buttonPosition_ == WEST)
            panel.add(buttonBox, BorderLayout.WEST);
        else if (buttonPosition_ == EAST)
            panel.add(buttonBox, BorderLayout.EAST);
        else
            panel.add(buttonBox, BorderLayout.SOUTH);

        buttonBox.add(okButton_);
        if (setBlank_)
            buttonBox.add(new JLabel(" "));
		/**
		 * code to add customs buttons
		 */
		for (int i = 0; i < jButtons.length; i++){
			buttonBox.add(jButtons[i]);
		}
        buttonBox.add(cancelButton_);

        ActionListener okClicker = new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                okButton_.doClick(150);
            }
        };

        for (int j = 0; j < comps.length; j++) {
            if (comps[j] instanceof JTextField) {
                ((JTextField) comps[j]).addActionListener(okClicker);
            }
        }

        okButton_.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                returnValue_ = JOptionPane.OK_OPTION;
                setVisible(false);
            }
        });
        cancelButton_.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                returnValue_ = JOptionPane.CANCEL_OPTION;
                setVisible(false);
            }
        });

        panel.registerKeyboardAction(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                okButton_.doClick(150);
            }
        },
            KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0),
            JComponent.WHEN_IN_FOCUSED_WINDOW);

        panel.registerKeyboardAction(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                cancelButton_.doClick(150);
            }
        },
            KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0),
            JComponent.WHEN_IN_FOCUSED_WINDOW);

        setContentPane(panel);
        pack();

        if (frame_ != null) {
            setLocationRelativeTo(frame_);
        } else {
            Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();

            setLocation(
                (screen.width - getSize().width) / 2,
                (screen.height - getSize().height) / 2);
        }

    }
    /**
     * Method initialise
     * @param message String
     * @param labels String[]
     * @param comps Component[]
     * @param cl JButton
     * @param ok JButton
     */
    protected void initialise(
        String message,
        String[] labels,
        final Component[] comps,
        JButton cl,
        JButton ok) {
        JLabel lab;
        JPanel panel = new JPanel();
        JPanel componentPanel = new JPanel();
        JPanel leftPanel = new JPanel();
        JPanel rightPanel = new JPanel();
        JTextArea messageArea = null;
        cancelButton_ = cl;
        okButton_ = ok;

        initialFocusComponent_ = (comps.length > 0) ? comps[0] : okButton_;
        /*
         * Commented out for Java 1.4 port - may need to be replaced with a 
         * FocusTraversalPolicy object.
         * 
         * cancelButton_.setNextFocusableComponent(initialFocusComponent_);
         */
        panel.setLayout(new BorderLayout());
        leftPanel.setLayout(new GridLayout(0, 1));
        rightPanel.setLayout(new GridLayout(0, 1));
        componentPanel.setLayout(new GridLayout(buttonRows_, buttonCols_));

        if (message != null) {
            messageArea = new JTextArea(message) {
                /*
                 * Commented out for Java 1.4 port
                 * 
                public boolean isFocusTraversable() {
                    return false;
                }

                public boolean isManagingFocus() {
                    return false;
                }
                 */
                public boolean isFocusable() {
                    return false;
                }
            };
            messageArea.setEditable(false);
            messageArea.setOpaque(false);

            messageArea.setFont(new JLabel().getFont());

            panel.add(messageArea, BorderLayout.NORTH);
        }

        panel.add(componentPanel, BorderLayout.CENTER);

        FocusLnr lnr = new FocusLnr();

        okButton_.addFocusListener(lnr);
        cancelButton_.addFocusListener(lnr);

        if (labels == null || labels.length != comps.length) {
            for (int j = 0; j < comps.length; j++) {
                rightPanel.add(comps[j]);
            }
        } else {
            componentPanel.add(leftPanel);
            for (int j = 0; j < comps.length; j++) {
                lab = new JLabel(labels[j]);
                lab.setHorizontalAlignment(JLabel.RIGHT);
                leftPanel.add(lab);
                rightPanel.add(comps[j]);
            }
        }

        componentPanel.add(rightPanel);

        // leftPanel.setBorder(
        // BorderFactory.createEmptyBorder(10, 10, 10, 10)
        // );

        // rightPanel.setBorder(
        // BorderFactory.createEmptyBorder(10, 10, 10, 10)
        // );

        panel.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));

        ActionListener okClicker = new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                okButton_.doClick(150);
            }
        };

        for (int j = 0; j < comps.length; j++) {
            if (comps[j] instanceof JTextField) {
                ((JTextField) comps[j]).addActionListener(okClicker);
            }
        }

        okButton_.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                returnValue_ = JOptionPane.OK_OPTION;
                setVisible(false);
            }
        });
        cancelButton_.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                returnValue_ = JOptionPane.CANCEL_OPTION;
                setVisible(false);
            }
        });

        panel.registerKeyboardAction(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                okButton_.doClick(150);
            }
        },
            KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0),
            JComponent.WHEN_IN_FOCUSED_WINDOW);

        panel.registerKeyboardAction(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                cancelButton_.doClick(150);
            }
        },
            KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0),
            JComponent.WHEN_IN_FOCUSED_WINDOW);

        setContentPane(panel);
        pack();

        if (frame_ != null) {
            setLocationRelativeTo(frame_);
        } else {
            Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();

            setLocation(
                (screen.width - getSize().width) / 2,
                (screen.height - getSize().height) / 2);
        }

    }
	/**
	 * Method initialise
	 * @param message String
	 * @param labels String[]
	 * @param comps Component[]
	 * @param ok JButton
	 */
	protected void initialise(
		 String message,
		 String[] labels,
		 final Component[] comps,
		 JButton ok) {
		 JLabel lab;
		 JPanel panel = new JPanel();
		 JPanel componentPanel = new JPanel();
		 JPanel leftPanel = new JPanel();
		 JPanel rightPanel = new JPanel();
		 JTextArea messageArea = null;
		 okButton_ = ok;

		 initialFocusComponent_ = (comps.length > 0) ? comps[0] : okButton_;
		 /*
		  * Commented out for Java 1.4 port - may need to be replaced with a 
		  * FocusTraversalPolicy object.
		  * 
		  * cancelButton_.setNextFocusableComponent(initialFocusComponent_);
		  */
		 panel.setLayout(new BorderLayout());
		 leftPanel.setLayout(new GridLayout(0, 1));
		 rightPanel.setLayout(new GridLayout(0, 1));
		 componentPanel.setLayout(new GridLayout(buttonRows_, buttonCols_));

		 if (message != null) {
			 messageArea = new JTextArea(message) {
				 /*
				  * Commented out for Java 1.4 port
				  * 
				 public boolean isFocusTraversable() {
					 return false;
				 }

				 public boolean isManagingFocus() {
					 return false;
				 }
				  */
				 public boolean isFocusable() {
					 return false;
				 }
			 };
			 messageArea.setEditable(false);
			 messageArea.setOpaque(false);

			 messageArea.setFont(new JLabel().getFont());

			 panel.add(messageArea, BorderLayout.NORTH);
		 }

		 panel.add(componentPanel, BorderLayout.CENTER);

		 FocusLnr lnr = new FocusLnr();

		 okButton_.addFocusListener(lnr);

		 if (labels == null || labels.length != comps.length) {
			 for (int j = 0; j < comps.length; j++) {
				 rightPanel.add(comps[j]);
			 }
		 } else {
			 componentPanel.add(leftPanel);
			 for (int j = 0; j < comps.length; j++) {
				 lab = new JLabel(labels[j]);
				 lab.setHorizontalAlignment(JLabel.RIGHT);
				 leftPanel.add(lab);
				 rightPanel.add(comps[j]);
			 }
		 }

		 componentPanel.add(rightPanel);

		 // leftPanel.setBorder(
		 // BorderFactory.createEmptyBorder(10, 10, 10, 10)
		 // );

		 // rightPanel.setBorder(
		 // BorderFactory.createEmptyBorder(10, 10, 10, 10)
		 // );

		 panel.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));

		 JPanel buttonBox = new JPanel(new GridLayout(buttonRows_, buttonCols_));
		 //panel.add(buttonBox, BorderLayout.SOUTH);
		 if (buttonPosition_ == NORTH)
			 panel.add(buttonBox, BorderLayout.NORTH);
		 else if (buttonPosition_ == WEST)
			 panel.add(buttonBox, BorderLayout.WEST);
		 else if (buttonPosition_ == EAST)
			 panel.add(buttonBox, BorderLayout.EAST);
		 else
			 panel.add(buttonBox, BorderLayout.SOUTH);

		 buttonBox.add(okButton_);
		 ActionListener okClicker = new ActionListener() {
			 public void actionPerformed(ActionEvent evt) {
				 okButton_.doClick(150);
			 }
		 };

		 for (int j = 0; j < comps.length; j++) {
			 if (comps[j] instanceof JTextField) {
				 ((JTextField) comps[j]).addActionListener(okClicker);
			 }
		 }

		 okButton_.addActionListener(new ActionListener() {
			 public void actionPerformed(ActionEvent evt) {
				 returnValue_ = JOptionPane.OK_OPTION;
				 setVisible(false);
			 }
		 });
		

		 panel.registerKeyboardAction(new ActionListener() {
			 public void actionPerformed(ActionEvent evt) {
				 okButton_.doClick(150);
			 }
		 },
			 KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0),
			 JComponent.WHEN_IN_FOCUSED_WINDOW);


		 setContentPane(panel);
		 pack();

		 if (frame_ != null) {
			 setLocationRelativeTo(frame_);
		 } else {
			 Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();

			 setLocation(
				 (screen.width - getSize().width) / 2,
				 (screen.height - getSize().height) / 2);
		 }

	 }
    /**
     * Method requestInitialFocus
     */
    protected void requestInitialFocus() {
        initialFocusComponent_.requestFocus();
    }

    /**
     * Set the focus on the user name when the dialog is first displayed
     */
    public void addNotify() {
        super.addNotify();
        requestInitialFocus();
    }

    /**
     */
    public static class FocusLnr implements FocusListener {
        /**
         * Field border
         */
        Border border;

        /**
         * Method focusGained
         * @param ev FocusEvent
         * @see java.awt.event.FocusListener#focusGained(FocusEvent)
         */
        public void focusGained(FocusEvent ev) {
            if (ev.getSource() instanceof JButton) {
                JButton btn = (JButton) ev.getSource();

                border = btn.getBorder();

                btn.setBorder(
                    BorderFactory.createBevelBorder(BevelBorder.RAISED));
            }
        }

        /**
         * Method focusLost
         * @param ev FocusEvent
         * @see java.awt.event.FocusListener#focusLost(FocusEvent)
         */
        public void focusLost(FocusEvent ev) {
            if (ev.getSource() instanceof JButton) {
                JButton btn = (JButton) ev.getSource();

                btn.setBorder(border);
            }
        }

    }

    /**
     * Get the result of the dialog, ie ok, cancel, etc.
     * @return int
     */
    public int getUserChoice() {
        return returnValue_;
    }

    /**
     * Method main
     * @param args String[]
     */
    public static void main(String[] args) {
        SimpleInputDialog dlg =
            new SimpleInputDialog(
                null,
                "TITLE",
                "Welcome",
                new String[] { "Username:", "Password:" },
                new Component[] { new JLabel("fred"), new JTextField()});

        dlg.show();
		
        stdlog_.debug("return value = " + dlg.getUserChoice());
        System.exit(0);

    }

}
