package org.eso.ohs.core.gui.widgets;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Toolkit;

import javax.swing.BorderFactory;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;

/**
 * Dialog used to display status (text) or progressive work.
 * The work must be performed in another (non-AWT) thread, and
 * the status is displayed in this dialog by calls to setMessage().
 */

public class StatusDialog extends JDialog {

	/**
	 * Current status displayed
	 */
	private String currentMessage_ = "";

	private JLabel statusLabel_;

	public StatusDialog(Frame frm, String msg) {
		super(frm);

		JPanel content = (JPanel) getContentPane();

		content.setLayout(new BorderLayout());

		currentMessage_ = msg;
		statusLabel_ = new JLabel(msg);

		content.add(statusLabel_, BorderLayout.CENTER);

		content.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));

		pack();

        if (frm != null) {
            setLocationRelativeTo(frm);
        } else {
            Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();

            setLocation(
                (screen.width - getSize().width) / 2,
                (screen.height - getSize().height) / 2);
        }
	}

	/**
	 * Set the status displayed in the dialog.
	 */
	public void setMessage(String msg) {
        currentMessage_ = msg;
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                statusLabel_.setText(currentMessage_);
            }
        });
	}

	/**
	 * @return the current message
	 */
	public String getMessage() {
		return currentMessage_;
	}
}