/* $Id: SchedRun.java,v 1.22 2005/03/30 15:42:47 tcanavan Exp $ */

package org.eso.ohs.dfs;

import java.math.BigDecimal;
import java.util.Date;

import org.eso.ohs.core.utilities.DateContainer;
import org.eso.ohs.core.utilities.DateConverter;

/**
 * Observing Programme business object class. <br>
 * 
 * @author D Dorigo,
 */
public class SchedRun extends BusinessObject {
	public static final int SEEING_DEFAULT = 2000;
    private int period_ = 0;

    private long tableId_ = 0;

    private String progId_ = "";

    private String obsMode_ = "";

    private String telescope_ = "";

    private long fromDate_ = 0;

    private long toDate_ = 0;

    private float nights = 0;

    private String piCoi_ = "";

    private String affil_ = "";

    private String instrument_ = "";

    private String remarks_ = "";

    private String title_ = "";

    private String observer_ = "";

    private String observerEmail_ = "";

    private long runId_ = 0;

    private String nightAssistant_ = "";

    private String supportAssistant_ = "";

    public static final String ANY_MONTH = "any";

    private String month_ = ANY_MONTH;

    private String moon_ = "";

    private int seeing_ = SEEING_DEFAULT;

    private int rank_;

    private String nightFract_ = "";

    private String rankClass_ = "";

    private String lastMod_ = "";

    //private boolean published_ = false;
    private int published_ = 0;

    //definition of NULL ipVersion
    private BigDecimal ipversion_ = null;

    private ObsRun obsRun_;
    public static final int PUBLIC_FLAG = 1; 
    public static final int PRIVATE_FLAG = 0; 

    //----------------------------------------
    // Constructor
    //----------------------------------------

    /**
     * Create a SchedRun and set its id.
     */
    public SchedRun() {
        super();
    }

    //----------------------------------------
    // Public Methods
    //----------------------------------------

    /**
     * Set new value for property
     * 
     * @param newValue
     *            the new value for the property
     */
    public void setTableId(long newValue) {
        tableId_ = newValue;
        //firePropertyChangeEvent();
    }

   
    /**
     * Get value for property
     * 
     * @return the current value of the property
     */
    public long getTableId() {
        return tableId_;
    }

    public int getPeriod() {
        return period_;
    };

    public void setPeriod(int period) {
        period_ = period;
    };

    public String getProgId() {
        return progId_;
    }

    public void setProgId(String progId) {
        progId_ = progId;
    }

    public String getObsMode() {
        return obsMode_;
    }

    public void setObsMode(String obsMode) {
        obsMode_ = obsMode;
    }

    public String getTelescope() {
        return telescope_;
    }

    public void setTelescope(String telescope) {
        telescope_ = telescope;
    }

    public long getFromDate() {
        return fromDate_;
    }

    public String getFromDateString() {
        return DateConverter.longToString(getFromDate());
    }

    public Date getFromDateObj() {
        Date dateObj = DateConverter.longToDate(getFromDate());
        return dateObj;
    }

    public Date getToDateObj() {
        Date dateObj = DateConverter.longToDate(getToDate());
        return dateObj;
    }

    public DateContainer getFromDateFields() {
        DateContainer date = DateContainer.convertDate(getFromDateObj());
        return date;
    }

    public DateContainer getToDateFields() {
        DateContainer date = DateContainer.convertDate(getToDateObj());
        return date;
    }

    public void setFromDate(long fromDate) {
        fromDate_ = fromDate;
    }

    public long getToDate() {
        return toDate_;
    }

    public String getToDateString() {
        return DateConverter.longToString(getToDate());
    }

    public void setToDate(long toDate) {
        toDate_ = toDate;
    }

    public float getNight() {
        return nights;
    }

    public void setNight(float night) {
        nights = night;
    }

    public String getPiCoi() {
        return piCoi_;
    }

    public void setPiCoi(String piCoi) {
        piCoi_ = piCoi;
    }

    public String getAffil() {
        return affil_;
    }

    public void setAffil(String affil) {
        affil = trim(affil);
        affil_ = affil;
    }

    public String getInstrument() {
        return instrument_;
    }

    public void setInstrument(String instrument) {
        instrument_ = instrument;
    }

    public String getRemarks() {
        return remarks_;
    }

    public void setRemarks(String remarks) {
        remarks = trim(remarks);
        remarks_ = remarks;
    }

    public String getTitle() {
        return title_;
    }

    public void setTitle(String title) {
        title_ = title;
    }

    public String getObserver() {
        return observer_;
    }

    public void setObserver(String observer) {
        observer = trim(observer);
        observer_ = observer;
    }

    public void setObserverEmail(String observerEmail) {
        observerEmail = trim(observerEmail);
        observerEmail_ = observerEmail;
    }

    /**
     * @param data
     * @return String
     */
    private String trim(String data) {
        if (data != null){
            data = data.trim();
        }
        return data;
    }

    public String getObserverEmail() {
        return observerEmail_;
    }

    public void setRunId(long runId) {
        runId_ = runId;
    }

    public long getRunId() {
        return runId_;
    }

    public void setNightAssistant(String nightAssistant) {
        nightAssistant = trim(nightAssistant);
        nightAssistant_ = nightAssistant;
    }

    public String getNightAssistant() {
        return nightAssistant_;
    }

    public String getSupportAssistant() {
        return supportAssistant_;
    }

    public void setSupportAssistant(String supportAssistant) {
        supportAssistant = trim(supportAssistant);
        supportAssistant_ = supportAssistant;
    }

    public String getMonth() {
        return month_;
    }

    public void setMonth(String month) {
        month_ = month;
    }

    public String getMoon() {
        return moon_;
    }

    public void setMoon(String moon) {
        moon_ = moon;
    }

    public int getSeeing() {
        return seeing_;
    }

    public void setSeeing(int seeing) {
        seeing_ = seeing;
    }

    public int getRank() {
        return rank_;
    }

    public void setRank(int rank) {
        rank_ = rank;
    }

    public String getNightFract_() {
        return nightFract_;
    }

    public void setNightFract_(String nightFract) {
        nightFract_ = nightFract;
    }

    public String getRankClass() {
        return rankClass_;
    }

    public void setRankClass(String rankClass) {
        rankClass_ = rankClass;
    }

    public String getLastMod() {
        return lastMod_;
    }

    public void setLastMod(String lastMod) {
        lastMod_ = lastMod;
    }

    //public boolean getPublished() {
    public int getPublished() {
        return published_;
    }

    //public void setPublished(boolean published ) {
    public void setPublished(int published) {
        published_ = published;
    }

    public BigDecimal getIpversion() {
        return ipversion_;
    }

    //public void setIpversion(String ipversion ) {
    public void setIpversion(BigDecimal ipversion) {
        ipversion_ = ipversion;
    }

    public Object accept(BusinessVisitor v, Object arg) {
        return null;
    }

    /**
     *  
     */
    public boolean isPublished() {
        boolean result = false;
        if (getPublished() == 0) {
            result = false;
        } else
            result = true;
        return result;
    }

    /**
     * @see org.eso.ohs.dfs.StorableObject#postProecess()
     */
    public void postProecess() {
    }

    /**
     * @return SchedRun clone obj
     */
    public SchedRun cloneObj() {
        SchedRun schedRun = new SchedRun();
        schedRun.setAffil(getAffil());
        schedRun.setFromDate(getFromDate());
        schedRun.setInstrument(getInstrument());
        schedRun.setIpversion(getIpversion());
        schedRun.setLastMod(getLastMod());
        schedRun.setMonth(getMonth());
        schedRun.setMoon(getMoon());
        schedRun.setNight(getNight());
        schedRun.setNightAssistant(getNightAssistant());
        schedRun.setNightFract_(getNightFract_());
        schedRun.setObserver(getObserver());
        schedRun.setObserverEmail(getObserverEmail());
        schedRun.setObsMode(getObsMode());
        schedRun.setPeriod(getPeriod());
        schedRun.setPiCoi(getPiCoi());
        schedRun.setProgId(getProgId());
        schedRun.setPublished(getPublished());
        schedRun.setRank(getRank());
        schedRun.setRankClass(getRankClass());
        schedRun.setRemarks(getRemarks());
        schedRun.setRunId(getRunId());
        schedRun.setSeeing(getSeeing());
        schedRun.setSupportAssistant(getSupportAssistant());
        schedRun.setTelescope(getTelescope());
        schedRun.setTitle(getTitle());
        schedRun.setToDate(getToDate());
        return schedRun;
    }

    /**
     * @return String
     */
    public String getSeeingString() {
        String seeingString = null;
        int seeing = getSeeing();
        if (seeing > 0) {
            float f = (float) seeing / 1000;
            Float fFloat = new Float(f);
            seeingString = fFloat.toString();
        }
        return seeingString;
    }

    public void setSeeingString(String seeingString) {
        int seeing = -1;
        if (seeingString != null && !seeingString.equals("")) {
            Float seeingFloat = new Float(seeingString);
            float f = seeingFloat.floatValue() * 1000;
            seeing = (int) f;
        }
        setSeeing(seeing);
    }
}