package org.eso.ohs.instruments;

import java.util.StringTokenizer;

/**
 * Instanciates a paramater class of type String. Verifies string input
 * in the GUI to make sure a string is within the specified range
 * <p>
 * $Id: StringParameter.java,v 1.4 2001/07/25 12:15:12 tcanavan Exp $
 * @author Tim Canavan            . 
 */



public class StringParameter extends Parameter {

    static final long serialVersionUID = 6422239514628897650L;

    /**
     * Constructor for String Parameter
     * @param parameterName parameter name
     */
   
    public StringParameter(ParamMetaData pMeta) {
        super(pMeta);
    }

    public char getDBTypeCode() {
        return 'S';
    }
    /**
     * Get tooltip text
     * @return tooltip text
     */

    public String getToolTipText() {
        String range = getRange();
        if(range != null &&
           range.length() > 0) {
            return range;
        }
        else 
            return "Enter a string";      
    }


    /**
     * Verify a valid string  has been entered.
     * @param value String value to be verified
     * @return If value is a valid number and in the
     *         range defined by the paramater class
     *         return OK
     *         The range of valid values T,F
     */

    public boolean verifyValue(String value) {
        String defaultValue = getDefaultValue();
        if(value.equals("NODEFAULT")) {
            return false;
        }
        if(value.equals(defaultValue))
            return true;
        String allowedValues = getRange();
        if(allowedValues == null || allowedValues.length() == 0)
            return true;
        StringTokenizer stringToken = new StringTokenizer(allowedValues," ");
        while(stringToken.hasMoreElements()) {
             String tkValues = stringToken.nextToken();
             int index = tkValues.indexOf('-');
             if(index!=-1) {
                 String min = tkValues.substring(0,index);
                 String max = tkValues.substring(index+1,tkValues.length());
                 if(value.compareTo(min)>0 && value.compareTo(max) < 0)
                     return true;
             }
             if(tkValues.equals(value)==true)
                  return true;
        }
        return false;
    }

   

}











