package org.eso.ohs.persistence;

import java.util.EventObject;

/**
 * This class provides extra information about the creation, change or
 * deletion of a directory.
 * 
 * <p>
 * $Id: DirectoryEvent.java,v 1.4 2003/07/03 08:49:49 ddorigo Exp $
 */
public class DirectoryEvent extends EventObject {

    /**
     * Constant to idenfity a directory created event
     */
    public static final int DIRECTORY_CREATED = 1;

    /**
     * Constant to identify a directory move event
     */
    public static final int DIRECTORY_MOVED = 2;

    /**
     * Constant to identify a directory renamed event
     */
    public static final int DIRECTORY_CHANGED = 3;

    /**
     * Constant to identify a directory renamed event
     */
    public static final int DIRECTORY_REMOVED = 4;

    /**
     * The type of event whether it is CREATED, CHANGED or REMOVED
     */
    private int type_;

    /**
     * The device where the directory is being created.This is 
     * usually Local Storeage
     */
    private Media device_;

    /**    
     * The directory that is created or deleted.
     */
    private DirectoryNode dir_;

    /**    
     * The old parent from which the directory was moved.
     */
    private DirectoryNode oldParent_;

    /**
     * A constructor for the directory event.
     * 
     * @param type Type of event that is generated
     * @param device The device               
     * @param dir The directory that the action is to take place on.
     */
    public DirectoryEvent(int type, Media device, DirectoryNode dir) {
	this(type, device, dir, null);
    }

    /**
     * A constructor for the directory event.
     * 
     * @param type Type of event that is generated
     * @param device The device               
     * @param dir The directory that the action is to take place on.
     */
    public DirectoryEvent(int type, Media device,
			    DirectoryNode dir, DirectoryNode oldParent) {
	super(dir);

	type_       = type;
	device_     = device;
	dir_	    = dir;
	oldParent_  = oldParent;
    }

    /**
     * Returns an integer which represents the different types
     * for a particular event
     * 
     * @return	the type of event, eg DIRECTORY_CREATED.
     */
    public int getType() {
	return type_;
    }

    /**
     * Returns the media the directory is stored on.
     * @return The media the directory is strored on. This is usually 
     *         local storeage      
     */
    public Media getMedia() {
	return device_;
    }

    /**
     * The directory that will be created, moved or removed. 
     * @return directory
     */
    public DirectoryNode getDirectoryNode() {
	return dir_;
    }

    /**
     * The old parent from which the directory was moved.
     * @return directory
     */
    public DirectoryNode getParent() {
	return oldParent_;
    }

}

