package org.eso.ohs.persistence;

import java.lang.reflect.Method;

import org.apache.log4j.Logger;
import org.eso.ohs.core.docview.datatrans.AdaptorFactory;
import org.eso.ohs.core.docview.datatrans.ObjAdaptor;
import org.eso.ohs.core.docview.datatrans.ObjectAdaptor;
import org.eso.ohs.dfs.BusinessObject;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.dfs.Mask;
import org.eso.ohs.dfs.MaskQueue;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.dfs.Queue;
import org.eso.ohs.dfs.Target;
import org.eso.ohs.dfs.docviewdatatrans.DecAdaptor;
import org.eso.ohs.dfs.docviewdatatrans.EphemerisFileAdaptor;
import org.eso.ohs.dfs.docviewdatatrans.ExecutionTimeAdaptor;
import org.eso.ohs.dfs.docviewdatatrans.FindingChartsAdaptor;
import org.eso.ohs.dfs.docviewdatatrans.ISODateAdaptor;
import org.eso.ohs.dfs.docviewdatatrans.IndirectNameAdaptor;
import org.eso.ohs.dfs.docviewdatatrans.MaskStatusAdaptor;
import org.eso.ohs.dfs.docviewdatatrans.RaAdaptor;
import org.eso.ohs.dfs.docviewdatatrans.STTimeIntervalsAdaptor;
import org.eso.ohs.dfs.docviewdatatrans.StatusAdaptor;
import org.eso.ohs.dfs.docviewdatatrans.TemplateSignatureAdaptor;
import org.eso.ohs.dfs.docviewdatatrans.TimeDurationAdaptor;
import org.eso.ohs.dfs.docviewdatatrans.TimeIntervalsAdaptor;
import org.eso.ohs.instruments.TemplateSignature;

/**
 * Object adaptors for the various OHS docview controls
 * 
 * <p>
 * $Id: OHSAdaptorFactory.java,v 1.2 2005/04/15 19:02:30 tcanavan Exp $
 * @author cmavros 13/Sept/01 Added Execution Time Adaptor OB class
 * <br>ChangeLog: cmavros 17/Oct/01 Added Execution Time Adaptor CB class
 *
 */
public class OHSAdaptorFactory extends AdaptorFactory {

    /** Standard log channel.  */
    protected static Logger stdlog_ =
        Logger.getLogger(OHSAdaptorFactory.class);

    public ObjectAdaptor getAdaptorImpl(Class cls, String property) {
        ObjectAdaptor objAdap = null;

        if (cls.equals(Target.class)) {

            if (property.equals("RA")) {
                objAdap = new RaAdaptor();
            } else if (property.equals("Declination")) {
                objAdap = new DecAdaptor();
            }
        } else if (cls.equals(ObservationBlock.class)) {
            if (property.equals("Status")) {
                objAdap = new StatusAdaptor();
            }
            if (property.equals("TimeIntervals")) {
                objAdap = new TimeIntervalsAdaptor();
            }
            
            if (property.equals("STTimeIntervals")) {
                objAdap = new STTimeIntervalsAdaptor();
            }

            //Added Execution Time
            //cmavros 13/Sept/01
            if (property.equals("ExecutionTime")) {
                objAdap = new ExecutionTimeAdaptor();
            }            
            if (property.equals("FindingCharts")) {
                objAdap = new FindingChartsAdaptor();
            }
            if (property.equals("EphemerisFile")) {
                objAdap = new EphemerisFileAdaptor();
            }
            if (property.equals("TemplateSignature")) {
                objAdap = new TemplateSignatureAdaptor(TemplateSignature.class);
            }
            
        } else if (cls.equals(CalibrationBlock.class)) {
            if (property.equals("Status")) {
                objAdap = new StatusAdaptor();
            }
            //Added Execution Time
            //cmavros 17/Oct/01
            if (property.equals("ExecutionTime")) {
                objAdap = new ExecutionTimeAdaptor();
            }
        } else if (cls.equals(Queue.class) || cls.equals(MaskQueue.class)) {

            if (property.equals("Duration")) {
                objAdap = new TimeDurationAdaptor();
            }
            if (property.equals("StartTime")) {
                objAdap = new ISODateAdaptor();
            }
        }
        if (cls.equals(Mask.class)) {
            if (property.equals("Status")) {
                objAdap = new MaskStatusAdaptor();
            }
        }

        if (objAdap == null) {
            try {
                Method getter = cls.getMethod("get" + property, new Class[] {
                });
                Class returnType = getter.getReturnType();

                if (BusinessObject.class.isAssignableFrom(returnType)) {
                    objAdap = new IndirectNameAdaptor(returnType);
                } else if (property.equals("DbaseId")) {
                    objAdap = new DbaseIdAdaptor();
                } else {
                    objAdap = ObjAdaptor.getAdaptor(cls, property);
                }
            } catch (NoSuchMethodException e) {
                stdlog_.error("class    = " + cls);
                stdlog_.error("property = " + property);
                e.printStackTrace();
            }
        }

        if (objAdap != null) {
            objAdap.setProperty(property);
        }

        return objAdap;
    }

}
