/* $Id: DbaseHandlerReportCard.java,v 1.1 2005/03/30 09:56:11 tcanavan Exp $ */
package org.eso.ohs.persistence.dbase.phase1;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.core.utilities.TextUtils;
import org.eso.ohs.dfs.ReportCard;
import org.eso.ohs.dfs.StorableObject;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.persistence.StorageManager.UnpackedStorableObject;
import org.eso.ohs.persistence.dbase.DbaseIOHandler;
/**
 * Sql handler to retrieve, to get and to update ReportCard objects
 * on the database.  
 * <br>
 *
 * @see ReportCard
 * @author Carlo Boarotto, ESO 21-Mar-2000 
 */
public class DbaseHandlerReportCard extends DbaseIOHandler {
	private final static int MAX_PHASE1_USER = 1000000;
	/**
	 * Managed class
	 * @return ReportCard class
	 */
	public Class getManagedClass() {
		return ReportCard.class;
	}
	public static long runIdFromId(long id) {
		long runId = (long) (id / (MAX_PHASE1_USER));
		return runId;
	}
	public static long userIdFromId(long id) {
		long userId = (long) (id % (MAX_PHASE1_USER));
		return userId;
	}
	public static long idForPhase1Comment(long runId, long userId) {
		return runId * MAX_PHASE1_USER + userId;
	}
	/** Standard log channel */
	private static Logger stdlog_ =
		Logger.getLogger(DbaseHandlerReportCard.class);
	/** A reference to the singleton ReportCardSql instance. */
	private static DbaseHandlerReportCard singleton_ =
		new DbaseHandlerReportCard();
	public static final String RUN_ID = "run_id";
	public static final String USER_ID = "user_id";
	public static final String COMMENT_SHORT = "comment_short";
	public static final String COMMENT_LONG = "comment_long";
	public static final String COMMENT_FLAG = "comment_flag";
	public static final String NIGHTS = "nights";
	public static final String GRADE = "grade";
	public static final String CONFLICTS = "conflicts";
	/** Sql select statement to retrieve ReportCard entries. */
	protected final static String comment_fields =
		Phase1SelectStmt.commentsTable.get()
			+ "."
			+ RUN_ID
			+ ","
			+ Phase1SelectStmt.commentsTable.get()
			+ "."
			+ USER_ID
			+ ","
			+ Phase1SelectStmt.commentsTable.get()
			+ "."
			+ COMMENT_FLAG
			+ ","
			+ Phase1SelectStmt.commentsTable.get()
			+ "."
			+ COMMENT_LONG
			+ ","
			+ Phase1SelectStmt.commentsTable.get()
			+ "."
			+ COMMENT_SHORT
			+ ","
			+ Phase1SelectStmt.ratedTable.get()
			+ "."
			+ NIGHTS
			+ ","
			+ Phase1SelectStmt.ratedTable.get()
			+ "."
			+ GRADE;
	protected final static String comment_fields_only =
		Phase1SelectStmt.commentsTable.get()
			+ "."
			+ RUN_ID
			+ ","
			+ Phase1SelectStmt.commentsTable.get()
			+ "."
			+ USER_ID
			+ ","
			+ Phase1SelectStmt.commentsTable.get()
			+ "."
			+ COMMENT_FLAG
			+ ","
			+ Phase1SelectStmt.commentsTable.get()
			+ "."
			+ COMMENT_LONG
			+ ","
			+ Phase1SelectStmt.commentsTable.get()
			+ "."
			+ COMMENT_SHORT;
	protected final static String rated_fields =
		Phase1SelectStmt.ratedTable.get()
			+ "."
			+ RUN_ID
			+ ","
			+ Phase1SelectStmt.ratedTable.get()
			+ "."
			+ USER_ID
			+ ","
			+ Phase1SelectStmt.ratedTable.get()
			+ "."
			+ NIGHTS
			+ ","
			+ Phase1SelectStmt.ratedTable.get()
			+ "."
			+ GRADE
			+ ","
			+ Phase1SelectStmt.ratedTable.get()
			+ "."
			+ CONFLICTS;
	/** Sql select statement to retrieve ReportCard entries. */
	protected final static String comment_sql =
		"SELECT "
			+ comment_fields
			+ " FROM "
			+ Phase1SelectStmt.commentsTable.get()
			+ ","
			+ Phase1SelectStmt.ratedTable.get()
			+ " ";
	/** Sql select statement to retrieve ReportCard entries. */
	protected final static String rated_sql_ =
		"SELECT "
			+ rated_fields
			+ " FROM "
			+ Phase1SelectStmt.ratedTable.get()
			+ " ";
	/** 
	 * Private constructor: it is a singleton class.
	 */
	public DbaseHandlerReportCard() {
		super();
	}
	/**
	    * Read  opcReferee data
	    * @param sql statement
	    * @param Object id to be read
	    * @exception SQLException
	    *		    an error occurred during the dbase operation
	    * @exception ObjectNotFoundException
	    *		    the requested object was not found in the database   
	    * @exception ObjectIOException IO Eror occured while reading from the database
	    */
	public UnpackedStorableObject read(Statement stmt, long objId)
		throws ObjectIOException, ObjectNotFoundException, SQLException {
		ReportCard report = (ReportCard)newInstance();
		UnpackedStorableObject uso = new UnpackedStorableObject(report);
		String sql =
			rated_sql_ + " WHERE " + splitRunIdUserIdWhereCondition(objId);
		stdlog_.debug(sql);
		ResultSet rs = stmt.executeQuery(sql);
		if (!rs.next()) {
			rs.close();
			throw new ObjectNotFoundException("Cannot  retrieve report card");
		} else {
			setRatedFields(report, rs);
			report.setId(
				Config.getCfg().tableToUniqueId(objId, ReportCard.class));
			rs.close();
			loadReportCardBody(stmt, report);
		}
		return new UnpackedStorableObject(report);
	}

	private String joinRatedComment() {
		String join =
			Phase1SelectStmt.ratedTable.get()
				+ ".run_id = "
				+ Phase1SelectStmt.commentsTable.get()
				+ ".run_id AND "
				+ Phase1SelectStmt.ratedTable.get()
				+ ".user_id = "
				+ Phase1SelectStmt.commentsTable.get()
				+ ".user_id ";
		return join;
	}
	/**
	  * Read  data
	  * @param sql statement
	  * @param Object id to be read
	  * @exception SQLException
	  *		    an error occurred during the dbase operation
	  * @exception ObjectNotFoundException
	  *		    the requested object was not found in the database   
	  * @exception ObjectIOException IO Eror occured while reading from the database
	  */
	public UnpackedStorableObject[] read(
		Statement stmt,
		String whereClause,
		String otherTables)
		throws ObjectIOException, ObjectNotFoundException, SQLException {
		UnpackedStorableObject[] reportCard_array = null;
		ReportCard reportCard = null;
		if (otherTables == null) {
			otherTables = "";
		}
		String sql = rated_sql_ + otherTables + " WHERE " + whereClause;
		stdlog_.info(sql);
		ResultSet rs = stmt.executeQuery(sql);
		try {
			Vector reportCard_vector = new Vector();
			Vector reportIdList = new Vector();
			while (rs.next()) {
				long reportId =
					idForPhase1Comment(rs.getLong(RUN_ID), rs.getLong(USER_ID));
				Long commentIdObj = new Long(reportId);
				// if the id is already selected, skip the next
				if (!reportIdList.contains(commentIdObj)) {
					reportIdList.addElement(commentIdObj);
					reportCard = (ReportCard)newInstance();
					// New element
					reportCard_vector.addElement(reportCard);
					// Set only when attaching the object as a new
					// element
					//SOG
					setRatedFields(reportCard, rs);
					reportCard.setId(
						Config.getCfg().tableToUniqueId(
							reportId,
							ReportCard.class));
				} else {
					stdlog_.warn(
						"The report card "
							+ reportId
							+ " already retrieved. The SQL '"
							+ sql
							+ "' returns duplicate rows.");
				}
			}
			rs.close();
			for (int i = 0; i < reportCard_vector.size(); i++) {
				ReportCard report = (ReportCard)reportCard_vector.elementAt(i);
				loadReportCardBody(stmt, report);
			}
			reportCard_array =
				new UnpackedStorableObject[reportCard_vector.size()];
			// add the affiliation to the user and insert them in array
			for (int i = 0; i < reportCard_vector.size(); i++) {
				reportCard = (ReportCard)reportCard_vector.elementAt(i);
				reportCard_array[i] = new UnpackedStorableObject(reportCard);
			}
		} catch (SQLException e) {
			stdlog_.error("Error retrieving card list: ", e);
		}
		return reportCard_array;
	}
	private void loadReportCardBody(Statement stmt, ReportCard report)
		throws SQLException, ObjectIOException {
		ResultSet rs;

		long id = Config.getCfg().uniqueToTableId(report.getId());
		String comment_sql =
			"SELECT "
				+ COMMENT_SHORT
				+ ","
				+ COMMENT_LONG
				+ "  FROM "
				+ Phase1SelectStmt.commentsTable.get()
				+ " WHERE "
				+ splitRunIdUserIdWhereCondition(id)
				+ " AND comment_flag = 0";
		stdlog_.info(comment_sql);
		rs = stmt.executeQuery(comment_sql);
		if (!rs.next()) {
			rs.close();
			insertCommentRow(stmt, report.getId(), "");
			// create comment row
		} else {
			setCommentFieldsOnly(report, rs);
			rs.close();
		}
	}
	/**
	 * 
	 * The assumption is that a rated row is already present.
	 * 
	 * @param stmt  JDBC Statement for contacting the dbase
	 * @param id    per-table id of the object to be written.
	 * @param obj   an array: the first element is the BO to be stored.
	 * 
	 * @exception ObjectIOException	operation is not permitted exception
	 * 
	 * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
	 * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
	 */
	public void write(Statement stmt, long id, UnpackedStorableObject obj)
		throws ObjectIOException {
		ReportCard reportCard = (ReportCard)obj.getObject();
		insertCommentRow(stmt, id, reportCard.getBody());
	}
	private void insertCommentRow(
		Statement stmt,
		long id,
		String reportCardBody)
		throws ObjectIOException {
		StringBuffer sb = new StringBuffer();
		long objId = Config.getCfg().uniqueToTableId(id);
		sb.append(runIdFromId(objId));
		sb.append(",");
		sb.append(userIdFromId(objId));
		sb.append(",0,\'");
		sb.append(TextUtils.quoteSqlString(reportCardBody));
		sb.append("\', NULL ");
		String sqlText =
			Phase1SelectStmt.beginTransaction
				+ " \n "
				+ "INSERT INTO "
				+ Phase1SelectStmt.commentsTable.get()
				+ " ("
				+ comment_fields_only
				+ " ) "
				+ " VALUES ("
				+ sb.toString()
				+ " ) ";
		stdlog_.debug(sqlText);
		try {
			stmt.executeUpdate(sqlText);
		} catch (SQLException e) {
			stdlog_.error("update error", e);
			throw new ObjectIOException("Error executing write: " + sqlText);
		}
	}
	// Update both rated and comments tables.
	public void update(Statement stmt, StorableObject obj)
		throws ObjectIOException, SQLException {
		ReportCard reportCard = (ReportCard)obj;

		long id = Config.getCfg().uniqueToTableId(reportCard.getId());

		StringBuffer sb = new StringBuffer();
		sb.append(COMMENT_LONG);
		sb.append(" = \'");
		sb.append(TextUtils.quoteSqlString(reportCard.getBody()));
		sb.append("\',");
		sb.append(COMMENT_SHORT);
		sb.append(" = NULL ");
		String sql =
			Phase1SelectStmt.beginTransaction
				+ " \n "
				+ "UPDATE  "
				+ Phase1SelectStmt.commentsTable.get()
				+ " SET "
				+ sb.toString()
				+ " WHERE "
				+ splitRunIdUserIdWhereCondition(id)
				+ " \n"
				+ "UPDATE "
				+ Phase1SelectStmt.ratedTable.get()
				+ " SET "
				+ NIGHTS
				+ " = "
				+ reportCard.getNights()
				+ ","
				+ GRADE
				+ " = "
				+ reportCard.getGrade()
				+ ","
				+ CONFLICTS
				+ " = "
				+ reportCard.getIntConflicts()
				+ " WHERE "
				+ splitRunIdUserIdWhereCondition(id);
		stdlog_.debug(sql);
		stmt.executeUpdate(sql);
	}
	/**
	 * Throws an ObjectIOException -- the phase 1 database cannot be
	 * changed from within P2PP.
	 * 
	 * @param stmt  JDBC Statement for contacting the dbase
	 * @param id    per-table id of the object to be written.
	 * @param obj   an array: the first element is the BO to be stored.
	 * 
	 * @exception ObjectIOException	operation is not permitted exception
	 * 
	 * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
	 * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
	 */
	public void update(Statement stmt, long id, UnpackedStorableObject obj)
		throws ObjectIOException {
		ReportCard reportCard = (ReportCard)obj.getObject();
		StringBuffer sb = new StringBuffer();
		sb.append(COMMENT_LONG);
		sb.append(" = \'");
		sb.append(TextUtils.quoteSqlString(reportCard.getBody()));
		sb.append("\',");
		sb.append(COMMENT_SHORT);
		sb.append(" = NULL ");
		String sqlText =
			Phase1SelectStmt.beginTransaction
				+ " \n "
				+ "UPDATE  "
				+ Phase1SelectStmt.commentsTable.get()
				+ " SET "
				+ sb.toString()
				+ " WHERE "
				+ splitObjIdWhereCondition(id)
				+ " \n"
				+ "UPDATE "
				+ Phase1SelectStmt.ratedTable.get()
				+ " SET "
				+ NIGHTS
				+ " = "
				+ reportCard.getNights()
				+ ","
				+ GRADE
				+ " = "
				+ reportCard.getGrade()
				+ " WHERE "
				+ RUN_ID
				+ " = "
				+ runIdFromId(id)
				+ " AND    "
				+ USER_ID
				+ "  = "
				+ userIdFromId(id);
		stdlog_.debug(sqlText);
		try {
			stmt.executeUpdate(sqlText);
		} catch (SQLException e) {
			stdlog_.error("update error", e);
			throw new ObjectIOException("Error executing write: " + sqlText);
		}
	}
	/**
	 * Throws an ObjectIOException -- the phase 1 database cannot be
	 * changed from within P2PP.
	 * 
	 * @param stmt  JDBC Statement for contacting the dbase
	 * @param id    per-table id of the object to be written.
	 * 
	 * @exception ObjectIOException	operation is not permitted exception
	 */
	public void delete(Statement stmt, long id) throws ObjectIOException {
		String sqlText =
			" \n DELETE  FROM "
				+ Phase1SelectStmt.commentsTable.get()
				+ " WHERE "
				+ splitObjIdWhereCondition(id);
		stdlog_.debug(sqlText);
		try {
			stmt.executeUpdate(sqlText);
		} catch (SQLException e) {
			stdlog_.error("update error", e);
			throw new ObjectIOException("Error executing write: " + sqlText);
		}
	}
	private String splitObjIdWhereCondition(long id) {
		return Phase1SelectStmt.ratedTable.get()
			+ "."
			+ RUN_ID
			+ " = "
			+ runIdFromId(id)
			+ " AND    "
			+ Phase1SelectStmt.ratedTable.get()
			+ "."
			+ USER_ID
			+ "  = "
			+ userIdFromId(id)
			+ " AND    "
			+ Phase1SelectStmt.commentsTable.get()
			+ "."
			+ COMMENT_FLAG
			+ "  = 0 ";
	}
	private String splitRunIdUserIdWhereCondition(long id) {
		return RUN_ID
			+ " = "
			+ runIdFromId(id)
			+ " AND    "
			+ USER_ID
			+ "  = "
			+ userIdFromId(id);
	}
	/**
	 * Set the user fields from the result set.
	 * Only for avoiding duplicated code
	 *
	 */
	protected void setCommentFields(ReportCard reportCard, ResultSet rs)
		throws SQLException {
		reportCard.setRunId(rs.getLong(RUN_ID));
		reportCard.setUserId(rs.getLong(USER_ID));
		reportCard.setNights(rs.getInt(NIGHTS));
		reportCard.setGrade(rs.getInt(GRADE));
		String commentBody = rs.getString(COMMENT_SHORT);
		if (commentBody == null) {
			commentBody = rs.getString(COMMENT_LONG);
		}
		reportCard.setBody(commentBody);
	}
	/**
		 * Set the user fields from the result set.
		 * Only for avoiding duplicated code
		 *
		 */
	protected void setCommentFieldsOnly(ReportCard reportCard, ResultSet rs)
		throws SQLException {
		String commentBody = rs.getString(COMMENT_SHORT);
		if (commentBody == null) {
			commentBody = rs.getString(COMMENT_LONG);
		}
		reportCard.setBody(commentBody);
	}
	/**
		 * Set the user fields from the result set.
		 * Only for avoiding duplicated code
		 *
		 */
	protected void setRatedFields(ReportCard reportCard, ResultSet rs)
		throws SQLException {
		reportCard.setRunId(rs.getLong(RUN_ID));
		reportCard.setUserId(rs.getLong(USER_ID));
		reportCard.setNights(rs.getInt(NIGHTS));
		reportCard.setGrade(rs.getInt(GRADE));
		reportCard.setConflicts(rs.getInt(CONFLICTS));
	}
}
