package org.eso.ohs.phase2.actions;
import java.awt.Frame;
import java.awt.event.ActionEvent;

import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JOptionPane;

import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.baseaction.IncrementalActionAutomaton;
import org.eso.ohs.core.utilities.ObjUtils;
import org.eso.ohs.persistence.ObjectContainer;
import org.eso.ohs.phase2.orang.OBScheduleInfo;
import org.eso.ohs.phase2.visiplot.ChartColorItem;
import org.eso.ohs.phase2.visiplot.ChartFrame;
import org.eso.ohs.phase2.visiplot.VPlotOptionsDialog;
import org.eso.ohs.phase2.visiplot.VisiPlotOptions;

/**
 * Action class to execute the visiplots
 * It might be called from a normal queue view, or from an orang view
 *
 * @version $Id: OrangVisiplotAction.java,v 1.4 2005/12/01 15:52:17 pnunes Exp $
 * @author Nick Kornweibel
*/

public class OrangVisiplotAction extends ActionSuperclass {
	
	
	private ObjectContainer folderView_;
	private boolean folder_;	
	
	//static ChartFrame chartFrame_ = null;
	protected String sOs_ = System.getProperty("os.name");
	private VisiPlotOptions currentSettings_ = null; 
	private OBScheduleInfo [] obs_ = null;
	private ChartColorItem[] chartColorItems_;
	
	
	public OrangVisiplotAction(ObjectContainer folderView, boolean folder,
			String name, VisiPlotOptions setting, OBScheduleInfo[] obs) {
		super((JComponent) folderView, name);
		folderView_ = folderView;
		folder_ = folder;
		currentSettings_ = setting;
		obs_ = obs;
	}
		
	
	/**
	 * Event handler for the Visiplot action.
	 * 
	 * @param ev  user-initiated ActionEvent to quit.
	 */
	public void actionPerformedImpl(ActionEvent evt) {
		Long[] idLongs = null;
		if (!folder_) {
			idLongs = folderView_.getAllSelected();
		} else {
			idLongs = folderView_.getAll();
		}
				
		idLongs = (Long[]) ObjUtils.uniqueOccurrences(idLongs);
		
		// checks if the any OBs are selected
		if (idLongs.length <= 0) {
			JOptionPane
			.showMessageDialog(
					folderView_.getTopLevelAncestor(),
					new Object[] {
						"There are no selected business objects",
						" ",
					"You must first select business objects to report on." },
					"Nothing selected", JOptionPane.ERROR_MESSAGE);			
		} else {
			
			// if there are selected OBs			
			// simply plots the graphs

			
			if (showDialog(true)) {
				new IncrementalActionAutomaton(
						new IncrementalOBVisiplot(idLongs,currentSettings_,obs_,new ChartFrame(folderView_,currentSettings_,chartColorItems_,null) ), 
						(JFrame) folderView_.getTopLevelAncestor(), "Calculating ...");
			}		
			
		}
	}
	
	private boolean showDialog(boolean modal) {
        return showDialog((JFrame) folderView_.getTopLevelAncestor(), modal);
    }

    private boolean showDialog(Frame owner, boolean modal) {
        VPlotOptionsDialog dlg = new VPlotOptionsDialog(owner, modal,true);        
        dlg.showDialog(currentSettings_);
        chartColorItems_ = dlg.getChartColorItems();
        return dlg.getSettingsAccepted();
    }
	
	
	
}

