package org.eso.ohs.phase2.apps.ot.icdVcs;

import org.eso.ohs.core.gui.widgets.ErrorMessages;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.instruments.TemplateSignature;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.ObjectManager;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.persistence.dbase.phase2.MaskDBIO;
import org.eso.ohs.persistence.dbase.phase2.QueueManager;
import org.eso.ohs.phase2.icdVcs.IcdVcsCfg;

/**
 * This class process all the Events from BOB that our received from the OT
 * 
 * @author tcanavan
 *
 */
public class ProcessOBEvents implements Runnable {

    private String BOBEventType;

    private long obId;

    public ProcessOBEvents(long obId, String BOBEventType) {
        this.BOBEventType = BOBEventType;
        this.obId = obId;
    }

    /**
     * 
     * @see java.lang.Runnable#run()
     */
    public void run() {
        ObjectManager objMgr = ObjectManager.getObjectManager();
        IcdVcsCfg cfg = IcdVcsCfg.getCfg();
        CalibrationBlock cb = null;
        try {
            cb =
                (CalibrationBlock) objMgr.getBusObj(
                    Media.DBASE,
                    Config.getCfg().tableToUniqueId(
                        obId,
                        ObservationBlock.class),
                    ObservationBlock.class);
        } catch (ObjectNotFoundException e) {
            try {
                cb =
                    (CalibrationBlock) objMgr.getBusObj(
                        Media.DBASE,
                        Config.getCfg().tableToUniqueId(
                            obId,
                            CalibrationBlock.class),
                        CalibrationBlock.class);
            } catch (ObjectNotFoundException e1) {
                e1.printStackTrace();
                ErrorMessages.announceNoObject(null, e1);
            } catch (ObjectIOException e2) {
                e2.printStackTrace();
                ErrorMessages.announceIOError(null, e2);
            }
        } catch (ObjectIOException e3) {
            e3.printStackTrace();
            ErrorMessages.announceIOError(null, e3);
        }
        String status = CalibrationBlock.convertBobEventToStatus(BOBEventType);
        cb.setStatus(status);
        objMgr.queueObject(cb);
        final long obId_ = obId;
        final String status_ = status;
        Class objTemp = CalibrationBlock.class;
        if (cb instanceof ObservationBlock) {
            objTemp = ObservationBlock.class;
        }
        final Class objType = objTemp;

        try {
            if (status.equals(CalibrationBlock.EXECUTED)) {
                long [] ids = new long[1];
                ids[0] = cb.getId();
                QueueManager.deleteOBFromOTExecSequence(ids);
                if (objType.equals(ObservationBlock.class)) {
                    TemplateSignature ts =
                        ((ObservationBlock) cb).getTemplateSignature();
                    if (ts.getInstrumentName().equals("VIMOS"))
                        MaskDBIO.incExecCount(ts);
                }
            }
            QueueManager.updateOBStatus(cb.getId(), status);

        } catch (ObjectIOException e) {
            ErrorMessages.announceIOError(null, e);
        }
    }

}
