package org.eso.ohs.phase2.apps.p2ppAppServer;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.dfs.ConstraintSet;
import org.eso.ohs.dfs.Mask;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.dfs.ObservationDescription;
import org.eso.ohs.dfs.ObservingRun;
import org.eso.ohs.dfs.Target;
import org.eso.ohs.p2pp.Folder;
import org.eso.ohs.persistence.AppConfig;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.DirectoryNode;
import org.eso.ohs.persistence.Media;
/**
 * Clearing house for low-level configuration information.
 * 
 * @author cmavros 2002/02/11, 1512/P2PP
 * <p>
 * $Id: MyPersistCfg.java,v 1.1 2005/04/25 16:40:48 tcanavan Exp $
 */
//TODO should inherit from apps.DefaultPersistConfig
public class MyPersistCfg extends Config {

    private static Logger stdlog_ = Logger.getLogger(MyPersistCfg.class);

    public String[] folderColumnProperties(Class objType) {
        return AppConfig.getAppConfig().folderColumnProperties(objType);
    }

    public String getDbUserName() {
        return AppConfig.getAppConfig().getDbUserName();
    }

    public String getDbPassword() {
        return AppConfig.getAppConfig().getDbPassword();
    }

    public String getUrl(String key) {
        return AppConfig.getAppConfig().getUrl(key);
    }

    public String getDbName(String key) {
        return AppConfig.getAppConfig().getDbName(key);
    }

    public boolean isOperationsModeId(int uid) {
        return AppConfig.getAppConfig().isOperationsModeId(uid);
    }

    public boolean isEngineeringModeId(int uid) {
        return AppConfig.getAppConfig().isEngineeringModeId(uid);
    }

    public boolean isVisitorModeId(int uid) {
        return AppConfig.getAppConfig().isVisitorModeId(uid);
    }

    public boolean isStandardModeId(int uid) {
        return AppConfig.getAppConfig().isStandardModeId(uid);
    }

    public boolean isCloneModeId() {
        return false;
    }

    /**
     * What is the first period for which IP versioning info
     * is expected in the data. 0 indicates no start period, no
     * version info is expected.
     */
    public int getStartPeriodForIPVersioning() {
        int rc = 0;
        if (AppConfig.getAppConfig().isIPVersioningEnabled()) {
            rc = AppConfig.getAppConfig().getIPVersionStartPeriod();
        }
        return rc;
    }

    /**
     * Ask for authentification before action can take place
     * @param device Media referreed to DBASE,LOCAL_STORAGE
     * @exception ObjectIOException Object I/O Excpetion
     */

    public void askForAuthentication(Media device) throws ObjectIOException {
        throw new IllegalArgumentException("Not Needed for AppServer");
    }

    /*
     * @return true if the given instrument is in the list of
     * instruments for which VM OBs may be checked into the repository.
     * See AppConfig.getInstrumentsWithVMCheckInOk() for details.
     */
    public boolean isVMOBCheckinAllowed(String inst) {
        return AppConfig.getAppConfig().canCheckInVMOBs(inst);
    }



    public boolean isUserLevel(Class cls) {
        return cls != null && cls != Folder.class && cls != DirectoryNode.class;
    }

    public void initPersistenceObjInfo() {
        ObjConfigPersistence ocp;

        ocp =
            new ObjConfigPersistence(
                ConstraintSet.class,
                new String[] { // Dbase columns for Summary info
            "cs_id",
                "item_name",
                "airmass",
                "moon_distance",
                "seeing" },
                "constraint_sets",
                "cs",
                4);
        objCfgPersistence.put(ocp.key, ocp);

        ocp =
            new ObjConfigPersistence(
                Target.class,
                new String[] { // Dbase columns for Summary info
            "tp_id",
                "item_name",
                "ra",
                "dec",
                "equinox",
                "epoch" },
                "target_packages",
                "tp",
                6);
        objCfgPersistence.put(ocp.key, ocp);

        ocp =
            new ObjConfigPersistence(
                ObservationBlock.class,
                new String[] { // Dbase columns for Summary info
            "obs_blocks.ob_id",
                "obs_blocks.item_name",
                "status",
                "target_packages.item_name",
                "obs_descriptions.item_name" },
                "obs_blocks, target_packages, obs_descriptions",
                "ob",
                3);
        objCfgPersistence.put(ocp.key, ocp);

        ocp =
            new ObjConfigPersistence(
                CalibrationBlock.class,
                new String[] { // Dbase columns for Summary info
            "obs_blocks.ob_id",
                "obs_blocks.item_name",
                "status",
                "obs_descriptions.item_name" },
                "obs_blocks, obs_descriptions",
                "cb",
                7);
        objCfgPersistence.put(ocp.key, ocp);

        ocp =
            new ObjConfigPersistence(
                ObservationDescription.class,
                new String[] { // Dbase columns for Summary info
            "od_id", "item_name", "instrument" }, "obs_descriptions", "od", 2);
        objCfgPersistence.put(ocp.key, ocp);

        ocp =
            new ObjConfigPersistence(
                ObservingRun.class,
                new String[] { // Dbase columns for Summary info
            // "period" is required by MainWindow.Phase1Information!
            "obs_runs.id",
                "obs_programmes.period",
                "obs_programmes.title" },
                "obs_runs, sched_rep, obs_programmes, eso_users, proposed",
                "or",
                5);
        objCfgPersistence.put(ocp.key, ocp);

        ocp =
            new ObjConfigPersistence(
                Folder.class,
                new String[] { // Dbase columns for Summary info
        }, "", "sum", 1);
        objCfgPersistence.put(ocp.key, ocp);

        ocp =
            new ObjConfigPersistence(
                DirectoryNode.class,
                new String[] { // Dbase columns for Subdir info
        }, "", "dir", 0);
        objCfgPersistence.put(ocp.key, ocp);

        ocp =
            new ObjConfigPersistence(
                Mask.class,
                new String[] { // Dbase columns for Summary info
            "masks.ob_id",
                "masks.ts_index",
                "masks.param_index",
                "masks.mask_naid",
                "masks.mask_id",
                "masks.mask_slot_num" },
                "masks",
                "msk",
                8);
        objCfgPersistence.put(ocp.key, ocp);
        
        

    }


}
