package org.eso.ohs.phase2.icdVcs;

import org.eso.vlt.base.Ccs.CcsException;
import org.eso.vlt.base.Ccs.CcsObject;

/**
 * OHS side of the interface to the VLT groups "BOB" program.
 * BOB stands for something like "Broker of Observation Blocks".
 * 
 * <p>
 * Note that this class actually interfaces with BOB using code from
 * the VLT group provided in package org.eso.vlt.base.Ccs.  We do not
 * communicate directly with BOB.
 * 
 * @version $Revision: 1.3 $
 * @author Bob Kemp
 */
public class BOB {

    private static final String rcsid = "$Id: BOB.java,v 1.3 2003/07/03 08:49:49 ddorigo Exp $";

    protected ScriptCmdBroadcaster scriptBroadcaster_;
    protected EventCmdBroadcaster eventBroadcaster_;

    protected static BOB instance_ = null;


    public static BOB getInstance() throws CcsException {
	return instance_;
    }


    /**
     * Default constructor.
     * 
     * @exception CcsException
     *			if Ccs initialisation failed.
     */
    public BOB() throws CcsException {
	initialiseCcs();

	createBroadcasters();
	createListeners();

	instance_ = this;
    }


    protected void initialiseCcs() throws CcsException {
	String serverName = IcdVcsCfg.getCfg().getCCSServer();

	// CcsDebug.setLevel(5);	// ******* DEBUGGING ******* 
					// Do not delete, just comment out!!

	CcsObject.setAppName(serverName);
    }

    protected void createBroadcasters() throws CcsException {
	scriptBroadcaster_ = new ScriptCmdBroadcaster();
	eventBroadcaster_  = new EventCmdBroadcaster();
    }

    protected void createListeners() throws CcsException {

	// @author amchavan, 05-Apr-2002: commented out
	// Log log = new Log("OB Execution log");
	ScriptCmds scriptCmds = new ScriptCmds();
	EventCmds eventCmds  = new EventCmds(scriptCmds);

	scriptBroadcaster_.addScriptCmdListener(scriptCmds);
	eventBroadcaster_.addEventCmdListener(eventCmds);
    }

    public ScriptCmdBroadcaster getScriptBroadcaster() {
	return scriptBroadcaster_;
    }

    public EventCmdBroadcaster getEventBroadcaster() {
	return eventBroadcaster_;
    }



}


