package org.eso.ohs.phase2.icdVcs;

import javax.swing.event.EventListenerList;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.TextUtils;
import org.eso.vlt.base.Ccs.CcsCommandMonitor;
import org.eso.vlt.base.Ccs.CcsException;
import org.eso.vlt.base.Ccs.CcsSeqListener;

/**
 * Listener for script events from the BOB interface
 *
 * <p>
 * <b>WARNING:</b>
 * This cannot be merged with the EventCmdBroadcaster currently,
 * due to limitations in the implementeation of the Java/Ccs library.
 * If you do merge them, one or the other will stop working due to a
 * Hashtable in the superclass.
 * 
 * <p>
 * NOTE: This class must be public in order for Java/Ccs to work.
 *       I'm not sure of the reason but it's probably to do with
 *       their use of the Reflection API.
 * 
 * <p>
 * $Id: ScriptCmdBroadcaster.java,v 1.5 2004/04/08 12:09:53 tcanavan Exp $
 * <br>
 * @author Bob Kemp
 */
public class ScriptCmdBroadcaster extends CcsSeqListener {

   /** Standard log channel */
   private static Logger stdlog_ =
       Logger.getLogger( ScriptCmdBroadcaster.class );

    /** Support for handling listeners */
    protected EventListenerList listenerList_ = new EventListenerList();;


    /**
     * Default constructor.
     * @exception CcsException CCS throws an exception
     */
    public ScriptCmdBroadcaster() throws CcsException {
	initialise();
    }

    /**
     * Initialisation method.  We keep this as a separate method, so
     * that the test routines can override it and avoid starting up CCS.
     * 
     * @exception CcsException CCS throws an exception
     */
    protected void initialise() 
			    throws CcsException {
	CcsCommandMonitor cmdMonitor = new CcsCommandMonitor("SCRIPT");
	stdlog_.debug("CommandMonitor created for SCRIPT");

	cmdMonitor.addCommandListener(this);
	stdlog_.debug("cmd listener added");
    }

    /**
     * Handler method for NextObsBlocks requests from BOB.
     * 
     * @param msg	Status message as a tcl array
     * 
     * @return		Observation block in OBD format 
     *		    	or null string if an error occurred.
     */
    public String NextObsBlocks(String msg) {
//	stdlog_.debug(">>> NextObsBlocks( \"" + msg + "\" )" );

	try {
	    Object[] listeners = listenerList_.getListenerList();
	    int numOBs;
	    Boolean isOffline = null;

	    String[] args = TextUtils.splitTclArray(msg);

// 	    for( int i = 0; i < args.length; i++ )
// 		stdlog_.debug( ">>>> args[" + i + "] = " + args[i] );

	    try {
		numOBs = Integer.parseInt(args[0]);
	    }
	    catch (NumberFormatException e) {
		numOBs = ScriptCmdListener.ALL_AVAILABLE;
	    }

	    if (args.length > 1) {
		isOffline = Boolean.valueOf(args[1]);
	    }

	    for (int i = listeners.length-2; i>=0; i-=2) {
		if (listeners[i] == ScriptCmdListener.class) {
		    ScriptCmdListener lnr = (ScriptCmdListener)listeners[i+1];
		    String result = lnr.NextObsBlocks(numOBs, isOffline);

		    if (result != null && result.length() > 0) {
			return result;
		    }
		}
	    }
	}
	catch (Throwable t) {
	    t.printStackTrace();
	}

	return "";
    }

    public void addScriptCmdListener(ScriptCmdListener lnr) {
	listenerList_.add(ScriptCmdListener.class, lnr);
    }

    public void removeScriptCmdListener(ScriptCmdListener lnr) {
	listenerList_.remove(ScriptCmdListener.class, lnr);
    }

}


