package org.eso.ohs.phase2.visibility;

/**
 * SiderealTime class converts Universal Time to 
 * Greenwich mean sidereal time.
 *
 * The following SLALIB routines are called:
 *
 * slaGmsta
 *
 * @version 25 July 2001
 * @author C. Mavros for ESO
 */


public class SiderealTime extends JNIWrapper{

    /**
    * getST calls the SLALIB function
    * that converts Universal time to 
    * Greenwich Mean Sidereal Time (radians, 
    * in the range 0 to 2pi)
    *
    * @param UT1 integer part of JD-2400000.5
    * @param ut fraction of a day
    * @return st Sideral Time
    */
    
    public static double getST(double jDate){

        double UT1 = jDate-2400000.5; 
        double ut = UT1 - Math.floor(jDate);
        double st = jnislaGmsta(UT1-ut, ut);

        return(st);
    }

}
